@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -232,12 +232,12 @@ class CEvent extends CZBXAPI {
 
 		// object
 		if (!is_null($options['object'])) {
-			$sqlParts['where']['o'] = 'e.object='.$options['object'];
+			$sqlParts['where']['o'] = 'e.object='.zbx_dbstr($options['object']);
 		}
 
 		// source
 		if (!is_null($options['source'])) {
-			$sqlParts['where'][] = 'e.source='.$options['source'];
+			$sqlParts['where'][] = 'e.source='.zbx_dbstr($options['source']);
 		}
 
 		// acknowledged
@@ -255,22 +255,22 @@ class CEvent extends CZBXAPI {
 
 		// time_from
 		if (!is_null($options['time_from'])) {
-			$sqlParts['where'][] = 'e.clock>='.$options['time_from'];
+			$sqlParts['where'][] = 'e.clock>='.zbx_dbstr($options['time_from']);
 		}
 
 		// time_till
 		if (!is_null($options['time_till'])) {
-			$sqlParts['where'][] = 'e.clock<='.$options['time_till'];
+			$sqlParts['where'][] = 'e.clock<='.zbx_dbstr($options['time_till']);
 		}
 
 		// eventid_from
 		if (!is_null($options['eventid_from'])) {
-			$sqlParts['where'][] = 'e.eventid>='.$options['eventid_from'];
+			$sqlParts['where'][] = 'e.eventid>='.zbx_dbstr($options['eventid_from']);
 		}
 
 		// eventid_till
 		if (!is_null($options['eventid_till'])) {
-			$sqlParts['where'][] = 'e.eventid<='.$options['eventid_till'];
+			$sqlParts['where'][] = 'e.eventid<='.zbx_dbstr($options['eventid_till']);
 		}
 
 		// value
