@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -101,9 +101,7 @@ class CProxy extends CZBXAPI {
 		}
 
 		// editable + PERMISSION CHECK
-		if (USER_TYPE_SUPER_ADMIN == $userType || $options['nopermissions']) {
-		}
-		else {
+		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ_ONLY;
 			if ($permission == PERM_READ_WRITE) {
 				return array();
@@ -116,12 +114,12 @@ class CProxy extends CZBXAPI {
 		// proxyids
 		if (!is_null($options['proxyids'])) {
 			zbx_value2array($options['proxyids']);
-			$sqlParts['where'][] = DBcondition('h.hostid', $options['proxyids']);
+			$sqlParts['where'][] = dbConditionInt('h.hostid', $options['proxyids']);
 		}
 
 		// filter
 		if (is_array($options['filter'])) {
-			zbx_db_filter('hosts h', $options, $sqlParts);
+			$this->dbFilter('hosts h', $options, $sqlParts);
 		}
 
 		// search
@@ -176,9 +174,9 @@ class CProxy extends CZBXAPI {
 		}
 		$sqlLimit = $sqlParts['limit'];
 
-		$sql = 'SELECT '.zbx_db_distinct($sqlParts).' '.$sqlSelect.'
-				FROM '.$sqlFrom.'
-				WHERE '.DBin_node('h.hostid', $nodeids).
+		$sql = 'SELECT '.zbx_db_distinct($sqlParts).' '.$sqlSelect.
+				' FROM '.$sqlFrom.
+				' WHERE '.DBin_node('h.hostid', $nodeids).
 					$sqlWhere.
 					$sqlOrder;
 		$res = DBselect($sql, $sqlLimit);
@@ -492,7 +490,7 @@ class CProxy extends CZBXAPI {
 		$dbProxies = DBselect(
 			'SELECT h.hostid,h.host'.
 					' FROM hosts h'.
-					' WHERE '.DBcondition('h.hostid', $proxyIds));
+					' WHERE '.dbConditionInt('h.hostid', $proxyIds));
 		$dbProxies = DBfetchArrayAssoc($dbProxies, 'hostid');
 
 		$actionids = array();
@@ -501,7 +499,7 @@ class CProxy extends CZBXAPI {
 			'SELECT DISTINCT c.actionid'.
 			' FROM conditions c'.
 			' WHERE c.conditiontype='.CONDITION_TYPE_PROXY.
-				' AND '.DBcondition('c.value', $proxyIds)
+				' AND '.dbConditionString('c.value', $proxyIds)
 		);
 		while ($dbAction = DBfetch($dbActions)) {
 			$actionids[$dbAction['actionid']] = $dbAction['actionid'];
@@ -620,7 +618,7 @@ class CProxy extends CZBXAPI {
 		$dRule = DBfetch(DBselect(
 			'SELECT dr.druleid,dr.name,dr.proxy_hostid'.
 					' FROM drules dr'.
-					' WHERE '.DBcondition('dr.proxy_hostid', zbx_objectValues($proxies, 'proxyid')), 1));
+					' WHERE '.dbConditionInt('dr.proxy_hostid', zbx_objectValues($proxies, 'proxyid')), 1));
 		if ($dRule) {
 			$proxy = DBfetch(DBselect('SELECT h.host FROM hosts h WHERE h.hostid='.$dRule['proxy_hostid']));
 
@@ -638,7 +636,7 @@ class CProxy extends CZBXAPI {
 		$host = DBfetch(DBselect(
 			'SELECT h.name,h.proxy_hostid'.
 					' FROM hosts h'.
-					' WHERE '.DBcondition('h.proxy_hostid', zbx_objectValues($proxies, 'proxyid')), 1));
+					' WHERE '.dbConditionInt('h.proxy_hostid', zbx_objectValues($proxies, 'proxyid')), 1));
 		if ($host) {
 			$proxy = DBfetch(DBselect('SELECT h.host FROM hosts h WHERE h.hostid='.$host['proxy_hostid']));
 			self::exception(ZBX_API_ERROR_PARAMETERS,
