@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -1021,7 +1021,7 @@ class CTemplate extends CHostGeneral {
 
 			foreach ($template['groups'] as $group) {
 				$hostgroupid = get_dbid('hosts_groups', 'hostgroupid');
-				$result = DBexecute("INSERT INTO hosts_groups (hostgroupid, hostid, groupid) VALUES ($hostgroupid, $templateid, {$group['groupid']})");
+				$result = DBexecute('INSERT INTO hosts_groups (hostgroupid,hostid,groupid) VALUES ('.zbx_dbstr($hostgroupid).','.zbx_dbstr($templateid).','.zbx_dbstr($group['groupid']).')');
 				if (!$result) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, 'DBerror');
 				}
@@ -1163,7 +1163,7 @@ class CTemplate extends CHostGeneral {
 		$actionids = array();
 		$sql = 'SELECT DISTINCT actionid'.
 			' FROM conditions'.
-			' WHERE conditiontype='.CONDITION_TYPE_HOST.
+			' WHERE conditiontype='.CONDITION_TYPE_HOST_TEMPLATE.
 			' AND '.dbConditionString('value', $templateids);
 		$dbActions = DBselect($sql);
 		while ($dbAction = DBfetch($dbActions)) {
@@ -1189,8 +1189,8 @@ class CTemplate extends CHostGeneral {
 
 		// delete action conditions
 		DB::delete('conditions', array(
-			'conditiontype'=>CONDITION_TYPE_HOST,
-			'value'=>$templateids
+			'conditiontype' => CONDITION_TYPE_HOST_TEMPLATE,
+			'value' => $templateids
 		));
 
 		// delete action operation commands
