@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -149,7 +149,7 @@ require_once dirname(__FILE__).'/include/page_header.php';
 		$p = $timetill - $timesince;				// graph size in time
 		$z = $p - ($timesince % $p);				// graphsize - mod(from_time,p) for Oracle...
 		$x = round($p / $scaleperiod);				// graph size in px
-		$calc_field = 'round('.$x.'*'.zbx_sql_mod(zbx_dbcast_2bigint('clock').'+'.$z, $p).'/('.$p.'),0)';	// required for 'group by' support of Oracle
+		$calc_field = 'floor('.$x.'*'.zbx_sql_mod(zbx_dbcast_2bigint('clock').'+'.$z, $p).'/('.$p.'))';	// required for 'group by' support of Oracle
 
 		$period_step = $scaleperiod;
 
@@ -408,8 +408,8 @@ require_once dirname(__FILE__).'/include/page_header.php';
 			$sql = 'SELECT DISTINCT hg.hostid'.
 				' FROM hosts_groups hg,hosts h'.
 				' WHERE h.hostid=hg.hostid'.
-					' AND'.DBcondition('h.status', array(HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED)).
-					' AND'.DBcondition('hg.groupid', $groupids);
+					' AND '.dbConditionInt('h.status', array(HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED)).
+					' AND '.dbConditionInt('hg.groupid', $groupids);
 			$res = DBselect($sql);
 			while($db_host = DBfetch($res)){
 				$hostids[$db_host['hostid']] = $db_host['hostid'];
@@ -642,6 +642,7 @@ require_once dirname(__FILE__).'/include/page_header.php';
 	$graph->setPeriodCaption($graph_data['captions']);
 
 	$graph->setHeader($title);
+	$graph->setPeriod($scaleperiod);
 	$graph->setXLabel($xlabel);
 	$graph->setYLabel($ylabel);
 
