@@ -1,7 +1,7 @@
 <?php
 /*
-** ZABBIX
-** Copyright (C) 2000-2009 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,33 +10,31 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
-?>
-<?php
-class CButtonCancel extends CButton{
-	public function __construct($vars=NULL,$action=NULL){
-		parent::__construct('cancel',S_CANCEL);
-		$this->attributes['type'] = 'button';
-		$this->setVars($vars);
-		if(!is_null($action))
-			$this->setAttribute('onclick', $action);
+
+
+class CButtonCancel extends CButton {
+
+	public function __construct($vars = null, $action = null, $class = null) {
+		parent::__construct('cancel', _('Cancel'), $action, $class);
+		if (is_null($action)) {
+			$this->setVars($vars);
+		}
 	}
-	public function setVars($value=NULL){
-		global $page;
 
+	public function setVars($value = null) {
 		$url = '?cancel=1';
-		if(!is_null($value)) $url.= $value;
-
+		if (!empty($value)) {
+			$url .= $value;
+		}
 		$uri = new Curl($url);
 		$url = $uri->getUrl();
-
 		return $this->setAttribute('onclick', "javascript: return redirect('".$url."');");
 	}
 }
-?>
