@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -100,6 +100,7 @@ class CTag extends CObject {
 	public function endToString() {
 		$res = ($this->paired === 'yes') ? $this->tag_body_end.'</'.$this->tagname.'>' : '';
 		$res .= $this->tag_end;
+
 		return $res;
 	}
 
@@ -110,6 +111,7 @@ class CTag extends CObject {
 		if ($destroy) {
 			$this->destroy();
 		}
+
 		return $res;
 	}
 
@@ -129,6 +131,7 @@ class CTag extends CObject {
 		if (!is_string($value)) {
 			return $this->error('Incorrect value for SetName "'.$value.'".');
 		}
+
 		return $this->setAttribute('name', $value);
 	}
 
@@ -136,6 +139,7 @@ class CTag extends CObject {
 		if (isset($this->attributes['name'])) {
 			return $this->attributes['name'];
 		}
+
 		return null;
 	}
 
@@ -146,9 +150,22 @@ class CTag extends CObject {
 		else {
 			$this->attributes['class'] .= ' '.$cssClass;
 		}
+
 		return $this->attributes['class'];
 	}
 
+	/**
+	 * HTML class check for existing, return thue if exist
+	 *
+	 * @param string $cssClass
+	 *
+	 * @return bool
+	 */
+	public function hasClass($cssClass) {
+		$chkClass = explode(' ', $this->getAttribute('class'));
+		return in_array($cssClass, $chkClass);
+	}
+
 	public function attr($name, $value) {
 		if (!is_null($value)) {
 			$this->setAttribute($name, $value);
@@ -177,7 +194,7 @@ class CTag extends CObject {
 	/**
 	 * Sets multiple HTML attributes.
 	 *
-	 * @param array $attributes     defined as array(attributeName1 => value1, attributeName2 => value2, ...)
+	 * @param array $attributes		defined as array(attributeName1 => value1, attributeName2 => value2, ...)
 	 */
 	public function setAttributes(array $attributes) {
 		foreach ($attributes as $name => $value) {
@@ -201,13 +218,15 @@ class CTag extends CObject {
 		if (empty($text)) {
 			return false;
 		}
+
 		encodeValues($text);
 		$text = unpack_object($text);
 
-		$this->addAction('onmouseover', 'hintBox.HintWraper(event, this, '.zbx_jsvalue($text).', \''.$width.'\', \''.$class.'\');');
+		$this->addAction('onmouseover', 'hintBox.HintWraper(event, this, '.zbx_jsvalue($text).', "'.$width.'", "'.$class.'");');
 		if ($byClick) {
-			$this->addAction('onclick', 'hintBox.showStaticHint(event, this, '.zbx_jsvalue($text).', \''.$width.'\', \''.$class.'\');');
+			$this->addAction('onclick', 'hintBox.showStaticHint(event, this, '.zbx_jsvalue($text).', "'.$width.'", "'.$class.'");');
 		}
+
 		return true;
 	}
 
@@ -244,6 +263,7 @@ class CTag extends CObject {
 	public function getForm($method = 'post', $action = null, $enctype = null) {
 		$form = new CForm($method, $action, $enctype);
 		$form->addItem($this);
+
 		return $form;
 	}
 
@@ -254,8 +274,8 @@ class CTag extends CObject {
 	/**
 	 * Sanitizes a string according to the given strategy before outputting it to the browser.
 	 *
-	 * @param string $value
-	 * @param int $strategy
+	 * @param string	$value
+	 * @param int		$strategy
 	 *
 	 * @return string
 	 */
