@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -26,7 +26,7 @@ abstract class CValidator {
 	 *
 	 * @var array
 	 */
-	private $errors = array();
+	private $error;
 	protected $options = array();
 
 	public function __construct(array $options = array()) {
@@ -48,9 +48,7 @@ abstract class CValidator {
 	 * @return string
 	 */
 	public function getError() {
-		$error = reset($this->errors);
-
-		return $error;
+		return $this->error;
 	}
 
 	/**
@@ -58,8 +56,8 @@ abstract class CValidator {
 	 *
 	 * @param $error
 	 */
-	protected function addError($error) {
-		$this->errors[] = $error;
+	protected function setError($error) {
+		$this->error = $error;
 	}
 
 	protected function initOptions() {}
