@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -76,9 +76,14 @@ require_once dirname(__FILE__).'/js.inc.php';
 // include validation
 require_once dirname(__FILE__).'/validate.inc.php';
 
+// disable deprecated errors in PHP newer then 5.5 to hide error messages caused by using the mysql extension
+if (version_compare(phpversion(), '5.5', '>=')) {
+	error_reporting(error_reporting() & ~E_DEPRECATED);
+}
+
 function zbx_err_handler($errno, $errstr, $errfile, $errline) {
-	// necessary to surpress errors when calling with error control operator like @function_name()
-	if (error_reporting() === 0) {
+	// skip errors that are disabled
+	if (!(error_reporting() & $errno)) {
 		return true;
 	}
 
