@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -121,8 +121,11 @@ function DBconnect(&$error) {
 					}
 				}
 
-				$DB['DB']= ociplogon($DB['USER'], $DB['PASSWORD'], $connect);
-				if (!$DB['DB']) {
+				$DB['DB'] = ociplogon($DB['USER'], $DB['PASSWORD'], $connect);
+				if ($DB['DB']) {
+					DBexecute('ALTER SESSION SET NLS_NUMERIC_CHARACTERS='.zbx_dbstr('. '));
+				}
+				else {
 					$error = 'Error connecting to database';
 					$result = false;
 				}
@@ -147,7 +150,7 @@ function DBconnect(&$error) {
 					);
 					db2_set_option($DB['DB'], $options, 1);
 					if (isset($DB['SCHEMA']) && ($DB['SCHEMA'] != '')) {
-						DBexecute("SET CURRENT SCHEMA='".$DB['SCHEMA']."'");
+						DBexecute('SET CURRENT SCHEMA='.zbx_dbstr($DB['SCHEMA']));
 					}
 				}
 				break;
@@ -555,12 +558,12 @@ function DBexecute($query, $skip_error_messages = 0) {
 	return (bool) $result;
 }
 
-function DBfetch(&$cursor) {
+function DBfetch(&$cursor, $convertNulls = true) {
 	global $DB;
 
 	$result = false;
 
-	if (!isset($DB['DB']) || empty($DB['DB'])) {
+	if (!isset($DB['DB']) || empty($DB['DB']) || is_bool($cursor)) {
 		return $result;
 	}
 
@@ -593,6 +596,16 @@ function DBfetch(&$cursor) {
 			if (!$result = db2_fetch_assoc($cursor)) {
 				db2_free_result($cursor);
 			}
+			else {
+				// cast all of the values to string to be consistent with other DB drivers: all of them return
+				// only strings.
+				foreach ($result as &$value) {
+					if ($value !== null) {
+						$value = (string) $value;
+					}
+				}
+				unset($value);
+			}
 			break;
 		case ZBX_DB_SQLITE3:
 			if ($DB['TRANSACTIONS'] == 0) {
@@ -615,7 +628,7 @@ function DBfetch(&$cursor) {
 			break;
 	}
 
-	if ($result) {
+	if ($convertNulls && $result) {
 		foreach ($result as $key => $val) {
 			if (is_null($val)) {
 				$result[$key] = '0';
@@ -641,7 +654,7 @@ if (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_MYSQL) {
 		return ' CAST('.$field.' AS UNSIGNED) ';
 	}
 
-	function zbx_limit($min = 1, $max = null) {
+	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
 		return !empty($max) ? 'LIMIT '.$min.','.$max : 'LIMIT '.$min;
 	}
 }
@@ -660,7 +673,7 @@ elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_POSTGRESQL) {
 		return ' CAST('.$field.' AS BIGINT) ';
 	}
 
-	function zbx_limit($min = 1, $max = null) {
+	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
 		return !empty($max) ? 'LIMIT '.$min.','.$max : 'LIMIT '.$min;
 	}
 }
@@ -679,8 +692,13 @@ elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_ORACLE) {
 		return ' CAST('.$field.' AS NUMBER(20)) ';
 	}
 
-	function zbx_limit($min = 1, $max = null) {
-		return !empty($max) ? 'ROWNUM BETWEEN '.$min.' AND '.$max : 'ROWNUM <='.$min;
+	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
+		if ($afterWhere) {
+			return !empty($max) ? ' AND ROWNUM BETWEEN '.$min.' AND '.$max : ' AND ROWNUM <='.$min;
+		}
+		else {
+			return !empty($max) ? ' WHERE ROWNUM BETWEEN '.$min.' AND '.$max : ' WHERE ROWNUM <='.$min;
+		}
 	}
 }
 elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_DB2) {
@@ -698,8 +716,13 @@ elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_DB2) {
 		return ' CAST('.$field.' AS BIGINT) ';
 	}
 
-	function zbx_limit($min = 1, $max = null) {
-		return !empty($max) ? 'ROWNUM BETWEEN '.$min.' AND '.$max : 'ROWNUM <='.$min;
+	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
+		if ($afterWhere) {
+			return !empty($max) ? ' AND ROWNUM BETWEEN '.$min.' AND '.$max : ' AND ROWNUM <='.$min;
+		}
+		else {
+			return !empty($max) ? ' WHERE ROWNUM BETWEEN '.$min.' AND '.$max : ' WHERE ROWNUM <='.$min;
+		}
 	}
 }
 elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_SQLITE3) {
@@ -719,7 +742,7 @@ elseif (isset($DB['TYPE']) && $DB['TYPE'] == ZBX_DB_SQLITE3) {
 		return ' CAST('.$field.' AS BIGINT) ';
 	}
 
-	function zbx_limit($min = 1, $max = null) {
+	function zbx_limit($min = 1, $max = null, $afterWhere = true) {
 		return !empty($max) ? 'LIMIT '.$min.','.$max : 'LIMIT '.$min;
 	}
 }
@@ -965,35 +988,6 @@ function zbx_db_search($table, $options, &$sql_parts) {
 	return false;
 }
 
-function zbx_db_filter($table, $options, &$sql_parts) {
-	list($table, $tableShort) = explode(' ', $table);
-
-	$tableSchema = DB::getSchema($table);
-	if (!$tableSchema) {
-		info(_s('Error in search request for table "%1$s".', $table));
-	}
-
-	$filter = array();
-	foreach ($options['filter'] as $field => $value) {
-		if (!isset($tableSchema['fields'][$field]) || zbx_empty($value)) {
-			continue;
-		}
-		zbx_value2array($value);
-		$filter[$field] = DBcondition($tableShort.'.'.$field, $value);
-	}
-
-	if (!empty($filter)) {
-		if (isset($sql_parts['where']['filter'])) {
-			$filter[] = $sql_parts['where']['filter'];
-		}
-
-		$glue = (is_null($options['searchByAny']) || $options['searchByAny'] === false) ? ' AND ' : ' OR ';
-		$sql_parts['where']['filter'] = '( '.implode($glue, $filter).' )';
-		return true;
-	}
-	return false;
-}
-
 function zbx_db_sorting(&$sql_parts, $options, $sort_columns, $alias) {
 	if (!zbx_empty($options['sortfield'])) {
 		if (!is_array($options['sortfield'])) {
@@ -1063,26 +1057,141 @@ function check_db_fields($db_fields, &$args) {
 	return true;
 }
 
-function DBcondition($fieldname, $array, $notin = false) {
-	$condition = '';
+/**
+ * Takes an initial part of SQL query and appends a generated WHERE condition.
+ * The WHERE condition is generated from the given list of values as a mix of
+ * <fieldname> BETWEEN <id1> AND <idN>" and "<fieldname> IN (<id1>,<id2>,...,<idN>)" elements.
+ *
+ * In some frontend places we can get array with bool as input values parameter. This is fail!
+ * Therefore we need check it and return 1=0 as temporary solution to not break the frontend.
+ *
+ * @param string $fieldName		field name to be used in SQL WHERE condition
+ * @param array  $values		array of numerical values sorted in ascending order to be included in WHERE
+ * @param bool   $notIn			builds inverted condition
+ * @param bool   $sort			values mandatory must be sorted
+ *
+ * @return string
+ */
+function dbConditionInt($fieldName, array $values, $notIn = false, $sort = true) {
+	$MAX_EXPRESSIONS = 950; // maximum  number of values for using "IN (id1>,<id2>,...,<idN>)"
+	$MIN_NUM_BETWEEN = 4; // minimum number of consecutive values for using "BETWEEN <id1> AND <idN>"
 
-	if (!is_array($array)) {
-		throw new APIException(1, 'DBcondition Error: ['.$fieldname.'] = '.$array);
-		return ' 1=0 ';
+	if (is_bool(reset($values))) {
+		return '1=0';
 	}
 
-	$in = $notin ? ' NOT IN ':' IN ';
-	$concat = $notin ? ' AND ':' OR ';
+	$values = array_keys(array_flip($values));
 
-	$items = array_chunk($array, 950);
+	if ($sort) {
+		natsort($values);
+
+		$values = array_values($values);
+	}
+
+	$betweens = array();
+	$data = array();
+
+	for ($i = 0, $size = count($values); $i < $size; $i++) {
+		$between = array();
+
+		// analyze by chunk
+		if (isset($values[$i + $MIN_NUM_BETWEEN])
+				&& bccomp(bcadd($values[$i], $MIN_NUM_BETWEEN), $values[$i + $MIN_NUM_BETWEEN]) == 0) {
+			for ($sizeMinBetween = $i + $MIN_NUM_BETWEEN; $i < $sizeMinBetween; $i++) {
+				$between[] = $values[$i];
+			}
+
+			$i--; // shift 1 back
+
+			// analyze by one
+			for (; $i < $size; $i++) {
+				if (isset($values[$i + 1]) && bccomp(bcadd($values[$i], 1), $values[$i + 1]) == 0) {
+					$between[] = $values[$i + 1];
+				}
+				else {
+					break;
+				}
+			}
+
+			$betweens[] = $between;
+		}
+		else {
+			$data[] = $values[$i];
+		}
+	}
+
+	// concatenate conditions
+	$dataSize = count($data);
+	$betweenSize = count($betweens);
+
+	$condition = '';
+	$operatorAnd = $notIn ? ' AND ' : ' OR ';
+
+	if ($betweens) {
+		$operatorNot = $notIn ? 'NOT ' : '';
+
+		foreach ($betweens as $between) {
+			$between = $operatorNot.$fieldName.' BETWEEN '.zbx_dbstr($between[0]).' AND '.zbx_dbstr(end($between));
+
+			$condition .= $condition ? $operatorAnd.$between : $between;
+		}
+	}
+
+	if ($dataSize == 1) {
+		$operator = $notIn ? '!=' : '=';
+
+		$condition .= ($condition ? $operatorAnd : '').$fieldName.$operator.zbx_dbstr($data[0]);
+	}
+	else {
+		$operatorNot = $notIn ? ' NOT' : '';
+		$data = array_chunk($data, $MAX_EXPRESSIONS);
+
+		foreach ($data as $chunk) {
+			$chunkIns = '';
+
+			foreach ($chunk as $value) {
+				$chunkIns .= ','.zbx_dbstr($value);
+			}
+
+			$chunkIns = $fieldName.$operatorNot.' IN ('.substr($chunkIns, 1).')';
+
+			$condition .= $condition ? $operatorAnd.$chunkIns : $chunkIns;
+		}
+	}
+
+	return (($dataSize && $betweenSize) || $betweenSize > 1 || $dataSize > $MAX_EXPRESSIONS) ? '('.$condition.')' : $condition;
+}
+
+/**
+ * Takes an initial part of SQL query and appends a generated WHERE condition.
+ *
+ * @param string $fieldName  field name to be used in SQL WHERE condition
+ * @param array  $values     array of string values sorted in ascending order to be included in WHERE
+ * @param bool   $notIn      builds inverted condition
+ *
+ * @return string
+ */
+function dbConditionString($fieldName, array $values, $notIn = false) {
+	switch (count($values)) {
+		case 0:
+			return '1=0';
+		case 1:
+			return $notIn
+				? $fieldName.'!='.zbx_dbstr(reset($values))
+				: $fieldName.'='.zbx_dbstr(reset($values));
+	}
+
+	$in = $notIn ? ' NOT IN ' : ' IN ';
+	$concat = $notIn ? ' AND ' : ' OR ';
+	$items = array_chunk($values, 950);
+
+	$condition = '';
 	foreach ($items as $values) {
-		$condition .= !empty($condition) ? ')'.$concat.$fieldname.$in.'(' : '';
+		$condition .= !empty($condition) ? ')'.$concat.$fieldName.$in.'(' : '';
 		$condition .= implode(',', zbx_dbstr($values));
 	}
-	if (zbx_empty($condition)) {
-		return ' 1=0 ';
-	}
-	return ' ('.$fieldname.$in.'('.$condition.')) ';
+
+	return '('.$fieldName.$in.'('.$condition.'))';
 }
 
 function zero2null($val) {
@@ -1160,7 +1269,7 @@ function unlock_sqlite3_access() {
  * @return bool
  */
 function idcmp($id1, $id2) {
-	return $id1 === $id2;
+	return (string) $id1 === (string) $id2;
 }
 
 /**
