@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -120,7 +120,7 @@ function getGraphDims($graphid = null) {
 	$dbGraphs = DBselect(
 		'SELECT MAX(g.graphtype) AS graphtype,MIN(gi.yaxisside) AS yaxissidel,MAX(gi.yaxisside) AS yaxissider,MAX(g.height) AS height'.
 		' FROM graphs g,graphs_items gi'.
-		' WHERE g.graphid='.$graphid.
+		' WHERE g.graphid='.zbx_dbstr($graphid).
 			' AND gi.graphid=g.graphid'
 	);
 	if ($graph = DBfetch($dbGraphs)) {
@@ -154,7 +154,7 @@ function get_graphs_by_hostid($hostid) {
 		' FROM graphs g,graphs_items gi,items i'.
 		' WHERE g.graphid=gi.graphid'.
 			' AND gi.itemid=i.itemid'.
-			' AND i.hostid='.$hostid
+			' AND i.hostid='.zbx_dbstr($hostid)
 	);
 }
 
@@ -172,13 +172,13 @@ function get_hosts_by_graphid($graphid) {
 		' FROM graphs_items gi,items i,hosts h'.
 		' WHERE h.hostid=i.hostid'.
 			' AND gi.itemid=i.itemid'.
-			' AND gi.graphid='.$graphid
+			' AND gi.graphid='.zbx_dbstr($graphid)
 	);
 }
 
-/*
+/**
  * Description:
- *     Return the time of the 1st appearance of items included in graph in trends
+ *	Return the time of the 1st appearance of items included in graph in trends
  * Comment:
  *	sql is split to many sql's to optimize search on history tables
  */
@@ -187,7 +187,7 @@ function get_min_itemclock_by_graphid($graphid) {
 	$dbItems = DBselect(
 		'SELECT DISTINCT gi.itemid'.
 		' FROM graphs_items gi'.
-		' WHERE gi.graphid='.$graphid
+		' WHERE gi.graphid='.zbx_dbstr($graphid)
 	);
 	while ($item = DBfetch($dbItems)) {
 		$itemids[$item['itemid']] = $item['itemid'];
@@ -196,18 +196,22 @@ function get_min_itemclock_by_graphid($graphid) {
 	return get_min_itemclock_by_itemid($itemids);
 }
 
-/*
- * Description:
- *     Return the time of the 1st appearance of item in trends
+/**
+ * Return the time of the 1st appearance of item in trends.
+ *
+ * @param array $itemIds
+ *
+ * @return int (unixtime)
  */
-function get_min_itemclock_by_itemid($itemids) {
-	zbx_value2array($itemids);
+function get_min_itemclock_by_itemid($itemIds) {
+	zbx_value2array($itemIds);
+
 	$min = null;
 	$result = time() - SEC_PER_YEAR;
 
-	$items_by_type = array(
+	$itemTypes = array(
 		ITEM_VALUE_TYPE_FLOAT => array(),
-		ITEM_VALUE_TYPE_STR =>  array(),
+		ITEM_VALUE_TYPE_STR => array(),
 		ITEM_VALUE_TYPE_LOG => array(),
 		ITEM_VALUE_TYPE_UINT64 => array(),
 		ITEM_VALUE_TYPE_TEXT => array()
@@ -216,72 +220,72 @@ function get_min_itemclock_by_itemid($itemids) {
 	$dbItems = DBselect(
 		'SELECT i.itemid,i.value_type'.
 		' FROM items i'.
-		' WHERE '.DBcondition('i.itemid', $itemids)
+		' WHERE '.dbConditionInt('i.itemid', $itemIds)
 	);
 
 	while ($item = DBfetch($dbItems)) {
-		$items_by_type[$item['value_type']][$item['itemid']] = $item['itemid'];
+		$itemTypes[$item['value_type']][$item['itemid']] = $item['itemid'];
 	}
 
 	// data for ITEM_VALUE_TYPE_FLOAT and ITEM_VALUE_TYPE_UINT64 can be stored in trends tables or history table
 	// get max trends and history values for such type items to find out in what tables to look for data
-	$sql_from = 'history';
-	$sql_from_num = '';
+	$sqlFrom = 'history';
+	$sqlFromNum = '';
 
-	if (!empty($items_by_type[ITEM_VALUE_TYPE_FLOAT]) || !empty($items_by_type[ITEM_VALUE_TYPE_UINT64])) {
-		$itemids_numeric = zbx_array_merge($items_by_type[ITEM_VALUE_TYPE_FLOAT], $items_by_type[ITEM_VALUE_TYPE_UINT64]);
+	if (!empty($itemTypes[ITEM_VALUE_TYPE_FLOAT]) || !empty($itemTypes[ITEM_VALUE_TYPE_UINT64])) {
+		$itemIdsNumeric = zbx_array_merge($itemTypes[ITEM_VALUE_TYPE_FLOAT], $itemTypes[ITEM_VALUE_TYPE_UINT64]);
 
 		$sql = 'SELECT MAX(i.history) AS history,MAX(i.trends) AS trends'.
 				' FROM items i'.
-				' WHERE '.DBcondition('i.itemid', $itemids_numeric);
-		if ($table_for_numeric = DBfetch(DBselect($sql))) {
-			$sql_from_num = ($table_for_numeric['history'] > $table_for_numeric['trends']) ? 'history' : 'trends';
-			$result = time() - (SEC_PER_DAY * max($table_for_numeric['history'], $table_for_numeric['trends']));
+				' WHERE '.dbConditionInt('i.itemid', $itemIdsNumeric);
+		if ($tableForNumeric = DBfetch(DBselect($sql))) {
+			$sqlFromNum = ($tableForNumeric['history'] > $tableForNumeric['trends']) ? 'history' : 'trends';
+			$result = time() - (SEC_PER_DAY * max($tableForNumeric['history'], $tableForNumeric['trends']));
 		}
 	}
 
-	foreach ($items_by_type as $type => $items) {
+	foreach ($itemTypes as $type => $items) {
 		if (empty($items)) {
 			continue;
 		}
 
-		switch($type) {
+		switch ($type) {
 			case ITEM_VALUE_TYPE_FLOAT:
-				$sql_from = $sql_from_num;
+				$sqlFrom = $sqlFromNum;
 				break;
 			case ITEM_VALUE_TYPE_STR:
-				$sql_from = 'history_str';
+				$sqlFrom = 'history_str';
 				break;
 			case ITEM_VALUE_TYPE_LOG:
-				$sql_from = 'history_log';
+				$sqlFrom = 'history_log';
 				break;
 			case ITEM_VALUE_TYPE_UINT64:
-				$sql_from = $sql_from_num.'_uint';
+				$sqlFrom = $sqlFromNum.'_uint';
 				break;
 			case ITEM_VALUE_TYPE_TEXT:
-				$sql_from = 'history_text';
+				$sqlFrom = 'history_text';
 				break;
 			default:
-				$sql_from = 'history';
+				$sqlFrom = 'history';
 		}
 
-		$res = DBselect(
-			'SELECT ht.itemid,MIN(ht.clock) AS min_clock'.
-			' FROM '.$sql_from.' ht'.
-			' WHERE '.DBcondition('ht.itemid', $itemids).
-			' GROUP BY ht.itemid'
-		);
-		while ($min_tmp = DBfetch($res)) {
-			$min = (is_null($min)) ? $min_tmp['min_clock'] : min($min, $min_tmp['min_clock']);
+		foreach ($itemIds as $itemId) {
+			$sqlUnions[] = 'SELECT MIN(ht.clock) AS c FROM '.$sqlFrom.' ht WHERE ht.itemid='.zbx_dbstr($itemId);
 		}
+
+		$dbMin = DBfetch(DBselect(
+			'SELECT MIN(ht.c) AS min_clock'.
+			' FROM ('.implode(' UNION ALL ', $sqlUnions).') ht'
+		));
+
+		$min = $min ? min($min, $dbMin['min_clock']) : $dbMin['min_clock'];
 	}
-	$result = is_null($min) ? $result : $min;
 
-	return $result;
+	return $min ? $min: $result;
 }
 
 function get_graph_by_graphid($graphid) {
-	$dbGraphs = DBselect('SELECT g.* FROM graphs g WHERE g.graphid='.$graphid);
+	$dbGraphs = DBselect('SELECT g.* FROM graphs g WHERE g.graphid='.zbx_dbstr($graphid));
 	$dbGraphs = DBfetch($dbGraphs);
 	if (!empty($dbGraphs)) {
 		return $dbGraphs;
@@ -308,8 +312,8 @@ function get_same_graphitems_for_host($gitems, $dest_hostid, $error = true) {
 			'SELECT dest.itemid,src.key_'.
 			' FROM items dest,items src'.
 			' WHERE dest.key_=src.key_'.
-				' AND dest.hostid='.$dest_hostid.
-				' AND src.itemid='.$gitem['itemid']
+				' AND dest.hostid='.zbx_dbstr($dest_hostid).
+				' AND src.itemid='.zbx_dbstr($gitem['itemid'])
 		));
 
 		if ($dbItem) {
@@ -368,24 +372,26 @@ function copy_graph_to_host($graphid, $hostid) {
 	}
 
 	$graph['gitems'] = $new_gitems;
+	unset($graph['templateid']);
 	$result = API::Graph()->create($graph);
 
 	return $result;
 }
 
 function navigation_bar_calc($idx = null, $idx2 = 0, $update = false) {
-	if (!is_null($idx)) {
+	if (!empty($idx)) {
 		if ($update) {
-			if (isset($_REQUEST['period']) && $_REQUEST['period'] >= ZBX_MIN_PERIOD) {
+			if (!empty($_REQUEST['period']) && $_REQUEST['period'] >= ZBX_MIN_PERIOD) {
 				CProfile::update($idx.'.period', $_REQUEST['period'], PROFILE_TYPE_INT, $idx2);
 			}
-			if (isset($_REQUEST['stime'])) {
+			if (!empty($_REQUEST['stime'])) {
 				CProfile::update($idx.'.stime', $_REQUEST['stime'], PROFILE_TYPE_STR, $idx2);
 			}
 		}
 		$_REQUEST['period'] = get_request('period', CProfile::get($idx.'.period', ZBX_PERIOD_DEFAULT, $idx2));
 		$_REQUEST['stime'] = get_request('stime', CProfile::get($idx.'.stime', null, $idx2));
 	}
+
 	$_REQUEST['period'] = get_request('period', ZBX_PERIOD_DEFAULT);
 	$_REQUEST['stime'] = get_request('stime', null);
 
@@ -404,7 +410,7 @@ function navigation_bar_calc($idx = null, $idx2 = 0, $update = false) {
 		$_REQUEST['period'] = ZBX_MAX_PERIOD;
 	}
 
-	if (isset($_REQUEST['stime'])) {
+	if (!empty($_REQUEST['stime'])) {
 		$time = zbxDateToTime($_REQUEST['stime']);
 		if (($time + $_REQUEST['period']) > time()) {
 			$_REQUEST['stime'] = date('YmdHis', time() - $_REQUEST['period']);
@@ -413,6 +419,7 @@ function navigation_bar_calc($idx = null, $idx2 = 0, $update = false) {
 	else {
 		$_REQUEST['stime'] = date('YmdHis', time() - $_REQUEST['period']);
 	}
+
 	return $_REQUEST['period'];
 }
 
