@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,20 +10,19 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
-?>
-<?php
+
 
 function setHostGroupInternal($groupids, $internal = ZBX_NOT_INTERNAL_GROUP) {
 	zbx_value2array($groupids);
 
-	return DBexecute('UPDATE groups SET internal='.$internal.' WHERE '.DBcondition('groupid', $groupids));
+	return DBexecute('UPDATE groups SET internal='.$internal.' WHERE '.dbConditionInt('groupid', $groupids));
 }
 
 /**
@@ -484,7 +483,7 @@ function hostInterfaceTypeNumToName($type) {
 }
 
 function get_hostgroup_by_groupid($groupid) {
-	$groups = DBfetch(DBselect('SELECT g.* FROM groups g WHERE g.groupid='.$groupid));
+	$groups = DBfetch(DBselect('SELECT g.* FROM groups g WHERE g.groupid='.zbx_dbstr($groupid)));
 	if (!empty($groups)) {
 		return $groups;
 	}
@@ -503,7 +502,7 @@ function get_host_by_itemid($itemids) {
 		'SELECT i.itemid,h.*'.
 		' FROM hosts h,items i'.
 		' WHERE i.hostid=h.hostid'.
-			' AND '.DBcondition('i.itemid', $itemids)
+			' AND '.dbConditionInt('i.itemid', $itemids)
 	);
 	while ($hostItem = DBfetch($db_hostsItems)) {
 		$result = true;
@@ -523,7 +522,7 @@ function get_host_by_itemid($itemids) {
 }
 
 function get_host_by_hostid($hostid, $no_error_message = 0) {
-	$row = DBfetch(DBselect('SELECT h.* FROM hosts h WHERE h.hostid='.$hostid));
+	$row = DBfetch(DBselect('SELECT h.* FROM hosts h WHERE h.hostid='.zbx_dbstr($hostid)));
 	if ($row) {
 		return $row;
 	}
@@ -539,7 +538,7 @@ function get_hosts_by_templateid($templateids) {
 		'SELECT h.*'.
 		' FROM hosts h,hosts_templates ht'.
 		' WHERE h.hostid=ht.hostid'.
-			' AND '.DBcondition('ht.templateid', $templateids)
+			' AND '.dbConditionInt('ht.templateid', $templateids)
 	);
 }
 
@@ -551,7 +550,7 @@ function updateHostStatus($hostids, $status) {
 	$db_hosts = DBselect(
 		'SELECT h.hostid,h.host,h.status'.
 			' FROM hosts h'.
-			' WHERE '.DBcondition('h.hostid', $hostids).
+			' WHERE '.dbConditionInt('h.hostid', $hostids).
 				' AND h.status='.$oldStatus
 	);
 	while ($host = DBfetch($db_hosts)) {
@@ -627,13 +626,11 @@ function get_viewed_groups($perm, $options = array(), $nodeid = null, $sql = arr
 	$_REQUEST['hostid'] = get_request('hostid', -1);
 
 	if (is_null($nodeid)) {
-		if (!$def_options['only_current_node']) {
-			$nodeid = get_current_nodeid();
-		}
-		else {
-			$nodeid = get_current_nodeid(false);
-		}
+		$nodeid = (!$def_options['only_current_node'])
+			? get_current_nodeid()
+			: get_current_nodeid(false);
 	}
+
 	$available_groups = get_accessible_groups_by_user($USER_DETAILS, $perm, PERM_RES_IDS_ARRAY, $nodeid, AVAILABLE_NOCACHE);
 
 	// nodes
@@ -673,40 +670,49 @@ function get_viewed_groups($perm, $options = array(), $nodeid = null, $sql = arr
 		$def_sql['from'][] = 'hosts_groups hg';
 
 		$def_sql['where'][] = 'hg.groupid=g.groupid';
-		$def_sql['where'][] = 'EXISTS (SELECT i.hostid FROM items i WHERE hg.hostid=i.hostid)';
+		$def_sql['where'][] = 'EXISTS (SELECT 1'.
+										' FROM items i'.
+										' WHERE hg.hostid=i.hostid '.zbx_limit(1).')';
 	}
 	elseif ($def_options['with_monitored_items']) {
 		$def_sql['from'][] = 'hosts_groups hg';
 
 		$def_sql['where'][] = 'hg.groupid=g.groupid';
-		$def_sql['where'][] = 'EXISTS (SELECT i.hostid FROM items i WHERE hg.hostid=i.hostid AND i.status='.ITEM_STATUS_ACTIVE.')';
+		$def_sql['where'][] = 'EXISTS (SELECT 1'.
+										' FROM items i'.
+										' WHERE hg.hostid=i.hostid'.
+											' AND i.status='.ITEM_STATUS_ACTIVE.' '.zbx_limit(1).')';
 	}
 	elseif ($def_options['with_historical_items']) {
 		$def_sql['from'][] = 'hosts_groups hg';
 		$def_sql['where'][] = 'hg.groupid=g.groupid';
-		$def_sql['where'][] = 'EXISTS (SELECT i.hostid FROM items i WHERE hg.hostid=i.hostid AND (i.status='.ITEM_STATUS_ACTIVE.' OR i.status='.ITEM_STATUS_NOTSUPPORTED.') AND i.lastvalue IS NOT NULL)';
+		$def_sql['where'][] = 'EXISTS (SELECT 1'.
+										' FROM items i'.
+										' WHERE hg.hostid=i.hostid'.
+											' AND (i.status='.ITEM_STATUS_ACTIVE.' OR i.status='.ITEM_STATUS_NOTSUPPORTED.')'.
+											' AND i.lastvalue IS NOT NULL '.zbx_limit(1).')';
 	}
 
 	// triggers
 	if ($def_options['with_triggers']) {
 		$def_sql['from'][] = 'hosts_groups hg';
 		$def_sql['where'][] = 'hg.groupid=g.groupid';
-		$def_sql['where'][] = 'EXISTS (SELECT t.triggerid'.
+		$def_sql['where'][] = 'EXISTS (SELECT 1'.
 										' FROM items i,functions f,triggers t'.
 										' WHERE i.hostid=hg.hostid'.
 											' AND f.itemid=i.itemid'.
-											' AND t.triggerid=f.triggerid)';
+											' AND t.triggerid=f.triggerid '.zbx_limit(1).')';
 	}
 	elseif ($def_options['with_monitored_triggers']) {
 		$def_sql['from'][] = 'hosts_groups hg';
 		$def_sql['where'][] = 'hg.groupid=g.groupid';
-		$def_sql['where'][] = 'EXISTS (SELECT t.triggerid'.
+		$def_sql['where'][] = 'EXISTS (SELECT 1'.
 										' FROM items i,functions f,triggers t'.
 										' WHERE i.hostid=hg.hostid'.
 											' AND i.status='.ITEM_STATUS_ACTIVE.
 											' AND i.itemid=f.itemid'.
 											' AND f.triggerid=t.triggerid'.
-											' AND t.status='.TRIGGER_STATUS_ENABLED.')';
+											' AND t.status='.TRIGGER_STATUS_ENABLED.' '.zbx_limit(1).')';
 	}
 
 	// httptests
@@ -714,28 +720,28 @@ function get_viewed_groups($perm, $options = array(), $nodeid = null, $sql = arr
 		$def_sql['from'][] = 'hosts_groups hg';
 		$def_sql['where'][] = 'hg.groupid=g.groupid';
 		$def_sql['where'][] = 'EXISTS (SELECT a.applicationid'.
-									' FROM applications a,httptest ht'.
-									' WHERE a.hostid=hg.hostid'.
-										' AND ht.applicationid=a.applicationid)';
+										' FROM applications a,httptest ht'.
+										' WHERE a.hostid=hg.hostid'.
+											' AND ht.applicationid=a.applicationid '.zbx_limit(1).')';
 	}
 	elseif ($def_options['with_monitored_httptests']) {
 		$def_sql['from'][] = 'hosts_groups hg';
 		$def_sql['where'][] = 'hg.groupid=g.groupid';
-		$def_sql['where'][] = 'EXISTS (SELECT a.applicationid'.
+		$def_sql['where'][] = 'EXISTS (SELECT 1'.
 										' FROM applications a,httptest ht'.
 										' WHERE a.hostid=hg.hostid'.
 											' AND ht.applicationid=a.applicationid'.
-											' AND ht.status='.HTTPTEST_STATUS_ACTIVE.')';
+											' AND ht.status='.HTTPTEST_STATUS_ACTIVE.' '.zbx_limit(1).')';
 	}
 
 	// graphs
 	if ($def_options['with_graphs']) {
 		$def_sql['from'][] = 'hosts_groups hg';
 		$def_sql['where'][] = 'hg.groupid=g.groupid';
-		$def_sql['where'][] = 'EXISTS (SELECT DISTINCT i.itemid'.
+		$def_sql['where'][] = 'EXISTS (SELECT 1'.
 										' FROM items i,graphs_items gi'.
 										' WHERE i.hostid=hg.hostid'.
-											' AND i.itemid=gi.itemid)';
+											' AND i.itemid=gi.itemid '.zbx_limit(1).')';
 	}
 
 	$def_sql['order'][] = 'g.name';
@@ -743,12 +749,9 @@ function get_viewed_groups($perm, $options = array(), $nodeid = null, $sql = arr
 	foreach ($sql as $key => $value) {
 		zbx_value2array($value);
 
-		if (isset($def_sql[$key])) {
-			$def_sql[$key] = zbx_array_merge($def_sql[$key], $value);
-		}
-		else {
-			$def_sql[$key] = $value;
-		}
+		$def_sql[$key] = (!empty($def_sql[$key]))
+			? zbx_array_merge($def_sql[$key], $value)
+			: $value;
 	}
 
 	$def_sql['select'] = array_unique($def_sql['select']);
@@ -776,12 +779,13 @@ function get_viewed_groups($perm, $options = array(), $nodeid = null, $sql = arr
 	$res = DBselect(
 		'SELECT DISTINCT '.$sql_select.
 		' FROM '.$sql_from.
-		' WHERE '.DBcondition('g.groupid', $available_groups).
+		' WHERE '.dbConditionInt('g.groupid', $available_groups).
 			$sql_where.
 		' ORDER BY '.$sql_order);
 	while ($group = DBfetch($res)) {
 		$groups[$group['groupid']] = $group['name'];
 		$groupids[$group['groupid']] = $group['groupid'];
+
 		if (bccomp($_REQUEST['groupid'], $group['groupid']) == 0) {
 			$result['selected'] = $group['groupid'];
 		}
@@ -800,12 +804,9 @@ function get_viewed_groups($perm, $options = array(), $nodeid = null, $sql = arr
 		$first_groupid = next($groupids);
 		reset($groupids);
 
-		if ($first_groupid !== false) {
-			$_REQUEST['groupid'] = $result['selected'] = $first_groupid;
-		}
-		else {
-			$_REQUEST['groupid'] = $result['selected'] = 0;
-		}
+		$_REQUEST['groupid'] = ($first_groupid !== false)
+			? $result['selected'] = $first_groupid
+			: $result['selected'] = 0;
 	}
 	else {
 		if ($config['dropdown_first_remember']) {
@@ -823,6 +824,7 @@ function get_viewed_groups($perm, $options = array(), $nodeid = null, $sql = arr
 			$_REQUEST['groupid'] = $result['selected'];
 		}
 	}
+
 	return $result;
 }
 
@@ -850,7 +852,6 @@ function get_viewed_hosts($perm, $groupid = 0, $options = array(), $nodeid = nul
 		'allow_all' =>						0,
 		'select_first_host' =>				0,
 		'select_first_host_if_empty' =>		0,
-		'select_host_on_group_switch' =>	0,
 		'do_not_select' =>					0,
 		'do_not_select_if_empty' =>			0,
 		'monitored_hosts' =>				0,
@@ -879,9 +880,6 @@ function get_viewed_hosts($perm, $groupid = 0, $options = array(), $nodeid = nul
 	if ($def_options['deny_all']) {
 		$dd_first_entry = ZBX_DROPDOWN_FIRST_NONE;
 	}
-	if ($dd_first_entry == ZBX_DROPDOWN_FIRST_ALL) {
-		$def_options['select_host_on_group_switch'] = 1;
-	}
 
 	$result = array('original' => -1, 'selected' => 0, 'hosts' => array(), 'hostids' => array());
 	$hosts = &$result['hosts'];
@@ -900,7 +898,7 @@ function get_viewed_hosts($perm, $groupid = 0, $options = array(), $nodeid = nul
 		zbx_value2array($groupid);
 
 		$def_sql['from'][] = 'hosts_groups hg';
-		$def_sql['where'][] = DBcondition('hg.groupid',$groupid);
+		$def_sql['where'][] = dbConditionInt('hg.groupid',$groupid);
 		$def_sql['where'][] = 'hg.hostid=h.hostid';
 	}
 
@@ -916,22 +914,18 @@ function get_viewed_hosts($perm, $groupid = 0, $options = array(), $nodeid = nul
 	}
 
 	if (USER_TYPE_SUPER_ADMIN != $USER_DETAILS['type']) {
-			$def_sql['from']['hg'] = 'hosts_groups hg';
-			$def_sql['from']['r'] = 'rights r';
-			$def_sql['from']['ug'] = 'users_groups ug';
-			$def_sql['where']['hgh'] = 'hg.hostid=h.hostid';
-			$def_sql['where'][] = 'r.id=hg.groupid ';
-			$def_sql['where'][] = 'r.groupid=ug.usrgrpid';
-			$def_sql['where'][] = 'ug.userid='.$userid;
-			$def_sql['where'][] = 'r.permission>='.$perm;
-			$def_sql['where'][] = 'NOT EXISTS ('.
-									' SELECT hgg.groupid'.
-									' FROM hosts_groups hgg,rights rr,users_groups gg'.
-									' WHERE hgg.hostid=hg.hostid'.
-										' AND rr.id=hgg.groupid'.
-										' AND rr.groupid=gg.usrgrpid'.
-										' AND gg.userid='.$userid.
-										' AND rr.permission<'.$perm.')';
+		$userGroups = getUserGroupsByUserId($userid);
+
+		$def_sql['where'][] = 'EXISTS ('.
+				'SELECT NULL'.
+				' FROM hosts_groups hgg'.
+					' JOIN rights r'.
+						' ON r.id=hgg.groupid'.
+							' AND '.dbConditionInt('r.groupid', $userGroups).
+				' WHERE h.hostid=hgg.hostid'.
+				' GROUP BY hgg.hostid'.
+				' HAVING MIN(r.permission)>='.$perm.
+				')';
 	}
 
 	// nodes
@@ -958,25 +952,25 @@ function get_viewed_hosts($perm, $groupid = 0, $options = array(), $nodeid = nul
 
 	// items
 	if ($def_options['with_items']) {
-		$def_sql['where'][] = 'EXISTS (SELECT i.hostid FROM items i WHERE h.hostid=i.hostid )';
+		$def_sql['where'][] = 'EXISTS (SELECT NULL FROM items i WHERE h.hostid=i.hostid )';
 	}
 	elseif ($def_options['with_monitored_items']) {
-		$def_sql['where'][] = 'EXISTS (SELECT i.hostid FROM items i WHERE h.hostid=i.hostid AND i.status='.ITEM_STATUS_ACTIVE.')';
+		$def_sql['where'][] = 'EXISTS (SELECT NULL FROM items i WHERE h.hostid=i.hostid AND i.status='.ITEM_STATUS_ACTIVE.')';
 	}
 	elseif ($def_options['with_historical_items']) {
-		$def_sql['where'][] = 'EXISTS (SELECT i.hostid FROM items i WHERE h.hostid=i.hostid AND (i.status='.ITEM_STATUS_ACTIVE.' OR i.status='.ITEM_STATUS_NOTSUPPORTED.') AND i.lastvalue IS NOT NULL)';
+		$def_sql['where'][] = 'EXISTS (SELECT NULL FROM items i WHERE h.hostid=i.hostid AND (i.status='.ITEM_STATUS_ACTIVE.' OR i.status='.ITEM_STATUS_NOTSUPPORTED.') AND i.lastvalue IS NOT NULL)';
 	}
 
 	// triggers
 	if ($def_options['with_triggers']) {
-		$def_sql['where'][] = 'EXISTS (SELECT i.itemid'.
+		$def_sql['where'][] = 'EXISTS (SELECT NULL'.
 										' FROM items i,functions f,triggers t'.
 										' WHERE i.hostid=h.hostid'.
 											' AND i.itemid=f.itemid'.
 											' AND f.triggerid=t.triggerid)';
 	}
 	elseif ($def_options['with_monitored_triggers']) {
-		$def_sql['where'][] = 'EXISTS (SELECT i.itemid'.
+		$def_sql['where'][] = 'EXISTS (SELECT NULL'.
 										' FROM items i,functions f,triggers t'.
 										' WHERE i.hostid=h.hostid'.
 											' AND i.status='.ITEM_STATUS_ACTIVE.
@@ -987,13 +981,13 @@ function get_viewed_hosts($perm, $groupid = 0, $options = array(), $nodeid = nul
 
 	// httptests
 	if ($def_options['with_httptests']) {
-		$def_sql['where'][] = 'EXISTS (SELECT a.applicationid'.
+		$def_sql['where'][] = 'EXISTS (SELECT NULL'.
 										' FROM applications a,httptest ht'.
 										' WHERE a.hostid=h.hostid'.
 											' AND ht.applicationid=a.applicationid)';
 	}
 	elseif ($def_options['with_monitored_httptests']) {
-		$def_sql['where'][] = 'EXISTS (SELECT a.applicationid'.
+		$def_sql['where'][] = 'EXISTS (SELECT NULL'.
 										' FROM applications a,httptest ht'.
 										' WHERE a.hostid=h.hostid'.
 											' AND ht.applicationid=a.applicationid'.
@@ -1002,7 +996,7 @@ function get_viewed_hosts($perm, $groupid = 0, $options = array(), $nodeid = nul
 
 	// graphs
 	if ($def_options['with_graphs']) {
-		$def_sql['where'][] = 'EXISTS (SELECT DISTINCT i.itemid'.
+		$def_sql['where'][] = 'EXISTS (SELECT NULL'.
 										' FROM items i,graphs_items gi'.
 										' WHERE i.hostid=h.hostid'.
 											' AND i.itemid=gi.itemid)';
@@ -1068,8 +1062,7 @@ function get_viewed_hosts($perm, $groupid = 0, $options = array(), $nodeid = nul
 		$_REQUEST['hostid'] = $result['selected'] = 0;
 	}
 	elseif ($def_options['select_first_host']
-				|| ($def_options['select_first_host_if_empty'] && $_REQUEST['hostid'] == -1 && is_null($profile_hostid))
-				|| ($def_options['select_host_on_group_switch'] && $_REQUEST['hostid'] != -1 && bccomp($_REQUEST['hostid'], $result['selected']) != 0)) {
+				|| ($def_options['select_first_host_if_empty'] && $_REQUEST['hostid'] == -1 && is_null($profile_hostid))) {
 		$first_hostid = next($hostids);
 		reset($hostids);
 
@@ -1116,7 +1109,7 @@ function validate_group_with_host(&$PAGE_GROUPS, &$PAGE_HOSTS, $reset_host = tru
 
 	if ($_REQUEST['groupid'] > 0) {
 		if ($_REQUEST['hostid'] > 0) {
-			if (!DBfetch(DBselect('SELECT hg.groupid FROM hosts_groups hg WHERE hg.hostid='.$_REQUEST['hostid'].' AND hg.groupid='.$_REQUEST['groupid']))) {
+			if (!DBfetch(DBselect('SELECT hg.groupid FROM hosts_groups hg WHERE hg.hostid='.zbx_dbstr($_REQUEST['hostid']).' AND hg.groupid='.zbx_dbstr($_REQUEST['groupid'])))) {
 				$_REQUEST['hostid'] = 0;
 			}
 		}
@@ -1152,7 +1145,7 @@ function validate_group_with_host(&$PAGE_GROUPS, &$PAGE_HOSTS, $reset_host = tru
 }
 
 function get_application_by_applicationid($applicationid, $no_error_message = 0) {
-	$row = DBfetch(DBselect('SELECT a.* FROM applications a WHERE a.applicationid='.$applicationid));
+	$row = DBfetch(DBselect('SELECT a.* FROM applications a WHERE a.applicationid='.zbx_dbstr($applicationid)));
 	if ($row) {
 		return $row;
 	}
@@ -1164,7 +1157,7 @@ function get_application_by_applicationid($applicationid, $no_error_message = 0)
 }
 
 function get_applications_by_templateid($applicationid) {
-	return DBselect('SELECT a.* FROM applications a WHERE a.templateid='.$applicationid);
+	return DBselect('SELECT a.* FROM applications a WHERE a.templateid='.zbx_dbstr($applicationid));
 }
 
 function get_realhost_by_applicationid($applicationid) {
@@ -1176,7 +1169,7 @@ function get_realhost_by_applicationid($applicationid) {
 }
 
 function get_host_by_applicationid($applicationid) {
-	$row = DBfetch(DBselect('SELECT h.* FROM hosts h,applications a WHERE a.hostid=h.hostid AND a.applicationid='.$applicationid));
+	$row = DBfetch(DBselect('SELECT h.* FROM hosts h,applications a WHERE a.hostid=h.hostid AND a.applicationid='.zbx_dbstr($applicationid)));
 	if ($row) {
 		return $row;
 	}
@@ -1211,7 +1204,7 @@ function validate_templates($templateid_list) {
 	$res = DBselect(
 		'SELECT key_,COUNT(*) AS cnt'.
 		' FROM items'.
-		' WHERE '.DBcondition('hostid', $templateid_list).
+		' WHERE '.dbConditionInt('hostid', $templateid_list).
 		' GROUP BY key_'.
 		' ORDER BY cnt DESC'
 	);
@@ -1225,7 +1218,7 @@ function validate_templates($templateid_list) {
 	$res = DBselect(
 		'SELECT name,COUNT(*) AS cnt'.
 		' FROM applications'.
-		' WHERE '.DBcondition('hostid',$templateid_list).
+		' WHERE '.dbConditionInt('hostid',$templateid_list).
 		' GROUP BY name'.
 		' ORDER BY cnt DESC'
 	);
@@ -1256,13 +1249,13 @@ function getUnlinkableHosts($groupids, $hostids = null) {
 
 	$sql_where = '';
 	if ($hostids !== null) {
-		$sql_where = ' AND '.DBcondition('hg.hostid', $hostids);
+		$sql_where = ' AND '.dbConditionInt('hg.hostid', $hostids);
 	}
 
 	$result = DBselect(
 		'SELECT hg.hostid,COUNT(hg.groupid) AS grp_count'.
 		' FROM hosts_groups hg'.
-		' WHERE '.DBcondition('hg.groupid', $groupids, true).
+		' WHERE '.dbConditionInt('hg.groupid', $groupids, true).
 				$sql_where.
 		' GROUP BY hg.hostid'.
 		' HAVING COUNT(hg.groupid)>0'
@@ -1282,7 +1275,7 @@ function getDeletableHostGroups($groupids = null) {
 
 	$sql_where = '';
 	if (!is_null($groupids)) {
-		$sql_where .= ' AND '.DBcondition('g.groupid', $groupids);
+		$sql_where .= ' AND '.dbConditionInt('g.groupid', $groupids);
 	}
 
 	$db_groups = DBselect(
@@ -1291,15 +1284,16 @@ function getDeletableHostGroups($groupids = null) {
 		' WHERE g.internal='.ZBX_NOT_INTERNAL_GROUP.
 			$sql_where.
 			' AND NOT EXISTS ('.
-				'SELECT hg.groupid'.
+				'SELECT NULL'.
 				' FROM hosts_groups hg'.
 				' WHERE g.groupid=hg.groupid'.
-					(!empty($hostids) ? ' AND '.DBcondition('hg.hostid', $hostids, true) : '').
+					(!empty($hostids) ? ' AND '.dbConditionInt('hg.hostid', $hostids, true) : '').
 			')'
 	);
 	while ($group = DBfetch($db_groups)) {
 		$deletable_groupids[$group['groupid']] = $group['groupid'];
 	}
+
 	return $deletable_groupids;
 }
 
@@ -1307,6 +1301,9 @@ function getDeletableHostGroups($groupids = null) {
  * A helper function for building the value for the 'data-host-menu' attribute.
  *
  * @param array $host
+ * @param int $host['hostid']
+ * @param mixed $host['screens']
+ * @param mixed $host['inventory']
  * @param array $scripts
  *
  * @return array
@@ -1330,7 +1327,7 @@ function hostMenuData(array $host, array $scripts = array()) {
 }
 
 function isTemplate($hostid) {
-	$dbHost = DBfetch(DBselect('SELECT h.status FROM hosts h WHERE h.hostid='.$hostid));
+	$dbHost = DBfetch(DBselect('SELECT h.status FROM hosts h WHERE h.hostid='.zbx_dbstr($hostid)));
 
 	return !empty($dbHost) && $dbHost['status'] == HOST_STATUS_TEMPLATE;
 }
@@ -1347,4 +1344,3 @@ function isTemplateInHost($hosts) {
 
 	return 0;
 }
-?>
