@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -23,7 +23,7 @@ function get_maintenance_by_maintenanceid($maintenanceid) {
 	$sql = 'SELECT m.*'.
 			' FROM maintenances m'.
 			' WHERE '.DBin_node('m.maintenanceid').
-				' AND maintenanceid='.$maintenanceid;
+				' AND maintenanceid='.zbx_dbstr($maintenanceid);
 	return DBfetch(DBselect($sql));
 }
 
