@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -59,7 +59,7 @@ function updateValueMap(array $valueMap, array $mappings) {
 	unset($valueMap['valuemapid']);
 
 	// check existence
-	if (!DBfetch(DBselect('SELECT v.valuemapid FROM valuemaps v WHERE v.valuemapid='.$valueMapId))) {
+	if (!DBfetch(DBselect('SELECT v.valuemapid FROM valuemaps v WHERE v.valuemapid='.zbx_dbstr($valueMapId)))) {
 		throw new Exception(_s('Value map with valuemapid "%1$s" does not exist.', $valueMapId));
 	}
 
@@ -227,8 +227,8 @@ function getValueMapMappings($valueMapId) {
 
 	$dbMappings = DBselect(
 		'SELECT m.mappingid,m.value,m.newvalue'.
-				' FROM mappings m'.
-				' WHERE m.valuemapid='.$valueMapId
+		' FROM mappings m'.
+		' WHERE m.valuemapid='.zbx_dbstr($valueMapId)
 	);
 	while ($mapping = DBfetch($dbMappings)) {
 		$mappings[$mapping['mappingid']] = $mapping;
@@ -237,6 +237,40 @@ function getValueMapMappings($valueMapId) {
 	return $mappings;
 }
 
+/**
+ * Get mapping for value.
+ * If there is no mapping return false.
+ *
+ * @param string $value	     value that mapping should be applied to
+ * @param int    $valueMapId value map id which should be used
+ *
+ * @return string|bool
+ */
+function getMappedValue($value, $valueMapId) {
+	static $valueMaps = array();
+
+	if ($valueMapId < 1) {
+		return false;
+	}
+
+	if (isset($valueMaps[$valueMapId][$value])) {
+		return $valueMaps[$valueMapId][$value];
+	}
+
+	$dbMappings = DBselect(
+		'SELECT m.newvalue'.
+			' FROM mappings m'.
+			' WHERE m.valuemapid='.zbx_dbstr($valueMapId).
+			' AND m.value='.zbx_dbstr($value)
+	);
+	if ($mapping = DBfetch($dbMappings)) {
+		$valueMaps[$valueMapId][$value] = $mapping['newvalue'];
+		return $mapping['newvalue'];
+	}
+
+	return false;
+}
+
 /**
  * Apply value mapping to value.
  * If value map or mapping is not found unchanged value returned,
@@ -248,24 +282,7 @@ function getValueMapMappings($valueMapId) {
  * @return string
  */
 function applyValueMap($value, $valueMapId) {
-	if ($valueMapId < 1) {
-		return $value;
-	}
+	$mapping = getMappedValue($value, $valueMapId);
 
-	static $valueMaps = array();
-	if (isset($valueMaps[$valueMapId][$value])) {
-		return $valueMaps[$valueMapId][$value];
-	}
-
-	$db_mappings = DBselect(
-		'SELECT m.newvalue'.
-			' FROM mappings m'.
-			' WHERE m.valuemapid='.$valueMapId.
-			' AND m.value='.zbx_dbstr($value)
-	);
-	if ($mapping = DBfetch($db_mappings)) {
-		$valueMaps[$valueMapId][$value] = $mapping['newvalue'].' '.'('.$value.')';
-		return $valueMaps[$valueMapId][$value];
-	}
-	return $value;
+	return $mapping === false ? $value : $mapping.' ('.$value.')';
 }
