@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,13 +9,14 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
+
 var cookie = {
 	cookies: [],
 
@@ -45,7 +46,7 @@ var cookie = {
 		}
 		document.cookie = name + '=' + value + expires + '; path=/';
 
-		// Apache header size limit
+		// apache header size limit
 		if (document.cookie.length > 8000) {
 			document.cookie = name + '=; path=/';
 			alert(locale['S_MAX_COOKIE_SIZE_REACHED']);
@@ -200,6 +201,7 @@ var cookie = {
  * http://www.gnu.org/licenses/gpl.html
  *
  */
+
 /**
  * Create a cookie with the given key and value and other optional parameters.
  *
@@ -246,31 +248,31 @@ var cookie = {
  * @author Klaus Hartl/klaus.hartl@stilbuero.de
  */
 jQuery.cookie = function (key, value, options) {
-    // key and value given, set cookie...
-    if (arguments.length > 1 && (value === null || typeof value !== "object")) {
-        options = jQuery.extend({}, options);
+	// key and value given, set cookie...
+	if (arguments.length > 1 && (value === null || typeof value !== 'object')) {
+		options = jQuery.extend({}, options);
 
-        if (value === null) {
-            options.expires = -1;
-        }
+		if (value === null) {
+			options.expires = -1;
+		}
 
-        if (typeof options.expires === 'number') {
-            var days = options.expires, t = options.expires = new Date();
-            t.setDate(t.getDate() + days);
-        }
+		if (typeof options.expires === 'number') {
+			var days = options.expires, t = options.expires = new Date();
+			t.setDate(t.getDate() + days);
+		}
 
-        return (document.cookie = [
-            encodeURIComponent(key), '=',
-            options.raw ? String(value) : encodeURIComponent(String(value)),
-            options.expires ? '; expires=' + options.expires.toUTCString() : '', // use expires attribute, max-age is not supported by IE
-            options.path ? '; path=' + options.path : '',
-            options.domain ? '; domain=' + options.domain : '',
-            options.secure ? '; secure' : ''
-        ].join(''));
-    }
+		return (document.cookie = [
+			encodeURIComponent(key), '=',
+			options.raw ? String(value) : encodeURIComponent(String(value)),
+			options.expires ? '; expires=' + options.expires.toUTCString() : '', // use expires attribute, max-age is not supported by IE
+			options.path ? '; path=' + options.path : '',
+			options.domain ? '; domain=' + options.domain : '',
+			options.secure ? '; secure' : ''
+		].join(''));
+	}
 
-    // key and possibly options given, get cookie...
-    options = value || {};
-    var result, decode = options.raw ? function (s) { return s; } : decodeURIComponent;
-    return (result = new RegExp('(?:^|; )' + encodeURIComponent(key) + '=([^;]*)').exec(document.cookie)) ? decode(result[1]) : null;
+	// key and possibly options given, get cookie...
+	options = value || {};
+	var result, decode = options.raw ? function (s) { return s; } : decodeURIComponent;
+	return (result = new RegExp('(?:^|; )' + encodeURIComponent(key) + '=([^;]*)').exec(document.cookie)) ? decode(result[1]) : null;
 };
