@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2012 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -32,30 +32,32 @@ var timeControl = {
 	timeRefreshTimeoutHandler: null,
 
 	addObject: function(id, time, objData) {
-		this.objectList[id] = {
-			id: id,
-			containerid: null,
-			refresh: false,
-			processed: 0,
-			time: {},
-			objDims: {},
-			src: location.href,
-			dynamic: 1,
-			periodFixed: 1,
-			loadSBox: 0,
-			loadImage: 0,
-			loadScroll: 0,
-			mainObject: 0, // object on changing will reflect on all others
-			sliderMaximumTimePeriod: null // max period in seconds
-		};
+		if (typeof this.objectList[id] === 'undefined') {
+			this.objectList[id] = {
+				id: id,
+				containerid: null,
+				refresh: false,
+				processed: 0,
+				time: {},
+				objDims: {},
+				src: location.href,
+				dynamic: 1,
+				periodFixed: 1,
+				loadSBox: 0,
+				loadImage: 0,
+				loadScroll: 0,
+				mainObject: 0, // object on changing will reflect on all others
+				sliderMaximumTimePeriod: null // max period in seconds
+			};
 
-		for (var key in this.objectList[id]) {
-			if (isset(key, objData)) {
-				this.objectList[id][key] = objData[key];
+			for (var key in this.objectList[id]) {
+				if (isset(key, objData)) {
+					this.objectList[id][key] = objData[key];
+				}
 			}
-		}
 
-		this.objectList[id].time = time;
+			this.objectList[id].time = time;
+		}
 	},
 
 	processObjects: function() {
@@ -166,6 +168,7 @@ var timeControl = {
 			$(obj.containerid).appendChild(img);
 		}
 
+		// apply sbox events to image
 		if (obj.loadSBox && empty(obj.sbox_listener)) {
 			obj.sbox_listener = this.addSBox.bindAsEventListener(this, id);
 			addListener(img, 'load', obj.sbox_listener);
@@ -1057,10 +1060,13 @@ var CScrollBar = Class.create(CDebug, {
 		this.dom.info_period.innerHTML = formatTimestamp(period, false, true);
 
 		// info left
-		this.dom.info_left.innerHTML = new CDate((usertime - period) * 1000).format(locale['S_DATE_FORMAT']);
+		var userStartTime = usertime - period;
+		this.dom.info_left.innerHTML = new CDate(userStartTime * 1000).format(locale['S_DATE_FORMAT']);
+		this.dom.info_left.setAttribute('data-timestamp', userStartTime);
 
 		// info right
 		var right_info = new CDate(usertime * 1000).format(locale['S_DATE_FORMAT']);
+		this.dom.info_right.setAttribute('data-timestamp', usertime);
 
 		if (timeControl.timeline.now()) {
 			right_info += ' (' + locale['S_NOW_SMALL'] + '!) ';
@@ -1298,7 +1304,8 @@ var CScrollBar = Class.create(CDebug, {
 		this.dom.info_left = document.createElement('span');
 		this.dom.timeline.appendChild(this.dom.info_left);
 		this.dom.info_left.className = 'info_left link';
-		this.dom.info_left.appendChild(document.createTextNode('02.07.2009 12:15:12'));
+		this.dom.info_left.appendChild(document.createTextNode('01.01.1970 00:00:00'));
+		this.dom.info_left.setAttribute('data-timestamp', 1);
 
 		var sep = document.createElement('span');
 		sep.className = 'info_sep1';
@@ -1309,7 +1316,8 @@ var CScrollBar = Class.create(CDebug, {
 		this.dom.info_right = document.createElement('span');
 		this.dom.timeline.appendChild(this.dom.info_right);
 		this.dom.info_right.className = 'info_right link';
-		this.dom.info_right.appendChild(document.createTextNode('02.07.2009 12:15:12'));
+		this.dom.info_right.appendChild(document.createTextNode('01.01.1970 00:00:00'));
+		this.dom.info_right.setAttribute('data-timestamp', 1);
 
 		// <sublevel>
 		this.dom.sublevel = document.createElement('div');
@@ -1558,7 +1566,6 @@ function sbox_init(id) {
 	addListener(document, 'mouseup', sboxGlobalMouseup);
 	addListener(document, 'mousemove', sboxGlobalMousemove);
 	ZBX_SBOX[id].addListeners();
-
 	return ZBX_SBOX[id];
 }
 
@@ -1614,8 +1621,10 @@ var sbox = Class.create(CDebug, {
 
 		if (IE) {
 			jQuery(this.grphobj).mousedown(jQuery.proxy(this.mouseDown, this));
-			this.grphobj.onmousemove = this.mouseMove.bindAsEventListener(this);
-			jQuery(this.containerId).find('a').attr('onclick', 'javascript: return ZBX_SBOX["' + this.sbox_id + '"].ieMouseClick();');
+			jQuery(this.grphobj).mousemove(jQuery.proxy(this.mouseMove, this));
+			jQuery('#flickerfreescreen_' + this.sbox_id + ' a').click(function() {
+				ZBX_SBOX[this.sbox_id].ieMouseClick();
+			});
 		}
 		else {
 			jQuery(this.dom_obj).mousedown(jQuery.proxy(this.mouseDown, this));
