@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -21,6 +21,7 @@
 if (!Array.prototype.indexOf) {
 	Array.prototype.indexOf = function (searchElement) {
 		'use strict';
+
 		if (this === void 0 || this === null) {
 			throw new TypeError();
 		}
@@ -73,7 +74,7 @@ var PageRefresh = {
 
 		this.delayLeft -= 1000;
 		if (this.delayLeft < 0) {
-			location.reload();
+			location.replace(location.href);
 		}
 		else {
 			this.timeout = setTimeout('PageRefresh.check()', 1000);
@@ -344,7 +345,7 @@ var chkbxRange = {
 		}
 	},
 
-	submitGo: function(e){
+	submitGo: function(e) {
 		e = e || window.event;
 
 		if (this.pageGoCount > 0) {
@@ -461,7 +462,7 @@ var AudioList = {
 		this.endLoop(audiofile);
 	},
 
-	stopAll: function(e){
+	stopAll: function(e) {
 		for (var name in this.list) {
 			if (empty(this.dom[name])) {
 				continue;
@@ -674,13 +675,17 @@ var jqBlink = {
 };
 
 /*
- * ZABBIX HintBoxes
+ * HintBox class.
  */
 var hintBox = {
 
 	createBox: function(e, target, hintText, width, className, isStatic) {
 		var box = jQuery('<div></div>').addClass('hintbox');
 
+		if (typeof hintText === 'string') {
+			hintText = hintText.replace(/\n/g, '<br />');
+		}
+
 		if (!empty(className)) {
 			box.append(jQuery('<span></span>').addClass(className).html(hintText));
 		}
@@ -711,12 +716,20 @@ var hintBox = {
 
 	HintWraper: function(e, target, hintText, width, className) {
 		target.isStatic = false;
-		jQuery(target).bind('mouseenter', function(e, d){
-			if (d) e = d;
+
+		jQuery(target).on('mouseenter', function(e, d) {
+			if (d) {
+				e = d;
+			}
 			hintBox.showHint(e, target, hintText, width, className, false);
-		}).bind('mouseleave', function(e){
+
+		}).on('mouseleave', function(e) {
 			hintBox.hideHint(e, target);
+
+		}).on('remove', function(e) {
+			hintBox.deleteHint(target);
 		});
+
 		jQuery(target).removeAttr('onmouseover');
 		jQuery(target).trigger('mouseenter', e);
 	},
@@ -724,11 +737,13 @@ var hintBox = {
 	showStaticHint: function(e, target, hint, width, className, resizeAfterLoad) {
 		var isStatic = target.isStatic;
 		hintBox.hideHint(e, target, true);
+
 		if (!isStatic) {
 			target.isStatic = true;
 			hintBox.showHint(e, target, hint, width, className, true);
+
 			if (resizeAfterLoad) {
-				hint.one('load', function(e){
+				hint.one('load', function(e) {
 					hintBox.positionHint(e, target);
 				});
 			}
@@ -736,11 +751,12 @@ var hintBox = {
 	},
 
 	showHint: function(e, target, hintText, width, className, isStatic) {
-		if (target.hintBoxItem) return;
+		if (target.hintBoxItem) {
+			return;
+		}
+
 		target.hintBoxItem = hintBox.createBox(e, target, hintText, width, className, isStatic);
-
 		hintBox.positionHint(e, target);
-
 		target.hintBoxItem.show();
 	},
 
@@ -785,7 +801,7 @@ var hintBox = {
 
 		// fallback if doesnt't fit verticaly but could fit if aligned to right or left
 		if ((top - scrollTop + target.hintBoxItem.height() > wHeight)
-			&& (target.clientX - 10 > target.hintBoxItem.width() || wWidth - target.clientX - 10 > target.hintBoxItem.width())) {
+				&& (target.clientX - 10 > target.hintBoxItem.width() || wWidth - target.clientX - 10 > target.hintBoxItem.width())) {
 
 			// align to left if fit
 			if (wWidth - target.clientX - 10 > target.hintBoxItem.width()) {
@@ -807,17 +823,25 @@ var hintBox = {
 		}
 
 		target.hintBoxItem.css({
-			'top': top + 'px',
-			'left': left + 'px',
-			'zIndex': '999'
+			top: top + 'px',
+			left: left + 'px',
+			zIndex: 100
 		});
 	},
 
 	hideHint: function(e, target, hideStatic) {
-		if (target.isStatic && !hideStatic) return;
+		if (target.isStatic && !hideStatic) {
+			return;
+		}
+
+		hintBox.deleteHint(target);
+	},
+
+	deleteHint: function(target) {
 		if (target.hintBoxItem) {
 			target.hintBoxItem.remove();
 			delete target.hintBoxItem;
+
 			if (target.isStatic) {
 				delete target.isStatic;
 			}
@@ -832,6 +856,7 @@ function hide_color_picker() {
 	if (!color_picker) {
 		return;
 	}
+
 	color_picker.style.zIndex = 1000;
 	color_picker.style.visibility = 'hidden';
 	color_picker.style.left = '-' + ((color_picker.style.width) ? color_picker.style.width : 100) + 'px';
@@ -843,6 +868,7 @@ function show_color_picker(id) {
 	if (!color_picker) {
 		return;
 	}
+
 	curr_lbl = document.getElementById('lbl_' + id);
 	curr_txt = document.getElementById(id);
 	var pos = getPosition(curr_lbl);
@@ -857,6 +883,7 @@ function create_color_picker() {
 	if (color_picker) {
 		return;
 	}
+
 	color_picker = document.createElement('div');
 	color_picker.setAttribute('id', 'color_picker');
 	color_picker.innerHTML = color_table;
@@ -974,6 +1001,7 @@ function changeHatStateUI(icon, divid) {
 		'favref': divid,
 		'favstate': hat_state
 	};
+
 	send_params(params);
 }
 
@@ -997,6 +1025,7 @@ function change_hat_state(icon, divid) {
 		'favref': divid,
 		'favstate': hat_state
 	};
+
 	send_params(params);
 }
 
@@ -1031,6 +1060,7 @@ function setRefreshRate(pmasterid, dollid, interval, params) {
 	params['favobj'] = 'set_rf_rate';
 	params['favref'] = dollid;
 	params['favcnt'] = interval;
+
 	send_params(params);
 }
 
@@ -1048,6 +1078,7 @@ function switch_mute(icon) {
 		'favref': 'sound',
 		'favstate': sound_state
 	};
+
 	send_params(params);
 }
 
