@@ -1,7 +1,7 @@
 <?php
 /*
-** ZABBIX
-** Copyright (C) 2000-2010 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,23 +10,23 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 ?>
 <?php
-require_once('include/config.inc.php');
+require_once dirname(__FILE__).'/include/config.inc.php';
 
-$page['title'] = "S_RESOURCE";
+$page['title'] = _('Resource');
 $page['file'] = 'popup_right.php';
 
 define('ZBX_PAGE_NO_MENU', 1);
 
-include_once('include/page_header.php');
+require_once dirname(__FILE__).'/include/page_header.php';
 ?>
 <?php
 //		VAR			TYPE	OPTIONAL FLAGS	VALIDATION	EXCEPTION
@@ -53,17 +53,17 @@ check_fields($fields);
 		$available_nodes = get_accessible_nodes_by_user($USER_DETAILS, PERM_READ_ONLY, PERM_RES_IDS_ARRAY);
 
 		$cmbResourceNode = new CComboBox('nodeid',$nodeid,'submit();');
-		$cmbResourceNode->addItem(0, S_ALL_S);
+		$cmbResourceNode->addItem(0, _('All'));
 
 		$sql = 'SELECT name,nodeid '.
 			' FROM nodes '.
-			' WHERE '.DBcondition('nodeid',$available_nodes);
+			' WHERE '.dbConditionInt('nodeid', $available_nodes);
 		$db_nodes = DBselect($sql);
 		while($node = DBfetch($db_nodes)){
 			$cmbResourceNode->addItem($node['nodeid'], $node['name']);
 		}
 
-		$frmTitle->addItem(array(S_NODE, SPACE, $cmbResourceNode));
+		$frmTitle->addItem(array(_('Node'), SPACE, $cmbResourceNode));
 	}
 
 	show_table_header(permission2str($permission),$frmTitle);
@@ -71,8 +71,8 @@ check_fields($fields);
 	$form = new CForm();
 	$form->setAttribute('id', 'groups');
 
-	$table = new CTableInfo(S_NO_RESOURCES_DEFINED);
-	$table->setHeader(new CCol(array(new CCheckBox('all_groups', NULL, 'check_all(this.checked)'),S_NAME)));
+	$table = new CTableInfo(_('No resources defined.'));
+	$table->setHeader(new CCol(array(new CCheckBox('all_groups', NULL, 'check_all(this.checked)'),_('Name'))));
 
 // NODES
 	if($nodeid == 0) $nodeids = get_current_nodeid(true);
@@ -85,7 +85,7 @@ check_fields($fields);
 		'nodeids' => $nodeids,
 		'output' => API_OUTPUT_EXTEND
 	);
-	$groups = CHostGroup::get($options);
+	$groups = API::HostGroup()->get($options);
 	foreach($groups as $gnum => $row){
 		$groups[$gnum]['nodename'] = get_node_name_by_elid($row['groupid'], true, ':').$row['name'];
 		if($nodeid == 0) $groups[$gnum]['name'] = $groups[$gnum]['nodename'];
@@ -106,8 +106,7 @@ check_fields($fields);
 
 	insert_js('var grouplist = '.zbx_jsvalue($grouplist).';');
 
-	$button = new CButton('select', S_SELECT, 'add_groups("'.$dstfrm.'")');
-	$button->setType('button');
+	$button = new CButton('select', _('Select'), 'add_groups("'.$dstfrm.'")');
 	$table->setFooter(new CCol($button,'right'));
 
 	$form->addItem($table);
@@ -140,6 +139,6 @@ function check_all(value) {
 </script>
 <?php
 
-include_once('include/page_footer.php');
+require_once dirname(__FILE__).'/include/page_footer.php';
 
 ?>
