@@ -1,7 +1,7 @@
 <?php
 /*
-** ZABBIX
-** Copyright (C) 2000-2009 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -10,19 +10,19 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; ifnot, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 ?>
 <?php
-include_once('include/config.inc.php');
-require_once('include/triggers.inc.php');
+require_once dirname(__FILE__).'/include/config.inc.php';
+require_once dirname(__FILE__).'/include/triggers.inc.php';
 
-$page['title'] = S_TEST;
+$page['title'] = _('Test');
 $page['file'] = 'tr_testexpr.php';
 
 define('ZBX_PAGE_NO_MENU', 1);
@@ -30,20 +30,20 @@ define('COMBO_PATTERN', 'str_in_array({},array(');
 define('COMBO_PATTERN_LENGTH', zbx_strlen(COMBO_PATTERN));
 
 $definedErrorPhrases = array(
-			EXPRESSION_VALUE_TYPE_UNKNOWN => S_EXPRESSION_VALUE_TYPE_UNKNOWN,
-			EXPRESSION_HOST_UNKNOWN => S_EXPRESSION_HOST_UNKNOWN,
-			EXPRESSION_HOST_ITEM_UNKNOWN => S_EXPRESSION_HOST_ITEM_UNKNOWN,
-			EXPRESSION_NOT_A_MACRO_ERROR => S_EXPRESSION_NOT_A_MACRO_ERROR,
-			EXPRESSION_FUNCTION_UNKNOWN => S_INCORRECT_FUNCTION_IS_USED
+	EXPRESSION_VALUE_TYPE_UNKNOWN => _('Unknown variable type, testing not available'),
+	EXPRESSION_HOST_UNKNOWN => _('Unknown host, no such host present in system'),
+	EXPRESSION_HOST_ITEM_UNKNOWN => _('Unknown host item, no such item in selected host'),
+	EXPRESSION_NOT_A_MACRO_ERROR => _('Given expression is not a macro'),
+	EXPRESSION_FUNCTION_UNKNOWN => _('Incorrect function is used')
 );
 
-include_once('include/page_header.php');
+require_once dirname(__FILE__).'/include/page_header.php';
 ?>
 <?php
 //----------------------------------------------------------------------
 
 // expression analyze
-	$expression = urldecode(get_request('expression', ''));
+	$expression = get_request('expression', '');
 
 	define('NO_LINK_IN_TESTING', true);
 	list($outline, $eHTMLTree) = analyzeExpression($expression);
@@ -53,7 +53,7 @@ include_once('include/page_header.php');
 	$data_table = new CTable(null, 'tableinfo');
 	$data_table->setAttribute('id', 'data_list');
 
-	$data_table->setHeader(array(S_EXPRESSION_VARIABLE_ELEMENTS, S_RESULT_TYPE, S_VALUE));
+	$data_table->setHeader(array(_('Expression Variable Elements'), _('Result type'), _('Value')));
 
 	$octet = false;
 	$datas = array();
@@ -77,25 +77,28 @@ include_once('include/page_header.php');
 
 			$info = get_item_function_info($exprPart['expression']);
 
-			$octet = ($info['value_type'] == 'HHMMSS');
-
-			$validation = $info['validation'];
-			if(substr($validation, 0, COMBO_PATTERN_LENGTH) == COMBO_PATTERN){
-				$vals = explode(',', substr($validation, COMBO_PATTERN_LENGTH, zbx_strlen($validation) - COMBO_PATTERN_LENGTH - 4));
-
-				$control = new CComboBox($fname, $macrosData[$exprPart['expression']]);
-				foreach ($vals as $v) $control->addItem($v, $v);
-			}
-			else
-				$control = new CTextBox($fname, $macrosData[$exprPart['expression']], 30);
-
-			if(!is_array($info) && isset($definedErrorPhrases[$info])) {
-				$control->setAttribute('disabled', 'disabled');
+			if (!is_array($info) && isset($definedErrorPhrases[$info])) {
 				$allowedTesting = false;
+				$control = new CTextBox($fname, $macrosData[$exprPart['expression']], 30);
+				$control->setAttribute('disabled', 'disabled');
+			}
+			else {
+				$octet = ($info['value_type'] == 'HHMMSS');
+				$validation = $info['validation'];
+				if(substr($validation, 0, COMBO_PATTERN_LENGTH) == COMBO_PATTERN){
+					$vals = explode(',', substr($validation, COMBO_PATTERN_LENGTH, zbx_strlen($validation) - COMBO_PATTERN_LENGTH - 4));
+					$control = new CComboBox($fname, $macrosData[$exprPart['expression']]);
+					foreach ($vals as $v) {
+						$control->addItem($v, $v);
+					}
+				}
+				else {
+					$control = new CTextBox($fname, $macrosData[$exprPart['expression']], 30);
+				}
+				$fields[$fname] = array($info['type'], O_OPT, null, $validation, 'isset({test_expression})', $exprPart['expression']);
 			}
 
 			$data_table->addRow(new CRow(array($exprPart['expression'], (is_array($info) || !isset($definedErrorPhrases[$info])) ? $info['value_type'] : new CCol($definedErrorPhrases[$info], 'disaster'), $control)));
-			$fields[$fname] = array($info['type'], O_OPT, null, $validation, 'isset({test_expression})', $exprPart['expression']);
 		}
 	}
 //---------------------------------- CHECKS ------------------------------------
@@ -115,29 +118,25 @@ include_once('include/page_header.php');
 
 //------------------------ <FORM> ---------------------------
 
-	$frm_test = new CFormTable(S_TEST, 'tr_testexpr.php');
+	$frm_test = new CFormTable(_('Test'), 'tr_testexpr.php');
 	$frm_test->setHelp('web.testexpr.service.php');
 	$frm_test->setTableClass('formlongtable formtable');
 	$frm_test->addVar('form_refresh', get_request('form_refresh', 1));
-	$frm_test->addVar('expression', urlencode($expression));
+	$frm_test->addVar('expression', $expression);
 
 /* test data */
-	$frm_test->addRow(S_TEST_DATA, $data_table);
+	$frm_test->addRow(_('Test data'), $data_table);
 
 /* result */
-	$res_table = new CTable();
-	$res_table->setClass('tableinfo');
+	$res_table = new CTable(null, 'tableinfo');
 	$res_table->setAttribute('id', 'result_list');
 	$res_table->setOddRowClass('even_row');
 	$res_table->setEvenRowClass('even_row');
-	$res_table->setHeader(array(S_EXPRESSION, S_RESULT));
+	$res_table->setHeader(array(_('Expression'), _('Result')));
 
 	ksort($rplcts, SORT_NUMERIC);
 
-	//$exprs = make_disp_tree($tree, $map);
 	foreach($eHTMLTree as $e){
-		//if(!isset($e['expression']))
-			//continue;
 		$result = '-';
 		if($allowedTesting && $test && isset($e['expression'])){
 			$result = evalExpressionData($e['expression']['value'], $macrosData, $octet);
@@ -165,17 +164,15 @@ include_once('include/page_header.php');
 	$col->setAttribute('style', $style);
 	$res_table->setFooter(array($outline, $col), $res_table->headerClass);
 
-	$frm_test->addRow(S_RESULT, $res_table);
+	$frm_test->addRow(_('Result'), $res_table);
 
 // action buttons
-	$btn_test = new CButton('test_expression', S_TEST);
+	$btn_test = new CSubmit('test_expression', _('Test'));
 	if(!$allowedTesting) $btn_test->setAttribute('disabled', 'disabled');
 	$frm_test->addItemToBottomRow($btn_test);
 	$frm_test->addItemToBottomRow(SPACE);
 
-	$btn_close = new CButton('close', S_CLOSE);
-	$btn_close->setType('button');
-	$btn_close->setAction('javascript: self.close();');
+	$btn_close = new CButton('close', _('Close'),'javascript: self.close();');
 	$frm_test->addItemToBottomRow($btn_close);
 
 	$frm_test->show();
@@ -184,6 +181,6 @@ include_once('include/page_header.php');
 ?>
 <?php
 
-include_once('include/page_footer.php');
+require_once dirname(__FILE__).'/include/page_footer.php';
 
 ?>
