@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2010 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #ifndef ZABBIX_COMMON_H
@@ -22,6 +22,7 @@
 
 #include "sysinc.h"
 #include "zbxtypes.h"
+#include "version.h"
 
 #ifdef DEBUG
 #	include "threads.h"
@@ -120,11 +121,6 @@
 #define ON	1
 #define OFF	0
 
-#define	APPLICATION_NAME	"Zabbix Agent"
-#define	ZABBIX_REVDATE		"26 July 2013"
-#define	ZABBIX_VERSION		"1.8.17"
-#define	ZABBIX_REVISION		"37338"
-
 #if defined(_WINDOWS)
 #	define	ZBX_SERVICE_NAME_LEN	64
 extern char ZABBIX_SERVICE_NAME[ZBX_SERVICE_NAME_LEN];
@@ -139,16 +135,27 @@ extern char ZABBIX_EVENT_SOURCE[ZBX_SERVICE_NAME_LEN];
 #define	NETWORK_ERROR	-3
 #define	TIMEOUT_ERROR	-4
 #define	AGENT_ERROR	-5
+#define	GATEWAY_ERROR	-6
+
+#define SUCCEED_OR_FAIL(result) (FAIL != (result) ? SUCCEED : FAIL)
 const char	*zbx_result_string(int result);
 
 #define MAX_ID_LEN		21
 #define MAX_STRING_LEN		2048
 #define MAX_BUFFER_LEN		65536
 #define MAX_ZBX_HOSTNAME_LEN	64
+#define MAX_EXECUTE_OUTPUT_LEN	(512 * ZBX_KIBIBYTE)
 
 #define ZBX_MAX_UINT64_LEN	21
 #define ZBX_DM_DELIMITER	'\255'
 
+typedef struct
+{
+	int	sec;	/* seconds */
+	int	ns;	/* nanoseconds */
+}
+zbx_timespec_t;
+
 /* item types */
 typedef enum
 {
@@ -167,8 +174,31 @@ typedef enum
 	ITEM_TYPE_IPMI,
 	ITEM_TYPE_SSH,
 	ITEM_TYPE_TELNET,
-	ITEM_TYPE_CALCULATED
-} zbx_item_type_t;
+	ITEM_TYPE_CALCULATED,
+	ITEM_TYPE_JMX,
+	ITEM_TYPE_SNMPTRAP	/* 17 */
+}
+zbx_item_type_t;
+const char	*zbx_host_type_string(zbx_item_type_t item_type);
+
+typedef enum
+{
+	INTERFACE_TYPE_UNKNOWN = 0,
+	INTERFACE_TYPE_AGENT,
+	INTERFACE_TYPE_SNMP,
+	INTERFACE_TYPE_IPMI,
+	INTERFACE_TYPE_JMX,
+	INTERFACE_TYPE_ANY = 255
+}
+zbx_interface_type_t;
+const char	*zbx_interface_type_string(zbx_interface_type_t type);
+
+#define INTERFACE_TYPE_COUNT	4	/* number of interface types */
+extern const int	INTERFACE_TYPE_PRIORITY[INTERFACE_TYPE_COUNT];
+
+#define ZBX_FLAG_DISCOVERY		0x01	/* low-level discovery rule */
+#define ZBX_FLAG_DISCOVERY_CHILD	0x02	/* low-level discovery proto-item, proto-trigger or proto-graph */
+#define ZBX_FLAG_DISCOVERY_CREATED	0x04	/* auto-created item, trigger or graph */
 
 typedef enum
 {
@@ -176,7 +206,7 @@ typedef enum
 	ITEM_AUTHTYPE_PUBLICKEY
 } zbx_item_authtype_t;
 
-/* Event sources */
+/* event sources */
 typedef enum
 {
 	EVENT_SOURCE_TRIGGERS = 0,
@@ -184,7 +214,7 @@ typedef enum
 	EVENT_SOURCE_AUTO_REGISTRATION
 } zbx_event_source_t;
 
-/* Event objects */
+/* event objects */
 typedef enum
 {
 /* EVENT_SOURCE_TRIGGERS */
@@ -204,7 +234,7 @@ typedef enum
 	DOBJECT_STATUS_LOST
 } zbx_dstatus_t;
 
-/* Item value types */
+/* item value types */
 typedef enum
 {
 	ITEM_VALUE_TYPE_FLOAT = 0,
@@ -215,22 +245,26 @@ typedef enum
 } zbx_item_value_type_t;
 const char	*zbx_item_value_type_string(zbx_item_value_type_t value_type);
 
-/* Item data types */
+typedef union
+{
+	double		dbl;
+	zbx_uint64_t	ui64;
+	char		*str;
+	char		*err;
+}
+history_value_t;
+
+/* item data types */
 typedef enum
 {
 	ITEM_DATA_TYPE_DECIMAL = 0,
 	ITEM_DATA_TYPE_OCTAL,
-	ITEM_DATA_TYPE_HEXADECIMAL
+	ITEM_DATA_TYPE_HEXADECIMAL,
+	ITEM_DATA_TYPE_BOOLEAN
 } zbx_item_data_type_t;
+const char	*zbx_item_data_type_string(zbx_item_data_type_t data_type);
 
-/* HTTP test states */
-typedef enum
-{
-	HTTPTEST_STATE_IDLE = 0,
-	HTTPTEST_STATE_BUSY
-} zbx_httptest_state_type_t;
-
-/* Service supported by discoverer */
+/* service supported by discoverer */
 typedef enum
 {
 	SVC_SSH = 0,
@@ -246,20 +280,22 @@ typedef enum
 	SVC_SNMPv1,
 	SVC_SNMPv2c,
 	SVC_ICMPPING,
-	SVC_SNMPv3
+	SVC_SNMPv3,
+	SVC_HTTPS,
+	SVC_TELNET
 } zbx_dservice_type_t;
 const char	*zbx_dservice_type_string(zbx_dservice_type_t service);
 
-/* Item snmpv3 security levels */
+/* item snmpv3 security levels */
 #define ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV	0
 #define ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV	1
 #define ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV	2
 
-/* Item multiplier types */
+/* item multiplier types */
 #define ITEM_MULTIPLIER_DO_NOT_USE		0
 #define ITEM_MULTIPLIER_USE			1
 
-/* Item delta types */
+/* item delta types */
 typedef enum
 {
 	ITEM_STORE_AS_IS = 0,
@@ -267,19 +303,19 @@ typedef enum
 	ITEM_STORE_SIMPLE_CHANGE
 } zbx_item_store_type_t;
 
-/* Object types for operations */
+/* object types for operations */
 #define OPERATION_OBJECT_USER	0
 #define OPERATION_OBJECT_GROUP	1
 
-/* Condition types */
+/* condition evaluation types */
 typedef enum
 {
 	ACTION_EVAL_TYPE_AND_OR	= 0,
 	ACTION_EVAL_TYPE_AND,
 	ACTION_EVAL_TYPE_OR
-}	zbx_action_eval_type_t;
+} zbx_action_eval_type_t;
 
-/* Condition types */
+/* condition types */
 typedef enum
 {
 	CONDITION_TYPE_HOST_GROUP = 0,
@@ -307,7 +343,7 @@ typedef enum
 	CONDITION_TYPE_HOST_NAME
 } zbx_condition_type_t;
 
-/* Condition operators */
+/* condition operators */
 typedef enum
 {
 	CONDITION_OPERATOR_EQUAL = 0,
@@ -369,21 +405,22 @@ typedef enum
 	AUDIT_RESOURCE_REGEXP
 } zbx_auditlog_resourcetype_t;
 
-/* special item key used for storing server status */
-#define SERVER_STATUS_KEY	"status"
 /* special item key used for ICMP pings */
 #define SERVER_ICMPPING_KEY	"icmpping"
 /* special item key used for ICMP ping latency */
 #define SERVER_ICMPPINGSEC_KEY	"icmppingsec"
 /* special item key used for ICMP ping loss packages */
 #define SERVER_ICMPPINGLOSS_KEY	"icmppingloss"
-/* special item key used for internal Zabbix log */
-#define SERVER_ZABBIXLOG_KEY	"zabbix[log]"
 
 /* runtime control options */
 #define ZBX_CONFIG_CACHE_RELOAD	"config_cache_reload"
 
-/* Media types */
+/* value for not supported items */
+#define ZBX_NOTSUPPORTED	"ZBX_NOTSUPPORTED"
+/* Zabbix Agent non-critical error */
+#define ZBX_ERROR		"ZBX_ERROR"
+
+/* media types */
 typedef enum
 {
 	MEDIA_TYPE_EMAIL = 0,
@@ -393,33 +430,35 @@ typedef enum
 	MEDIA_TYPE_EZ_TEXTING = 100
 } zbx_media_type_t;
 
-/* Alert statuses */
+/* alert statuses */
 typedef enum
 {
 	ALERT_STATUS_NOT_SENT = 0,
 	ALERT_STATUS_SENT,
 	ALERT_STATUS_FAILED
 } zbx_alert_status_t;
+const char	*zbx_alert_status_string(unsigned char type, unsigned char status);
 
-/* Escalation statuses */
+/* escalation statuses */
 typedef enum
 {
 	ESCALATION_STATUS_ACTIVE = 0,
-	ESCALATION_STATUS_RECOVERY,
+	ESCALATION_STATUS_RECOVERY,	/* only in server code, never in DB */
 	ESCALATION_STATUS_SLEEP,
-	ESCALATION_STATUS_COMPLETED, /* only in server code, never in DB */
-	ESCALATION_STATUS_SUPERSEDED_ACTIVE,
-	ESCALATION_STATUS_SUPERSEDED_RECOVERY
-} zbx_escalation_status_t;
+	ESCALATION_STATUS_COMPLETED	/* only in server code, never in DB */
+}
+zbx_escalation_status_t;
+const char      *zbx_escalation_status_string(unsigned char status);
 
-/* Alert types */
+/* alert types */
 typedef enum
 {
 	ALERT_TYPE_MESSAGE = 0,
 	ALERT_TYPE_COMMAND
 } zbx_alert_type_t;
+const char	*zbx_alert_type_string(unsigned char type);
 
-/* Item statuses */
+/* item statuses */
 typedef enum
 {
 	ITEM_STATUS_ACTIVE = 0,
@@ -430,14 +469,14 @@ typedef enum
 /*ITEM_STATUS_NOTAVAILABLE	5*/
 } zbx_item_status_t;
 
-/* Trigger types */
+/* trigger types */
 typedef enum
 {
 	TRIGGER_TYPE_NORMAL = 0,
 	TRIGGER_TYPE_MULTIPLE_TRUE
 } zbx_trigger_type_t;
 
-/* GROUP statuses */
+/* group statuses */
 typedef enum
 {
        GROUP_STATUS_ACTIVE = 0,
@@ -483,15 +522,15 @@ typedef enum
 	ZBX_CASE_SENSITIVE
 } zbx_case_sensitive_t;
 
-/* HTTP Tests statuses */
+/* HTTP tests statuses */
 #define HTTPTEST_STATUS_MONITORED	0
 #define HTTPTEST_STATUS_NOT_MONITORED	1
 
-/* Discovery rule */
+/* discovery rule */
 #define DRULE_STATUS_MONITORED		0
 #define DRULE_STATUS_NOT_MONITORED	1
 
-/* Host statuses */
+/* host statuses */
 #define HOST_STATUS_MONITORED		0
 #define HOST_STATUS_NOT_MONITORED	1
 /*#define HOST_STATUS_UNREACHABLE	2*/
@@ -500,35 +539,44 @@ typedef enum
 #define HOST_STATUS_PROXY_ACTIVE	5
 #define HOST_STATUS_PROXY_PASSIVE	6
 
-/* Host maintenance status */
+/* host maintenance status */
 #define HOST_MAINTENANCE_STATUS_OFF	0
 #define HOST_MAINTENANCE_STATUS_ON	1
 
-/* Host availability */
+/* host inventory mode */
+#define HOST_INVENTORY_MANUAL		0
+#define HOST_INVENTORY_AUTOMATIC	1
+
+/* host availability */
 #define HOST_AVAILABLE_UNKNOWN	0
 #define HOST_AVAILABLE_TRUE	1
 #define HOST_AVAILABLE_FALSE	2
 
-/* Trigger statuses */
+/* trigger statuses */
 #define TRIGGER_STATUS_ENABLED	0
 #define TRIGGER_STATUS_DISABLED	1
 
-/* Trigger values */
+/* trigger values */
 #define TRIGGER_VALUE_FALSE	0
 #define TRIGGER_VALUE_TRUE	1
-#define TRIGGER_VALUE_UNKNOWN	2
+#define TRIGGER_VALUE_UNKNOWN	2 /* only in "events" table */
 
-/* Trigger severity */
-typedef enum
-{
-	TRIGGER_SEVERITY_NOT_CLASSIFIED = 0,
-	TRIGGER_SEVERITY_INFORMATION,
-	TRIGGER_SEVERITY_WARNING,
-	TRIGGER_SEVERITY_AVERAGE,
-	TRIGGER_SEVERITY_HIGH,
-	TRIGGER_SEVERITY_DISASTER
-} zbx_trigger_severity_t;
-const char	*zbx_trigger_severity_string(zbx_trigger_severity_t severity);
+/* trigger value flags */
+#define TRIGGER_VALUE_FLAG_NORMAL	0
+#define TRIGGER_VALUE_FLAG_UNKNOWN	1
+
+/* trigger value change flags */
+#define TRIGGER_VALUE_CHANGED_NO	0
+#define TRIGGER_VALUE_CHANGED_YES	1
+
+/* trigger severity */
+#define TRIGGER_SEVERITY_NOT_CLASSIFIED	0
+#define TRIGGER_SEVERITY_INFORMATION	1
+#define TRIGGER_SEVERITY_WARNING	2
+#define TRIGGER_SEVERITY_AVERAGE	3
+#define TRIGGER_SEVERITY_HIGH		4
+#define TRIGGER_SEVERITY_DISASTER	5
+#define TRIGGER_SEVERITY_COUNT		6	/* number of trigger severities */
 
 typedef enum
 {
@@ -539,18 +587,23 @@ typedef enum
 	ITEM_LOGTYPE_SUCCESS_AUDIT
 } zbx_item_logtype_t;
 const char	*zbx_item_logtype_string(zbx_item_logtype_t logtype);
-/* Media statuses */
+
+/* media statuses */
 #define MEDIA_STATUS_ACTIVE	0
 #define MEDIA_STATUS_DISABLED	1
 
-/* Action statuses */
+/* action statuses */
 #define ACTION_STATUS_ACTIVE	0
 #define ACTION_STATUS_DISABLED	1
 
-/* Max number of retries for alerts */
+/* max number of retries for alerts */
 #define ALERT_MAX_RETRIES	3
 
-/* Operation types */
+/* media type statuses */
+#define MEDIA_TYPE_STATUS_ACTIVE	0
+#define MEDIA_TYPE_STATUS_DISABLED	1
+
+/* operation types */
 #define OPERATION_TYPE_MESSAGE		0
 #define OPERATION_TYPE_COMMAND		1
 #define OPERATION_TYPE_HOST_ADD		2
@@ -562,38 +615,28 @@ const char	*zbx_item_logtype_string(zbx_item_logtype_t logtype);
 #define OPERATION_TYPE_HOST_ENABLE	8
 #define OPERATION_TYPE_HOST_DISABLE	9
 
-/* Algorithms for service status calculation */
+/* algorithms for service status calculation */
 #define SERVICE_ALGORITHM_NONE	0
 #define SERVICE_ALGORITHM_MAX	1
 #define SERVICE_ALGORITHM_MIN	2
 
-/* Types of nodes check sums */
+/* types of nodes check sums */
 #define	NODE_CKSUM_TYPE_OLD	0
 #define	NODE_CKSUM_TYPE_NEW	1
 
-/* Types of operation in config log */
+/* types of operation in config log */
 #define	NODE_CONFIGLOG_OP_UPDATE	0
 #define	NODE_CONFIGLOG_OP_ADD		1
 #define	NODE_CONFIGLOG_OP_DELETE	2
 
-#define	ZBX_TYPE_INT	0
-#define	ZBX_TYPE_CHAR	1
-#define	ZBX_TYPE_FLOAT	2
-#define	ZBX_TYPE_BLOB	3
-#define	ZBX_TYPE_TEXT	4
-#define	ZBX_TYPE_UINT	5
-#define	ZBX_TYPE_ID	6
-
 /* HTTP item types */
-typedef enum
-{
-	ZBX_HTTPITEM_TYPE_RSPCODE = 0,
-	ZBX_HTTPITEM_TYPE_TIME,
-	ZBX_HTTPITEM_TYPE_SPEED,
-	ZBX_HTTPITEM_TYPE_LASTSTEP
-} zbx_httpitem_type_t;
+#define ZBX_HTTPITEM_TYPE_RSPCODE	0
+#define ZBX_HTTPITEM_TYPE_TIME		1
+#define ZBX_HTTPITEM_TYPE_SPEED		2
+#define ZBX_HTTPITEM_TYPE_LASTSTEP	3
+#define ZBX_HTTPITEM_TYPE_LASTERROR	4
 
-/* User permissions */
+/* user permissions */
 typedef enum
 {
 	USER_TYPE_ZABBIX_USER = 1,
@@ -612,13 +655,33 @@ typedef enum
 
 const char	*zbx_permission_string(int perm);
 
-/* Flags */
-#define	ZBX_SYNC		0x01
-#define ZBX_NOTNULL		0x02
-#define ZBX_HISTORY		0x04
-#define ZBX_HISTORY_SYNC	0x08
-#define ZBX_HISTORY_TRENDS	0x10
-#define ZBX_PROXY		0x20
+#define	ZBX_NODE_SLAVE	0
+#define	ZBX_NODE_MASTER	1
+const char	*zbx_nodetype_string(unsigned char nodetype);
+
+typedef struct
+{
+	unsigned char	type;
+	unsigned char	execute_on;
+	char		*port;
+	unsigned char	authtype;
+	char		*username;
+	char		*password;
+	char		*publickey;
+	char		*privatekey;
+	char		*command;
+	zbx_uint64_t	scriptid;
+}
+zbx_script_t;
+
+#define ZBX_SCRIPT_TYPE_CUSTOM_SCRIPT	0
+#define ZBX_SCRIPT_TYPE_IPMI		1
+#define ZBX_SCRIPT_TYPE_SSH		2
+#define ZBX_SCRIPT_TYPE_TELNET		3
+#define ZBX_SCRIPT_TYPE_GLOBAL_SCRIPT	4
+
+#define ZBX_SCRIPT_EXECUTE_ON_AGENT	0
+#define ZBX_SCRIPT_EXECUTE_ON_SERVER	1
 
 #define POLLER_DELAY		5
 #define DISCOVERER_DELAY	60
@@ -638,24 +701,36 @@ const char	*zbx_permission_string(int perm);
 #define zbx_realloc(src, size)		zbx_realloc2(__FILE__, __LINE__, src, size)
 #define zbx_strdup(old, str)		zbx_strdup2(__FILE__, __LINE__, old, str)
 
+#define ZBX_STRDUP(var, str)	(var = zbx_strdup(var, str))
+
 void    *zbx_calloc2(const char *filename, int line, void *old, size_t nmemb, size_t size);
 void    *zbx_malloc2(const char *filename, int line, void *old, size_t size);
-void    *zbx_realloc2(const char *filename, int line, void *src, size_t size);
+void    *zbx_realloc2(const char *filename, int line, void *old, size_t size);
 char    *zbx_strdup2(const char *filename, int line, char *old, const char *str);
 
 #define zbx_free(ptr)		\
+				\
+do				\
+{				\
 	if (ptr)		\
 	{			\
 		free(ptr);	\
 		ptr = NULL;	\
-	}
+	}			\
+}				\
+while (0)
 
 #define zbx_fclose(file)	\
+				\
+do				\
+{				\
 	if (file)		\
 	{			\
 		fclose(file);	\
 		file = NULL;	\
-	}
+	}			\
+}				\
+while (0)
 
 #define THIS_SHOULD_NEVER_HAPPEN	zbx_error("ERROR [file:%s,line:%d] "				\
 							"Something impossible has just happened.",	\
@@ -673,8 +748,8 @@ void	help();
 void	usage();
 void	version();
 
-/* MAX Length of base64 data */
-#define ZBX_MAX_B64_LEN 16*1024
+/* max length of base64 data */
+#define ZBX_MAX_B64_LEN 16 * 1024
 
 const char	*get_program_name(const char *path);
 
@@ -711,28 +786,27 @@ ZBX_TASK_EX;
 
 char	*string_replace(const char *str, const char *sub_str1, const char *sub_str2);
 
-int	is_double_prefix(const char *str);
+int	is_double_suffix(const char *str);
 int	is_double(const char *c);
-int	is_uint_prefix(const char *c);
+int	is_uint_suffix(const char *c, unsigned int *value);
 int	is_uint(const char *c);
 int	is_int_prefix(const char *c);
 #define is_uint64(src, value)	is_uint64_n(src, ZBX_MAX_UINT64_LEN, value)
 int	is_uint64_n(const char *str, size_t n, zbx_uint64_t *value);
 int	is_ushort(const char *str, unsigned short *value);
+int	is_boolean(const char *str, zbx_uint64_t *value);
 int	is_uoct(const char *str);
 int	is_uhex(const char *str);
 int	is_hex_string(const char *str);
 int	is_ascii_string(const char *str);
 int	zbx_rtrim(char *str, const char *charlist);
 void	zbx_ltrim(char *str, const char *charlist);
+void	zbx_lrtrim(char *str, const char *charlist);
 void	zbx_remove_chars(register char *str, const char *charlist);
 #define ZBX_WHITESPACE			" \t\r\n"
 #define zbx_remove_spaces(str)		zbx_remove_chars(str, " ")
 #define zbx_remove_whitespace(str)	zbx_remove_chars(str, ZBX_WHITESPACE)
 void	compress_signs(char *str);
-void	ltrim_spaces(char *c);
-void	rtrim_spaces(char *c);
-void	lrtrim_spaces(char *c);
 void	del_zeroes(char *s);
 int	get_param(const char *param, int num, char *buf, size_t max_len);
 int	num_param(const char *param);
@@ -741,21 +815,22 @@ void	remove_param(char *param, int num);
 const char	*get_string(const char *p, char *buf, size_t bufsize);
 int	get_key_param(char *param, int num, char *buf, size_t max_len);
 int	num_key_param(char *param);
+size_t	zbx_get_escape_string_len(const char *src, const char *charlist);
 char	*zbx_dyn_escape_string(const char *src, const char *charlist);
-int	calculate_item_nextcheck(zbx_uint64_t itemid, int item_type, int delay,
-		const char *delay_flex, time_t now, int *effective_delay);
+int	calculate_item_nextcheck(zbx_uint64_t interfaceid, zbx_uint64_t itemid, int item_type,
+		int delay, const char *flex_intervals, time_t now, int *effective_delay);
 time_t	calculate_proxy_nextcheck(zbx_uint64_t hostid, unsigned int delay, time_t now);
 int	check_time_period(const char *period, time_t now);
 char	zbx_num2hex(u_char c);
 u_char	zbx_hex2num(char c);
-int	zbx_binary2hex(const u_char *input, int ilen, char **output, int *olen);
-int     zbx_hex2binary(char *io);
+size_t	zbx_binary2hex(const u_char *input, size_t ilen, char **output, size_t *olen);
+size_t	zbx_hex2binary(char *io);
 void	zbx_hex2octal(const char *input, char **output, int *olen);
 #ifdef HAVE_POSTGRESQL
-int	zbx_pg_escape_bytea(const u_char *input, int ilen, char **output, int *olen);
-int	zbx_pg_unescape_bytea(u_char *io);
+size_t	zbx_pg_escape_bytea(const u_char *input, size_t ilen, char **output, size_t *olen);
+size_t	zbx_pg_unescape_bytea(u_char *io);
 #endif
-int	zbx_get_next_field(const char **line, char **output, int *olen, char separator);
+size_t	zbx_get_next_field(const char **line, char **output, size_t *olen, char separator);
 int	str_in_list(const char *list, const char *value, char delimiter);
 char	*str_linefeed(const char *src, size_t maxline, const char *delim);
 void	zbx_strarr_init(char ***arr);
@@ -785,26 +860,28 @@ void	__zbx_zbx_setproctitle(const char *fmt, ...);
 #define ZBX_MAX_RECV_DATA_SIZE	(128 * ZBX_MEBIBYTE)
 
 double	zbx_time();
+void	zbx_timespec(zbx_timespec_t *ts);
 double	zbx_current_time();
 
 #ifdef HAVE___VA_ARGS__
 #	define zbx_error(fmt, ...) __zbx_zbx_error(ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
 #	define zbx_snprintf(str, count, fmt, ...) __zbx_zbx_snprintf(str, count, ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
-#	define zbx_snprintf_alloc(str, alloc_len, offset, max_len, fmt, ...) \
-       			__zbx_zbx_snprintf_alloc(str, alloc_len, offset, max_len, ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
+#	define zbx_snprintf_alloc(str, alloc_len, offset, fmt, ...) \
+       			__zbx_zbx_snprintf_alloc(str, alloc_len, offset, ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
 #else
 #	define zbx_error __zbx_zbx_error
 #	define zbx_snprintf __zbx_zbx_snprintf
 #	define zbx_snprintf_alloc __zbx_zbx_snprintf_alloc
 #endif
 void	__zbx_zbx_error(const char *fmt, ...);
-int	__zbx_zbx_snprintf(char *str, size_t count, const char *fmt, ...);
-void	__zbx_zbx_snprintf_alloc(char **str, int *alloc_len, int *offset, int max_len, const char *fmt, ...);
+size_t	__zbx_zbx_snprintf(char *str, size_t count, const char *fmt, ...);
+void	__zbx_zbx_snprintf_alloc(char **str, size_t *alloc_len, size_t *offset, const char *fmt, ...);
 
-int	zbx_vsnprintf(char *str, size_t count, const char *fmt, va_list args);
+size_t	zbx_vsnprintf(char *str, size_t count, const char *fmt, va_list args);
 
-void	zbx_strcpy_alloc(char **str, int *alloc_len, int *offset, const char *src);
-void	zbx_chrcpy_alloc(char **str, int *alloc_len, int *offset, const char src);
+void	zbx_strncpy_alloc(char **str, size_t *alloc_len, size_t *offset, const char *src, size_t n);
+void	zbx_strcpy_alloc(char **str, size_t *alloc_len, size_t *offset, const char *src);
+void	zbx_chrcpy_alloc(char **str, size_t *alloc_len, size_t *offset, char c);
 
 /* secure string copy */
 #define strscpy(x, y)	zbx_strlcpy(x, y, sizeof(x))
@@ -823,7 +900,7 @@ char	*zbx_dvsprintf(char *dest, const char *f, va_list args);
 #endif
 char	*__zbx_zbx_dsprintf(char *dest, const char *f, ...);
 char	*zbx_strdcat(char *dest, const char *src);
-char	* __zbx_zbx_strdcatf(char *dest, const char *f, ...);
+char	*__zbx_zbx_strdcatf(char *dest, const char *f, ...);
 
 int	xml_get_data_dyn(const char *xml, const char *tag, char **data);
 void	xml_free_data_dyn(char **data);
@@ -835,7 +912,7 @@ int	comms_parse_response(char *xml, char *host, size_t host_len, char *key, size
 
 int 	parse_command(const char *command, char *cmd, size_t cmd_max_len, char *param, size_t param_max_len);
 
-typedef struct zbx_regexp_s
+typedef struct
 {
 	char			*name;
 	char			*expression;
@@ -847,7 +924,6 @@ ZBX_REGEXP;
 
 /* regular expressions */
 char    *zbx_regexp_match(const char *string, const char *pattern, int *len);
-/* non case sensitive */
 char    *zbx_iregexp_match(const char *string, const char *pattern, int *len);
 
 void	clean_regexps_ex(ZBX_REGEXP *regexps, int *regexps_num);
@@ -857,7 +933,9 @@ int	regexp_match_ex(ZBX_REGEXP *regexps, int regexps_num, const char *string, co
 		zbx_case_sensitive_t cs);
 
 /* misc functions */
+#ifdef HAVE_IPV6
 int	is_ip6(const char *ip);
+#endif
 int	is_ip4(const char *ip);
 
 void	zbx_on_exit(); /* calls exit() at the end! */
@@ -889,29 +967,32 @@ int	uint64_array_add(zbx_uint64_t **values, int *alloc, int *num, zbx_uint64_t v
 void	uint64_array_merge(zbx_uint64_t **values, int *alloc, int *num, zbx_uint64_t *value, int value_num, int alloc_step);
 int	uint64_array_exists(zbx_uint64_t *values, int num, zbx_uint64_t value);
 void	uint64_array_remove(zbx_uint64_t *values, int *num, zbx_uint64_t *rm_values, int rm_num);
+void	uint64_array_remove_both(zbx_uint64_t *values, int *num, zbx_uint64_t *rm_values, int *rm_num);
 
 #ifdef _WINDOWS
 LPTSTR	zbx_acp_to_unicode(LPCSTR acp_string);
+LPTSTR	zbx_oemcp_to_unicode(LPCSTR oemcp_string);
 int	zbx_acp_to_unicode_static(LPCSTR acp_string, LPTSTR wide_string, int wide_size);
 LPTSTR	zbx_utf8_to_unicode(LPCSTR utf8_string);
 LPSTR	zbx_unicode_to_utf8(LPCTSTR wide_string);
-int	zbx_unicode_to_utf8_static(LPCTSTR wide_string, LPSTR utf8_string, int utf8_size);
+LPSTR	zbx_unicode_to_utf8_static(LPCTSTR wide_string, LPSTR utf8_string, int utf8_size);
 int	_wis_uint(LPCTSTR wide_string);
 #endif
+void	zbx_strlower(char *str);
 void	zbx_strupper(char *str);
 #if defined(_WINDOWS) || defined(HAVE_ICONV)
 char	*convert_to_utf8(char *in, size_t in_size, const char *encoding);
 #endif	/* HAVE_ICONV */
-int	zbx_utf8_char_len(const char *text);
+size_t	zbx_utf8_char_len(const char *text);
 size_t	zbx_strlen_utf8(const char *text);
 size_t	zbx_strlen_utf8_n(const char *text, size_t utf8_maxlen);
 
+int	zbx_is_utf8(const char *text);
 #define ZBX_UTF8_REPLACE_CHAR	'?'
 char	*zbx_replace_utf8(const char *text);
 void	zbx_replace_invalid_utf8(char *text);
 
-void	win2unix_eol(char *text);
-int	str2uint(const char *str);
+void	dos2unix(char *str);
 int	str2uint64(const char *str, const char *suffixes, zbx_uint64_t *value);
 double	str2double(const char *str);
 
@@ -932,6 +1013,9 @@ int	zbx_check_hostname(const char *hostname);
 int	is_hostname_char(char c);
 int	is_key_char(char c);
 int	is_function_char(char c);
+int	is_macro_char(char c);
+
+int	is_time_function(const char *func);
 
 int	parse_host(char **exp, char **host);
 int	parse_key(char **exp, char **key);
@@ -941,8 +1025,12 @@ int	parse_host_key(char *exp, char **host, char **key);
 
 void	make_hostname(char *host);
 
+unsigned char	get_interface_type_by_item_type(unsigned char type);
+
 int	calculate_sleeptime(int nextcheck, int max_sleeptime);
 
+void	zbx_replace_string(char **data, size_t l, size_t *r, const char *value);
+
 int	parse_serveractive_element(char *str, char **host, unsigned short *port, unsigned short port_default);
 
 #endif
