@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -195,7 +195,7 @@ zbx_graph_item_type;
 
 #ifdef HAVE_ORACLE
 #define	DBbegin_multiple_update(sql, sql_alloc, sql_offset)	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, "begin\n")
-#define	DBend_multiple_update(sql, sql_alloc, sql_offset)	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, "end;\n")
+#define	DBend_multiple_update(sql, sql_alloc, sql_offset)	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, "end;")
 
 #define	ZBX_SQL_STRCMP		"%s%s%s"
 #define	ZBX_SQL_STRVAL_EQ(str)	'\0' != *str ? "='"  : "",		\
@@ -443,6 +443,11 @@ void	DBinit();
 
 void	DBclose();
 
+#ifdef HAVE_ORACLE
+void	DBstatement_prepare(const char *sql);
+void	DBbind_parameter(int position, void *buffer, unsigned char type);
+int	DBstatement_execute();
+#endif
 #ifdef HAVE___VA_ARGS__
 #	define DBexecute(fmt, ...) __zbx_DBexecute(ZBX_CONST_STRING(fmt), ##__VA_ARGS__)
 #else
@@ -483,7 +488,7 @@ zbx_uint64_t	DBget_maxid_num(const char *tablename, int num);
  ******************************************************************************/
 typedef struct
 {
-	zbx_uint64_t	itemid; /* itemid should come first for correct sorting */
+	zbx_uint64_t	itemid;	/* itemid should come first for correct sorting */
 	zbx_uint64_t	gitemid;
 	char		key[ITEM_KEY_LEN * 4 + 1];
 	int		drawtype;
@@ -535,6 +540,8 @@ int	DBcopy_template_elements(zbx_uint64_t hostid, zbx_vector_uint64_t *lnk_templ
 int	DBdelete_template_elements(zbx_uint64_t hostid, zbx_vector_uint64_t *del_templateids);
 
 void	DBdelete_items(zbx_vector_uint64_t *itemids);
+void	DBdelete_triggers(zbx_vector_uint64_t *triggerids);
+void	DBdelete_graphs(zbx_vector_uint64_t *graphids);
 void	DBdelete_host(zbx_uint64_t hostid);
 void	DBget_graphitems(const char *sql, ZBX_GRAPH_ITEMS **gitems, size_t *gitems_alloc, size_t *gitems_num);
 void	DBupdate_services(zbx_uint64_t triggerid, int status, int clock);
@@ -580,4 +587,9 @@ void	DBfree_history(char **value);
 int	DBtxn_status();
 int	DBtxn_ongoing();
 
+void	DBexecute_multiple_query(const char *query, const char *field_name, zbx_vector_uint64_t *ids);
+
+void	zbx_create_services_lock();
+void	zbx_destroy_services_lock();
+
 #endif
