@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #ifndef ZABBIX_ZBXSELF_H
@@ -22,7 +22,7 @@
 
 #if defined(_WINDOWS)
 #	error "This module allowed only for Unix OS"
-#endif	/* _WINDOWS */
+#endif
 
 #define ZBX_PROCESS_STATE_IDLE		0
 #define ZBX_PROCESS_STATE_BUSY		1
@@ -32,22 +32,24 @@
 #define ZBX_PROCESS_TYPE_UNREACHABLE	1
 #define ZBX_PROCESS_TYPE_IPMIPOLLER	2
 #define ZBX_PROCESS_TYPE_PINGER		3
-#define ZBX_PROCESS_TYPE_HTTPPOLLER	4
-#define ZBX_PROCESS_TYPE_TRAPPER	5
-#define ZBX_PROCESS_TYPE_PROXYPOLLER	6
-#define ZBX_PROCESS_TYPE_ESCALATOR	7
-#define ZBX_PROCESS_TYPE_HISTSYNCER	8
-#define ZBX_PROCESS_TYPE_DISCOVERER	9
-#define ZBX_PROCESS_TYPE_ALERTER	10
-#define ZBX_PROCESS_TYPE_TIMER		11
-#define ZBX_PROCESS_TYPE_NODEWATCHER	12
-#define ZBX_PROCESS_TYPE_HOUSEKEEPER	13
-#define ZBX_PROCESS_TYPE_WATCHDOG	14
-#define ZBX_PROCESS_TYPE_DATASENDER	15
-#define ZBX_PROCESS_TYPE_CONFSYNCER	16
-#define ZBX_PROCESS_TYPE_HEARTBEAT	17
-#define ZBX_PROCESS_TYPE_SELFMON	18
-#define ZBX_PROCESS_TYPE_COUNT		19	/* number of process types */
+#define ZBX_PROCESS_TYPE_JAVAPOLLER	4
+#define ZBX_PROCESS_TYPE_HTTPPOLLER	5
+#define ZBX_PROCESS_TYPE_TRAPPER	6
+#define ZBX_PROCESS_TYPE_SNMPTRAPPER	7
+#define ZBX_PROCESS_TYPE_PROXYPOLLER	8
+#define ZBX_PROCESS_TYPE_ESCALATOR	9
+#define ZBX_PROCESS_TYPE_HISTSYNCER	10
+#define ZBX_PROCESS_TYPE_DISCOVERER	11
+#define ZBX_PROCESS_TYPE_ALERTER	12
+#define ZBX_PROCESS_TYPE_TIMER		13
+#define ZBX_PROCESS_TYPE_NODEWATCHER	14
+#define ZBX_PROCESS_TYPE_HOUSEKEEPER	15
+#define ZBX_PROCESS_TYPE_WATCHDOG	16
+#define ZBX_PROCESS_TYPE_DATASENDER	17
+#define ZBX_PROCESS_TYPE_CONFSYNCER	18
+#define ZBX_PROCESS_TYPE_HEARTBEAT	19
+#define ZBX_PROCESS_TYPE_SELFMON	20
+#define ZBX_PROCESS_TYPE_COUNT		21	/* number of process types */
 #define ZBX_PROCESS_TYPE_UNKNOWN	255
 
 #define ZBX_AGGR_FUNC_ONE		0
