@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
@@ -22,6 +22,7 @@
 #include "db.h"
 #include "zbxdb.h"
 #include "log.h"
+#include "mutexs.h"
 
 static int	txn_level = 0;	/* transaction level, nested transactions are not supported */
 static int	txn_error = 0;	/* failed transaction */
@@ -36,11 +37,11 @@ static zbx_oracle_db_handle_t	oracle;
 #elif defined(HAVE_POSTGRESQL)
 static PGconn			*conn = NULL;
 static int			ZBX_PG_BYTEAOID = 0;
-int				ZBX_PG_SVERSION = 0;
+static int			ZBX_PG_SVERSION = 0;
 char				ZBX_PG_ESCAPE_BACKSLASH = 1;
 #elif defined(HAVE_SQLITE3)
 static sqlite3			*conn = NULL;
-PHP_MUTEX			sqlite_access;
+static PHP_MUTEX		sqlite_access;
 #endif
 
 #if defined(HAVE_ORACLE)
@@ -111,12 +112,14 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 
 	txn_init = 1;
 
+	assert(NULL != host);
+
 #if defined(HAVE_IBM_DB2)
-	connect = strdup("PROTOCOL=TCPIP;");
+	connect = zbx_strdup(connect, "PROTOCOL=TCPIP;");
+	if ('\0' != *host)
+		connect = zbx_strdcatf(connect, "HOSTNAME=%s;", host);
 	if (NULL != dbname && '\0' != *dbname)
 		connect = zbx_strdcatf(connect, "DATABASE=%s;", dbname);
-	if (NULL != host && '\0' != *host)
-		connect = zbx_strdcatf(connect, "HOSTNAME=%s;", host);
 	if (0 != port)
 		connect = zbx_strdcatf(connect, "PORT=%d;", port);
 	if (NULL != user && '\0' != *user)
@@ -130,17 +133,20 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 	if (SUCCEED != zbx_ibm_db2_success(SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &ibm_db2.henv)))
 		ret = ZBX_DB_FAIL;
 
-	/* set attribute to enable application to run as ODBC 3.0 application; recommended for pure IBM DB2 CLI, but not required */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLSetEnvAttr(ibm_db2.henv, SQL_ATTR_ODBC_VERSION, (void *)SQL_OV_ODBC3, 0)))
+	/* set attribute to enable application to run as ODBC 3.0 application; */
+	/* recommended for pure IBM DB2 CLI, but not required */
+	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLSetEnvAttr(ibm_db2.henv, SQL_ATTR_ODBC_VERSION,
+			(void *)SQL_OV_ODBC3, 0)))
 		ret = ZBX_DB_FAIL;
 
 	/* allocate a database connection handle */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLAllocHandle(SQL_HANDLE_DBC, ibm_db2.henv, &ibm_db2.hdbc)))
+	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLAllocHandle(SQL_HANDLE_DBC, ibm_db2.henv,
+			&ibm_db2.hdbc)))
 		ret = ZBX_DB_FAIL;
 
 	/* connect to the database */
-	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLDriverConnect(ibm_db2.hdbc, NULL, (SQLCHAR *)connect, SQL_NTS,
-								NULL, 0, NULL, SQL_DRIVER_NOPROMPT)))
+	if (ZBX_DB_OK == ret && SUCCEED != zbx_ibm_db2_success(SQLDriverConnect(ibm_db2.hdbc, NULL, (SQLCHAR *)connect,
+			SQL_NTS, NULL, 0, NULL, SQL_DRIVER_NOPROMPT)))
 		ret = ZBX_DB_FAIL;
 
 	/* set autocommit on */
@@ -193,7 +199,7 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 
 	if (ZBX_DB_OK == ret)
 	{
-		DBexecute("SET NAMES utf8");
+		DBexecute("set names utf8");
 	}
 
 	if (ZBX_DB_FAIL == ret)
@@ -204,6 +210,7 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 			case CR_SERVER_GONE_ERROR:
 			case CR_CONNECTION_ERROR:
 			case CR_SERVER_LOST:
+			case CR_UNKNOWN_HOST:
 			case ER_SERVER_SHUTDOWN:
 			case ER_ACCESS_DENIED_ERROR:		/* wrong user or password */
 			case ER_ILLEGAL_GRANT_FOR_TABLE:	/* user without any privileges */
@@ -224,14 +231,12 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 
 	/* connection string format: [//]host[:port][/service name] */
 
-	if (NULL != host && '\0' != *host)
+	if ('\0' != *host)
 	{
 		connect = zbx_strdcatf(connect, "//%s", host);
-
-		if (port)
+		if (0 != port)
 			connect = zbx_strdcatf(connect, ":%d", port);
-
-		if (dbname && *dbname)
+		if (NULL != dbname && '\0' != *dbname)
 			connect = zbx_strdcatf(connect, "/%s", dbname);
 	}
 	else
@@ -265,24 +270,35 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 				(text *)connect, (ub4)strlen(connect),
 				OCI_DEFAULT);
 
+		if (OCI_SUCCESS == err)
+		{
+			err = OCIAttrGet((void *)oracle.svchp, OCI_HTYPE_SVCCTX, (void *)&oracle.srvhp, (ub4 *)0,
+					OCI_ATTR_SERVER, oracle.errhp);
+		}
+
 		if (OCI_SUCCESS != err)
 		{
 			zabbix_errlog(ERR_Z3001, connect, err, zbx_oci_error(err));
 			ret = ZBX_DB_DOWN;
 		}
-		else
-		{
-			err = OCIAttrGet((void *)oracle.svchp, OCI_HTYPE_SVCCTX, (void *)&oracle.srvhp, (ub4 *)0,
-					OCI_ATTR_SERVER, oracle.errhp);
+	}
 
-			if (OCI_SUCCESS != err)
-			{
-				zabbix_errlog(ERR_Z3001, connect, err, zbx_oci_error(err));
-				ret = ZBX_DB_DOWN;
-			}
+	if (ZBX_DB_OK == ret)
+	{
+		/* initialize statement handle */
+		err = OCIHandleAlloc((dvoid *)oracle.envhp, (dvoid **)&oracle.stmthp, OCI_HTYPE_STMT,
+				(size_t)0, (dvoid **)0);
+
+		if (OCI_SUCCESS != err)
+		{
+			zabbix_errlog(ERR_Z3001, connect, err, zbx_oci_error(err));
+			ret = ZBX_DB_DOWN;
 		}
 	}
 
+	if (ZBX_DB_OK == ret)
+		DBexecute("alter session set nls_numeric_characters='. '");
+
 	zbx_free(connect);
 
 	if (ZBX_DB_OK != ret)
@@ -354,8 +370,8 @@ int	zbx_db_connect(char *host, char *user, char *password, char *dbname, char *d
 		else
 			*path = '\0';
 
-		DBexecute("PRAGMA synchronous = 0"); /* OFF */
-		DBexecute("PRAGMA temp_store = 2"); /* MEMORY */
+		DBexecute("PRAGMA synchronous = 0");	/* OFF */
+		DBexecute("PRAGMA temp_store = 2");	/* MEMORY */
 		DBexecute("PRAGMA temp_store_directory = '%s'", path);
 
 		zbx_free(path);
@@ -376,9 +392,14 @@ void	zbx_create_sqlite3_mutex(const char *dbname)
 		exit(FAIL);
 	}
 }
+
+void	zbx_remove_sqlite3_mutex()
+{
+	php_sem_remove(&sqlite_access);
+}
 #endif	/* HAVE_SQLITE3 */
 
-void	zbx_db_init(char *host, char *user, char *password, char *dbname, char *dbschema, char *dbsocket, int port)
+void	zbx_db_init(char *dbname)
 {
 #if defined(HAVE_SQLITE3)
 	struct stat	buf;
@@ -422,27 +443,34 @@ void	zbx_db_close()
 	mysql_close(conn);
 	conn = NULL;
 #elif defined(HAVE_ORACLE)
-	if (oracle.svchp)
+	/* deallocate statement handle */
+	if (NULL != oracle.stmthp)
 	{
-		(void)OCILogoff(oracle.svchp, oracle.errhp);
+		OCIHandleFree((dvoid *)oracle.stmthp, OCI_HTYPE_STMT);
+		oracle.stmthp = NULL;
+	}
+
+	if (NULL != oracle.svchp)
+	{
+		OCILogoff(oracle.svchp, oracle.errhp);
 		oracle.svchp = NULL;
 	}
 
-	if (oracle.errhp)
+	if (NULL != oracle.errhp)
 	{
-		(void)OCIHandleFree(oracle.errhp, OCI_HTYPE_ERROR);
+		OCIHandleFree(oracle.errhp, OCI_HTYPE_ERROR);
 		oracle.errhp = NULL;
 	}
 
-	if (oracle.envhp)
+	if (NULL != oracle.envhp)
 	{
-		(void)OCIHandleFree((dvoid *)oracle.envhp, OCI_HTYPE_ENV);
+		OCIHandleFree((dvoid *)oracle.envhp, OCI_HTYPE_ENV);
 		oracle.envhp = NULL;
 	}
 
-	if (oracle.srvhp)
+	if (NULL != oracle.srvhp)
 	{
-		(void)OCIHandleFree(oracle.srvhp, OCI_HTYPE_SERVER);
+		OCIHandleFree(oracle.srvhp, OCI_HTYPE_SERVER);
 		oracle.srvhp = NULL;
 	}
 #elif defined(HAVE_POSTGRESQL)
@@ -658,6 +686,146 @@ int	zbx_db_txn_error()
 	return txn_error;
 }
 
+#ifdef HAVE_ORACLE
+static sword	zbx_oracle_statement_prepare(const char *sql)
+{
+	return OCIStmtPrepare(oracle.stmthp, oracle.errhp, (text *)sql, (ub4)strlen((char *)sql), (ub4)OCI_NTV_SYNTAX,
+			(ub4)OCI_DEFAULT);
+}
+
+static sword	zbx_oracle_bind_parameter(ub4 position, void *buffer, sb4 buffer_sz, ub2 dty)
+{
+	OCIBind	*bindhp = NULL;
+
+	return OCIBindByPos(oracle.stmthp, &bindhp, oracle.errhp, position, buffer, buffer_sz, dty,
+			0, 0, 0, 0, 0, (ub4)OCI_DEFAULT);
+}
+
+static sword	zbx_oracle_statement_execute(ub4 *nrows)
+{
+	sword	err;
+
+	if (OCI_SUCCESS == (err = OCIStmtExecute(oracle.svchp, oracle.stmthp, oracle.errhp, (ub4)1, (ub4)0,
+			(CONST OCISnapshot *)NULL, (OCISnapshot *)NULL,
+			0 == txn_level ? OCI_COMMIT_ON_SUCCESS : OCI_DEFAULT)))
+	{
+		err = OCIAttrGet((void *)oracle.stmthp, OCI_HTYPE_STMT, nrows, (ub4 *)0, OCI_ATTR_ROW_COUNT, oracle.errhp);
+	}
+
+	return err;
+}
+#endif
+
+#ifdef HAVE_ORACLE
+int	zbx_db_statement_prepare(const char *sql)
+{
+	sword	err;
+	int	ret = ZBX_DB_OK;
+
+	if (0 == txn_init && 0 == txn_level)
+		zabbix_log(LOG_LEVEL_DEBUG, "query without transaction detected");
+
+	if (1 == txn_error)
+	{
+		zabbix_log(LOG_LEVEL_DEBUG, "ignoring query [txnlev:%d] within failed transaction", txn_level);
+		return ZBX_DB_FAIL;
+	}
+
+	zabbix_log(LOG_LEVEL_DEBUG, "query [txnlev:%d] [%s]", txn_level, sql);
+
+	/* Oracle */
+	if (OCI_SUCCESS != (err = zbx_oracle_statement_prepare(sql)))
+	{
+		zabbix_errlog(ERR_Z3005, err, zbx_oci_error(err), sql);
+		ret = (OCI_SERVER_NORMAL == OCI_DBserver_status() ? ZBX_DB_FAIL : ZBX_DB_DOWN);
+	}
+
+	if (ZBX_DB_FAIL == ret && 0 < txn_level)
+	{
+		zabbix_log(LOG_LEVEL_DEBUG, "query [%s] failed, setting transaction as failed", sql);
+		txn_error = 1;
+	}
+
+	return ret;
+}
+
+int	zbx_db_bind_parameter(int position, void *buffer, unsigned char type)
+{
+	sword	err;
+	int	ret = ZBX_DB_OK;
+
+	if (1 == txn_error)
+	{
+		zabbix_log(LOG_LEVEL_DEBUG, "ignoring query [txnlev:%d] within failed transaction", txn_level);
+		return ZBX_DB_FAIL;
+	}
+
+	/* Oracle */
+	switch (type)
+	{
+		case ZBX_TYPE_ID:
+			err = zbx_oracle_bind_parameter((ub4)position, buffer, (sb4)sizeof(zbx_uint64_t), SQLT_INT);
+			break;
+		case ZBX_TYPE_INT:
+			err = zbx_oracle_bind_parameter((ub4)position, buffer, (sb4)sizeof(int), SQLT_INT);
+			break;
+		case ZBX_TYPE_TEXT:
+			err = zbx_oracle_bind_parameter((ub4)position, buffer, (sb4)strlen((char *)buffer), SQLT_LNG);
+			break;
+		default:
+			err = OCI_SUCCESS;
+	}
+
+	if (OCI_SUCCESS != err)
+	{
+		zabbix_errlog(ERR_Z3007, err, zbx_oci_error(err));
+		ret = (OCI_SERVER_NORMAL == OCI_DBserver_status() ? ZBX_DB_FAIL : ZBX_DB_DOWN);
+	}
+
+	if (ZBX_DB_FAIL == ret && 0 < txn_level)
+	{
+		zabbix_log(LOG_LEVEL_DEBUG, "query failed, setting transaction as failed");
+		txn_error = 1;
+	}
+
+	return ret;
+
+}
+
+int	zbx_db_statement_execute()
+{
+	const char	*__function_name = "zbx_db_statement_execute";
+	sword	err;
+	ub4	nrows;
+	int	ret;
+
+	if (1 == txn_error)
+	{
+		zabbix_log(LOG_LEVEL_DEBUG, "ignoring query [txnlev:%d] within failed transaction", txn_level);
+		ret = ZBX_DB_FAIL;
+		goto out;
+	}
+
+	if (OCI_SUCCESS != (err = zbx_oracle_statement_execute(&nrows)))
+	{
+		zabbix_errlog(ERR_Z3007, err, zbx_oci_error(err));
+		ret = (OCI_SERVER_NORMAL == OCI_DBserver_status() ? ZBX_DB_FAIL : ZBX_DB_DOWN);
+	}
+	else
+		ret = (int)nrows;
+
+	if (ZBX_DB_FAIL == ret && 0 < txn_level)
+	{
+		zabbix_log(LOG_LEVEL_DEBUG, "query failed, setting transaction as failed");
+		txn_error = 1;
+	}
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "%s():%d", __function_name, ret);
+
+	return ret;
+}
+#endif
+
 /*
  * Execute SQL statement. For non-select statements only.
  */
@@ -675,7 +843,6 @@ int	zbx_db_vexecute(const char *fmt, va_list args)
 #elif defined(HAVE_MYSQL)
 	int		status;
 #elif defined(HAVE_ORACLE)
-	OCIStmt		*stmthp = NULL;
 	sword		err = OCI_SUCCESS;
 #elif defined(HAVE_POSTGRESQL)
 	PGresult	*result;
@@ -791,28 +958,12 @@ int	zbx_db_vexecute(const char *fmt, va_list args)
 		}
 	}
 #elif defined(HAVE_ORACLE)
-	err = OCIHandleAlloc((dvoid *)oracle.envhp, (dvoid **)&stmthp, OCI_HTYPE_STMT, (size_t)0, (dvoid **)0);
-
-	if (OCI_SUCCESS == err)
+	if (OCI_SUCCESS == (err = zbx_oracle_statement_prepare(sql)))
 	{
-		err = OCIStmtPrepare(stmthp, oracle.errhp, (text *)sql, (ub4)strlen((char *)sql),
-				(ub4)OCI_NTV_SYNTAX, (ub4)OCI_DEFAULT);
-	}
+		ub4	nrows = 0;
 
-	if (OCI_SUCCESS == err)
-	{
-		err = OCIStmtExecute(oracle.svchp, stmthp, oracle.errhp, (ub4)1, (ub4)0,
-				(CONST OCISnapshot *)NULL, (OCISnapshot *)NULL, OCI_COMMIT_ON_SUCCESS);
-
-		if (OCI_SUCCESS == err)
-		{
-			ub4	nrows = 0;
-
-			err = OCIAttrGet((void *)stmthp, OCI_HTYPE_STMT, (ub4 *)&nrows,
-					  (ub4 *)0, OCI_ATTR_ROW_COUNT, oracle.errhp);
-
-			ret = nrows;
-		}
+		if (OCI_SUCCESS == (err = zbx_oracle_statement_execute(&nrows)))
+			ret = (int)nrows;
 	}
 
 	if (OCI_SUCCESS != err)
@@ -820,12 +971,6 @@ int	zbx_db_vexecute(const char *fmt, va_list args)
 		zabbix_errlog(ERR_Z3005, err, zbx_oci_error(err), sql);
 		ret = (OCI_SERVER_NORMAL == OCI_DBserver_status() ? ZBX_DB_FAIL : ZBX_DB_DOWN);
 	}
-
-	if (NULL != stmthp)
-	{
-		(void)OCIHandleFree((dvoid *)stmthp, OCI_HTYPE_STMT);
-		stmthp = NULL;
-	}
 #elif defined(HAVE_POSTGRESQL)
 	result = PQexec(conn,sql);
 
@@ -926,7 +1071,7 @@ DB_RESULT	zbx_db_vselect(const char *fmt, va_list args)
 	SQLRETURN	ret = SQL_SUCCESS;
 #elif defined(HAVE_ORACLE)
 	sword		err = OCI_SUCCESS;
-	ub4		counter;
+	ub4		prefetch_rows = 200, counter;
 #elif defined(HAVE_POSTGRESQL)
 	char		*error = NULL;
 #elif defined(HAVE_SQLITE3)
@@ -1041,6 +1186,22 @@ error:
 
 	err = OCIHandleAlloc((dvoid *)oracle.envhp, (dvoid **)&result->stmthp, OCI_HTYPE_STMT, (size_t)0, (dvoid **)0);
 
+	/* Prefetching when working with Oracle is needed because otherwise it fetches only 1 row at a time when doing */
+	/* selects (default behavior). There are 2 ways to do prefetching: memory based and rows based. Based on the   */
+	/* study optimal (speed-wise) memory based prefetch is 2 MB. But in case of many subsequent selects CPU usage  */
+	/* jumps up to 100 %. Using rows prefetch with up to 200 rows does not affect CPU usage, it is the same as     */
+	/* without prefetching at all. See ZBX-5920, ZBX-6493 for details.                                             */
+	/*                                                                                                             */
+	/* Tested on Oracle 11gR2.                                                                                     */
+	/*                                                                                                             */
+	/* Oracle info: docs.oracle.com/cd/B28359_01/appdev.111/b28395/oci04sql.htm                                    */
+
+	if (OCI_SUCCESS == err)
+	{
+		err = OCIAttrSet(result->stmthp, OCI_HTYPE_STMT, &prefetch_rows, sizeof(prefetch_rows),
+				OCI_ATTR_PREFETCH_ROWS, oracle.errhp);
+	}
+
 	if (OCI_SUCCESS == err)
 	{
 		err = OCIStmtPrepare(result->stmthp, oracle.errhp, (text *)sql, (ub4)strlen((char *)sql),
@@ -1050,7 +1211,8 @@ error:
 	if (OCI_SUCCESS == err)
 	{
 		err = OCIStmtExecute(oracle.svchp, result->stmthp, oracle.errhp, (ub4)0, (ub4)0,
-				(CONST OCISnapshot *)NULL, (OCISnapshot *)NULL, OCI_COMMIT_ON_SUCCESS);
+				(CONST OCISnapshot *)NULL, (OCISnapshot *)NULL,
+				0 == txn_level ? OCI_COMMIT_ON_SUCCESS : OCI_DEFAULT);
 	}
 
 	if (OCI_SUCCESS == err)
@@ -1066,53 +1228,84 @@ error:
 	assert(0 < result->ncolumn);
 
 	result->values = zbx_malloc(NULL, result->ncolumn * sizeof(char *));
+	result->clobs = zbx_malloc(NULL, result->ncolumn * sizeof(OCILobLocator *));
+	result->values_alloc = zbx_malloc(NULL, result->ncolumn * sizeof(ub4));
 	memset(result->values, 0, result->ncolumn * sizeof(char *));
+	memset(result->clobs, 0, result->ncolumn * sizeof(OCILobLocator *));
+	memset(result->values_alloc, 0, result->ncolumn * sizeof(ub4));
 
 	for (counter = 1; OCI_SUCCESS == err && counter <= result->ncolumn; counter++)
 	{
 		OCIParam	*parmdp = NULL;
 		OCIDefine	*defnp = NULL;
 		ub4		char_semantics;
-		ub2		col_width;
+		ub2		col_width = 0, data_type;
 
 		/* request a parameter descriptor in the select-list */
 		err = OCIParamGet((void *)result->stmthp, OCI_HTYPE_STMT, oracle.errhp, (void **)&parmdp, (ub4)counter);
 
 		if (OCI_SUCCESS == err)
 		{
-			/* retrieve the length semantics for the column */
-			char_semantics = 0;
-			err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&char_semantics, (ub4 *)0,
-					(ub4)OCI_ATTR_CHAR_USED, (OCIError *)oracle.errhp);
+			/* retrieve the data type for the column */
+			err = OCIAttrGet((void *)parmdp, OCI_DTYPE_PARAM, (dvoid *)&data_type, (ub4 *)NULL,
+					(ub4)OCI_ATTR_DATA_TYPE, (OCIError *)oracle.errhp);
 		}
 
-		if (OCI_SUCCESS == err)
+		if (SQLT_CLOB == data_type)
 		{
-			col_width = 0;
-			if (char_semantics)
+			if (ZBX_DB_OK == err)
 			{
-				/* retrieve the column width in characters */
-				err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&col_width, (ub4 *)0,
-						(ub4)OCI_ATTR_CHAR_SIZE, (OCIError *)oracle.errhp);
+				/* allocate the lob locator variable */
+				err = OCIDescriptorAlloc((dvoid *)oracle.envhp, (dvoid **)&result->clobs[counter - 1],
+						OCI_DTYPE_LOB, (size_t)0, (dvoid **)0);
 			}
-			else
+
+			if (OCI_SUCCESS == err)
 			{
-				/* retrieve the column width in bytes */
-				err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&col_width, (ub4 *)0,
-						(ub4)OCI_ATTR_DATA_SIZE, (OCIError *)oracle.errhp);
+				/* associate clob var with its define handle */
+				err = OCIDefineByPos((void *)result->stmthp, &defnp, (OCIError *)oracle.errhp,
+						(ub4)counter, (dvoid *)&result->clobs[counter - 1], (sb4)-1,
+						data_type, (dvoid *)0, (ub2 *)0, (ub2 *)0, (ub4)OCI_DEFAULT);
 			}
 		}
-		col_width++;
-
-		result->values[counter - 1] = zbx_malloc(NULL, col_width);
-		memset(result->values[counter - 1], 0, col_width);
-
-		if (OCI_SUCCESS == err)
+		else
 		{
-			/* represent any data as characters */
-			err = OCIDefineByPos(result->stmthp, &defnp, oracle.errhp, counter,
-					(dvoid *)result->values[counter - 1], col_width, SQLT_STR,
-					(dvoid *)0, (ub2 *)0, (ub2 *)0, OCI_DEFAULT);
+			if (OCI_SUCCESS == err)
+			{
+				/* retrieve the length semantics for the column */
+				char_semantics = 0;
+				err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&char_semantics,
+						(ub4 *)NULL, (ub4)OCI_ATTR_CHAR_USED, (OCIError *)oracle.errhp);
+			}
+
+			if (OCI_SUCCESS == err)
+			{
+				if (0 != char_semantics)
+				{
+					/* retrieve the column width in characters */
+					err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&col_width,
+							(ub4 *)NULL, (ub4)OCI_ATTR_CHAR_SIZE, (OCIError *)oracle.errhp);
+				}
+				else
+				{
+					/* retrieve the column width in bytes */
+					err = OCIAttrGet((void *)parmdp, (ub4)OCI_DTYPE_PARAM, (void *)&col_width,
+							(ub4 *)NULL, (ub4)OCI_ATTR_DATA_SIZE, (OCIError *)oracle.errhp);
+				}
+			}
+			col_width++;	/* add 1 byte for terminating '\0' */
+
+			result->values_alloc[counter - 1] = col_width;
+			result->values[counter - 1] = zbx_malloc(NULL, col_width);
+			*result->values[counter - 1] = '\0';
+
+			if (OCI_SUCCESS == err)
+			{
+				/* represent any data as characters */
+				err = OCIDefineByPos(result->stmthp, &defnp, oracle.errhp, counter,
+						(dvoid *)result->values[counter - 1], col_width, SQLT_STR,
+						(dvoid *)0, (ub2 *)0, (ub2 *)0, OCI_DEFAULT);
+			}
 		}
 
 		/* free cell descriptor */
@@ -1232,6 +1425,7 @@ DB_ROW	zbx_db_fetch(DB_RESULT result)
 #if defined(HAVE_IBM_DB2)
 	int		i;
 #elif defined(HAVE_ORACLE)
+	int		i;
 	sword		rc;
 	static char	errbuf[512];
 	sb4		errcode;
@@ -1256,6 +1450,41 @@ DB_ROW	zbx_db_fetch(DB_RESULT result)
 	if (OCI_NO_DATA == (rc = OCIStmtFetch2(result->stmthp, oracle.errhp, 1, OCI_FETCH_NEXT, 0, OCI_DEFAULT)))
 		return NULL;
 
+	for (i = 0; i < result->ncolumn; i++)
+	{
+		if (NULL != result->clobs[i])
+		{
+			ub4	alloc, amount, amt = 0;
+			ub1	csfrm;
+
+			rc = OCILobGetLength(oracle.svchp, oracle.errhp, result->clobs[i], &amount);
+
+			if (OCI_SUCCESS != rc)
+				break;
+
+			rc = OCILobCharSetForm(oracle.envhp, oracle.errhp, result->clobs[i], &csfrm);
+
+			if (OCI_SUCCESS != rc)
+				break;
+
+			if (result->values_alloc[i] < (alloc = amount * 4 + 1))
+			{
+				result->values_alloc[i] = alloc;
+				result->values[i] = zbx_realloc(result->values[i], result->values_alloc[i]);
+			}
+
+			amt = amount;
+			rc = OCILobRead(oracle.svchp, oracle.errhp, result->clobs[i], &amt, (ub4)1,
+					(dvoid *)result->values[i], (ub4)(result->values_alloc[i] - 1),
+					(dvoid *)NULL, (OCICallbackLobRead)NULL, (ub2)0, csfrm);
+
+			if (OCI_SUCCESS != rc)
+				zabbix_errlog(ERR_Z3006, rc, zbx_oci_error(rc));
+
+			result->values[i][amt] = '\0';
+		}
+	}
+
 	if (OCI_SUCCESS == rc)
 		return result->values;
 
@@ -1376,9 +1605,18 @@ void	OCI_DBfree_result(DB_RESULT result)
 		for (i = 0; i < result->ncolumn; i++)
 		{
 			zbx_free(result->values[i]);
+
+			/* deallocate the lob locator variable */
+			if (NULL != result->clobs[i])
+			{
+				OCIDescriptorFree((dvoid *)result->clobs[i], OCI_DTYPE_LOB);
+				result->clobs[i] = NULL;
+			}
 		}
 
 		zbx_free(result->values);
+		zbx_free(result->clobs);
+		zbx_free(result->values_alloc);
 	}
 
 	if (result->stmthp)
