@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
@@ -611,15 +611,51 @@ const char	*zbx_json_next(struct zbx_json_parse *jp, const char *p)
 	return NULL;
 }
 
-const char	*zbx_json_next_value(struct zbx_json_parse *jp, const char *p, char *string, size_t len)
+static size_t	zbx_json_string_size(const char *p)
 {
-	if (NULL == (p = zbx_json_next(jp, p)))
-		return NULL;
+	int	state = 0; /* 0 - init; 1 - inside string */
+	size_t	sz = 0;
 
-	if (NULL == zbx_json_decodevalue(p, string, len))
-		return NULL;
+	if ('"' != *p)
+		return (size_t)(-1);
 
-	return p;
+	while (*p != '\0')	/* this should never happen */
+	{
+		if (*p == '"')
+		{
+			if (state == 1)
+				return sz;
+			state = 1;
+		}
+		else if (state == 1)
+		{
+			if (*p == '\\')
+			{
+				switch (*++p)
+				{
+					case 'u':
+						p += 2;
+					case '"':
+					case '\\':
+					case '/':
+					case 'b':
+					case 'f':
+					case 'n':
+					case 'r':
+					case 't':
+						sz++;
+						break;
+					default:
+						THIS_SHOULD_NEVER_HAPPEN;
+				}
+			}
+			else
+				sz++;
+		}
+		p++;
+	}
+
+	return (size_t)(-1);
 }
 
 static const char	*zbx_json_decodestring(const char *p, char *string, size_t len)
@@ -688,6 +724,22 @@ static const char	*zbx_json_decodestring(const char *p, char *string, size_t len
 	return NULL;
 }
 
+static size_t	zbx_json_int_size(const char *p)
+{
+	size_t	sz = 0;
+
+	while (*p != '\0')	/* this should never happen */
+	{
+		if ((*p < '0' || *p > '9') && *p != '-')
+			return sz;
+		else
+			sz++;
+		p++;
+	}
+
+	return (size_t)(-1);
+}
+
 static const char	*zbx_json_decodeint(const char *p, char *string, size_t len)
 {
 	char	*o = string;
@@ -710,14 +762,71 @@ static const char	*zbx_json_decodeint(const char *p, char *string, size_t len)
 	return NULL;
 }
 
-const char	*zbx_json_decodevalue(const char *p, char *string, size_t len)
+static const char	*zbx_json_decodenull(const char *p)
+{
+	if ('n' == p[0] && 'u' == p[1] && 'l' == p[2] && 'l' == p[3])
+		return p + 4;
+
+	return NULL;
+}
+
+static size_t	zbx_json_value_size(const char *p, zbx_json_type_t jt)
+{
+	switch (jt)
+	{
+		case ZBX_JSON_TYPE_STRING:
+			return zbx_json_string_size(p);
+		case ZBX_JSON_TYPE_INT:
+			return zbx_json_int_size(p);
+		default:
+			return (size_t)(-1);
+	}
+}
+
+static const char	*zbx_json_decodevalue(const char *p, char *string, size_t len, int *is_null)
 {
 	switch (__zbx_json_type(p))
 	{
 		case ZBX_JSON_TYPE_STRING:
+			if (NULL != is_null)
+				*is_null = 0;
 			return zbx_json_decodestring(p, string, len);
 		case ZBX_JSON_TYPE_INT:
+			if (NULL != is_null)
+				*is_null = 0;
 			return zbx_json_decodeint(p, string, len);
+		case ZBX_JSON_TYPE_NULL:
+			if (NULL != is_null)
+				*is_null = 1;
+			*string = '\0';
+			return zbx_json_decodenull(p);
+		default:
+			return NULL;
+	}
+}
+
+static const char	*zbx_json_decodevalue_dyn(const char *p, char **string, size_t *string_alloc)
+{
+	zbx_json_type_t	jt;
+	size_t		sz;
+
+	jt = __zbx_json_type(p);
+
+	if ((size_t)(-1) == (sz = zbx_json_value_size(p, jt)))
+		return NULL;
+
+	if (*string_alloc <= sz)
+	{
+		*string_alloc = sz + 1;
+		*string = zbx_realloc(*string, *string_alloc);
+	}
+
+	switch (jt)
+	{
+		case ZBX_JSON_TYPE_STRING:
+			return zbx_json_decodestring(p, *string, *string_alloc);
+		case ZBX_JSON_TYPE_INT:
+			return zbx_json_decodeint(p, *string, *string_alloc);
 		default:
 			return NULL;
 	}
@@ -773,6 +882,29 @@ const char	*zbx_json_pair_by_name(struct zbx_json_parse *jp, const char *name)
 	return NULL;
 }
 
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_json_next_value                                              *
+ *                                                                            *
+ * Purpose:                                                                   *
+ *                                                                            *
+ * Parameters:                                                                *
+ *                                                                            *
+ * Return value:                                                              *
+ *                                                                            *
+ * Author: Alexander Vladishev                                                *
+ *                                                                            *
+ * Comments:                                                                  *
+ *                                                                            *
+ ******************************************************************************/
+const char	*zbx_json_next_value(struct zbx_json_parse *jp, const char *p, char *string, size_t len, int *is_null)
+{
+	if (NULL == (p = zbx_json_next(jp, p)))
+		return NULL;
+
+	return zbx_json_decodevalue(p, string, len, is_null);
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: zbx_json_value_by_name                                           *
@@ -781,9 +913,7 @@ const char	*zbx_json_pair_by_name(struct zbx_json_parse *jp, const char *name)
  *                                                                            *
  * Parameters:                                                                *
  *                                                                            *
- * Return value: pointer to value                                             *
- *        {"name":["a","b",...]}                                              *
- *                ^ - returned pointer                                        *
+ * Return value: SUCCEED - if value successfully parsed, FAIL - otherwise     *
  *                                                                            *
  * Author: Alexander Vladishev                                                *
  *                                                                            *
@@ -797,7 +927,35 @@ int	zbx_json_value_by_name(struct zbx_json_parse *jp, const char *name, char *st
 	if (NULL == (p = zbx_json_pair_by_name(jp, name)))
 		return FAIL;
 
-	if (NULL == zbx_json_decodevalue(p, string, len))
+	if (NULL == zbx_json_decodevalue(p, string, len, NULL))
+		return FAIL;
+
+	return SUCCEED;
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: zbx_json_value_by_name_dyn                                       *
+ *                                                                            *
+ * Purpose: return value by pair name                                         *
+ *                                                                            *
+ * Parameters:                                                                *
+ *                                                                            *
+ * Return value: SUCCEED - if value successfully parsed, FAIL - otherwise     *
+ *                                                                            *
+ * Author: Alexander Vladishev                                                *
+ *                                                                            *
+ * Comments:                                                                  *
+ *                                                                            *
+ ******************************************************************************/
+int	zbx_json_value_by_name_dyn(struct zbx_json_parse *jp, const char *name, char **string, size_t *string_alloc)
+{
+	const char	*p;
+
+	if (NULL == (p = zbx_json_pair_by_name(jp, name)))
+		return FAIL;
+
+	if (NULL == zbx_json_decodevalue_dyn(p, string, string_alloc))
 		return FAIL;
 
 	return SUCCEED;
