@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,17 +9,16 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
 #include "log.h"
-#include "zlog.h"
 #include "comms.h"
 #include "base64.h"
 
@@ -52,9 +51,9 @@ void	str_base64_encode_rfc2047(const char *src, char **p_base64)
 {
 	const char	*p0;			/* pointer in src to start encoding from */
 	const char	*p1;			/* pointer in src: 1st byte of UTF-8 character */
-	int		c_len;			/* length of UTF-8 character sequence */
-	int	 	p_base64_alloc;		/* allocated memory size for p_base64 */
-	int		p_base64_offset = 0;	/* offset for writing into p_base64 */
+	size_t		c_len;			/* length of UTF-8 character sequence */
+	size_t		p_base64_alloc;		/* allocated memory size for subject */
+	size_t		p_base64_offset = 0;	/* offset for writing into subject */
 
 	assert(src);
 	assert(NULL == *p_base64);		/* do not accept already allocated memory */
@@ -92,8 +91,7 @@ void	str_base64_encode_rfc2047(const char *src, char **p_base64)
 						ZBX_EMAIL_ENCODED_WORD_SEPARATOR);
 			}
 
-			zbx_snprintf_alloc(p_base64, &p_base64_alloc, &p_base64_offset,
-					ZBX_EMAIL_B64_MAXWORD_RFC2047 + 1, "=?UTF-8?B?%s?=", b64_buf);
+			zbx_snprintf_alloc(p_base64, &p_base64_alloc, &p_base64_offset, "=?UTF-8?B?%s?=", b64_buf);
 		}
 		else
 			break;
@@ -268,8 +266,6 @@ int	send_email(const char *smtp_server, const char *smtp_helo, const char *smtp_
 
 	if (FAIL == is_ascii_string(localsubject))
 	{
-		zbx_replace_invalid_utf8(localsubject);
-
 		/* split subject into multiple RFC 2047 "encoded-words" */
 		str_base64_encode_rfc2047(localsubject, &base64);
 		zbx_free(localsubject);
@@ -361,10 +357,7 @@ out:
 	zbx_tcp_close(&s);
 close:
 	if ('\0' != *error)
-	{
 		zabbix_log(LOG_LEVEL_DEBUG, "%s", error);
-		zabbix_syslog("%s", error);
-	}
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
 
