@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "zbxself.h"
@@ -52,11 +52,13 @@ static ZBX_MUTEX	sm_lock;
 
 extern char	*CONFIG_FILE;
 extern int	CONFIG_POLLER_FORKS;
-extern int	CONFIG_PINGER_FORKS;
-extern int	CONFIG_IPMIPOLLER_FORKS;
 extern int	CONFIG_UNREACHABLE_POLLER_FORKS;
+extern int	CONFIG_IPMIPOLLER_FORKS;
+extern int	CONFIG_PINGER_FORKS;
+extern int	CONFIG_JAVAPOLLER_FORKS;
 extern int	CONFIG_HTTPPOLLER_FORKS;
 extern int	CONFIG_TRAPPER_FORKS;
+extern int	CONFIG_SNMPTRAPPER_FORKS;
 extern int	CONFIG_PROXYPOLLER_FORKS;
 extern int	CONFIG_ESCALATOR_FORKS;
 extern int	CONFIG_HISTSYNCER_FORKS;
@@ -96,10 +98,14 @@ int	get_process_type_forks(unsigned char process_type)
 			return CONFIG_IPMIPOLLER_FORKS;
 		case ZBX_PROCESS_TYPE_PINGER:
 			return CONFIG_PINGER_FORKS;
+		case ZBX_PROCESS_TYPE_JAVAPOLLER:
+			return CONFIG_JAVAPOLLER_FORKS;
 		case ZBX_PROCESS_TYPE_HTTPPOLLER:
 			return CONFIG_HTTPPOLLER_FORKS;
 		case ZBX_PROCESS_TYPE_TRAPPER:
 			return CONFIG_TRAPPER_FORKS;
+		case ZBX_PROCESS_TYPE_SNMPTRAPPER:
+			return CONFIG_SNMPTRAPPER_FORKS;
 		case ZBX_PROCESS_TYPE_PROXYPOLLER:
 			return CONFIG_PROXYPOLLER_FORKS;
 		case ZBX_PROCESS_TYPE_ESCALATOR:
@@ -157,10 +163,14 @@ const char	*get_process_type_string(unsigned char process_type)
 			return "ipmi poller";
 		case ZBX_PROCESS_TYPE_PINGER:
 			return "icmp pinger";
+		case ZBX_PROCESS_TYPE_JAVAPOLLER:
+			return "java poller";
 		case ZBX_PROCESS_TYPE_HTTPPOLLER:
 			return "http poller";
 		case ZBX_PROCESS_TYPE_TRAPPER:
 			return "trapper";
+		case ZBX_PROCESS_TYPE_SNMPTRAPPER:
+			return "snmp trapper";
 		case ZBX_PROCESS_TYPE_PROXYPOLLER:
 			return "proxy poller";
 		case ZBX_PROCESS_TYPE_ESCALATOR:
