@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,16 +9,17 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
 #include "sysinfo.h"
+#include "zbxjson.h"
 
 static int	get_fs_size_stat(const char *fs, zbx_uint64_t *total, zbx_uint64_t *free,
 		zbx_uint64_t *used, double *pfree, double *pused)
@@ -126,14 +127,14 @@ static int	VFS_FS_PUSED(const char *fs, AGENT_RESULT *result)
 
 int	VFS_FS_SIZE(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
 {
-	MODE_FUNCTION fl[] =
+	const MODE_FUNCTION	fl[] =
 	{
 		{"free",	VFS_FS_FREE},
 		{"total",	VFS_FS_TOTAL},
 		{"used",	VFS_FS_USED},
 		{"pfree",	VFS_FS_PFREE},
 		{"pused",	VFS_FS_PUSED},
-		{0,		0}
+		{NULL,		0}
 	};
 
 	char	fsname[MAX_STRING_LEN], mode[8];
@@ -158,3 +159,61 @@ int	VFS_FS_SIZE(const char *cmd, const char *param, unsigned flags, AGENT_RESULT
 
 	return SYSINFO_RET_FAIL;
 }
+
+static const char	*zbx_get_vfs_name_by_type(int type)
+{
+	extern struct vfs_ent	*getvfsbytype(int type);
+
+	struct vfs_ent	*vfs;
+	static char	*vfs_names[MNT_AIXLAST + 1] = {};
+
+	if (NULL == vfs_names[type] && NULL != (vfs = getvfsbytype(type)))
+		vfs_names[type] = zbx_strdup(vfs_names[type], vfs->vfsent_name);
+
+	return NULL != vfs_names[type] ? vfs_names[type] : "unknown";
+}
+
+int	VFS_FS_DISCOVERY(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
+{
+	int		rc, sz, i, ret = SYSINFO_RET_FAIL;
+	struct vmount	*vm = NULL;
+	struct zbx_json	j;
+
+	/* check how many bytes to allocate for the mounted filesystems */
+	if (-1 == (rc = mntctl(MCTL_QUERY, sizeof(sz), (char *)&sz)))
+		return ret;
+
+	vm = zbx_malloc(vm, (size_t)sz);
+
+	/* get the list of mounted filesystems */
+	/* return code is number of filesystems returned */
+	if (-1 == (rc = mntctl(MCTL_QUERY, sz, (char *)vm)))
+		goto error;
+
+	zbx_json_init(&j, ZBX_JSON_STAT_BUF_LEN);
+
+	zbx_json_addarray(&j, ZBX_PROTO_TAG_DATA);
+
+	for (i = 0; i < rc; i++)
+	{
+		zbx_json_addobject(&j, NULL);
+		zbx_json_addstring(&j, "{#FSNAME}", (char *)vm + vm->vmt_data[VMT_STUB].vmt_off, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&j, "{#FSTYPE}", zbx_get_vfs_name_by_type(vm->vmt_gfstype), ZBX_JSON_TYPE_STRING);
+		zbx_json_close(&j);
+
+		/* go to the next vmount structure */
+		vm = (struct vmount *)((char *)vm + vm->vmt_length);
+	}
+
+	zbx_json_close(&j);
+
+	SET_STR_RESULT(result, zbx_strdup(NULL, j.buffer));
+
+	zbx_json_free(&j);
+
+	ret = SYSINFO_RET_OK;
+error:
+	zbx_free(vm);
+
+	return ret;
+}
