@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,17 +9,21 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
 #include "sysinfo.h"
 
+#if !defined(HAVE_SYS_PROCFS_H)
+#	include "../common/common.h"
+#endif
+
 #define DO_SUM 0
 #define DO_MAX 1
 #define DO_MIN 2
@@ -206,7 +210,7 @@ int	PROC_MEM(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *r
 		}
 
 		value = procsinfo.pi_size;
-		value <<= 10;	/* kB to Byte */
+		value <<= 12;	/* number of pages to bytes */
 
 		if (0 == proccount++)
 			memsize = value;
@@ -249,7 +253,6 @@ int	PROC_NUM(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *r
 	AGENT_RESULT		proc_args;
 #endif /* HAVE_SYS_PROCFS_H */
 	struct passwd		*usrinfo = NULL;
-	zbx_uint64_t		value = 0;
 	int			zbx_proc_stat;
 	zbx_uint64_t		proccount = 0;
 
