@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
@@ -28,24 +28,26 @@ ZBX_METRIC	parameters_specific[] =
 
 	{"vfs.fs.size",		CF_USEUPARAM,	VFS_FS_SIZE,		0,	"/,free"},
 	{"vfs.fs.inode",	CF_USEUPARAM,	VFS_FS_INODE,		0,	"/,free"},
+	{"vfs.fs.discovery",	0,		VFS_FS_DISCOVERY,	0,	0},
 
-	{"vfs.dev.read",	CF_USEUPARAM,	VFS_DEV_READ,		0,	"hda,ops,avg1"},
-	{"vfs.dev.write",	CF_USEUPARAM,	VFS_DEV_WRITE,		0,	"hda,ops,avg1"},
+	{"vfs.dev.read",	CF_USEUPARAM,	VFS_DEV_READ,		NULL,	"da0,operations"},
+	{"vfs.dev.write",	CF_USEUPARAM,	VFS_DEV_WRITE,		NULL,	"da0,operations"},
 
-	{"net.tcp.listen",      CF_USEUPARAM,   NET_TCP_LISTEN, 	0,      "80"},
+	{"net.tcp.listen",      CF_USEUPARAM,   NET_TCP_LISTEN, 	0,	"80"},
+	{"net.udp.listen",      CF_USEUPARAM,   NET_UDP_LISTEN, 	0,	"68"},
 
 	{"net.if.in",		CF_USEUPARAM,	NET_IF_IN,		0,	"lo0,bytes"},
 	{"net.if.out",		CF_USEUPARAM,	NET_IF_OUT,		0,	"lo0,bytes"},
 	{"net.if.total",	CF_USEUPARAM,	NET_IF_TOTAL,		0,	"lo0,bytes"},
-	{"net.if.collisions",   CF_USEUPARAM,   NET_IF_COLLISIONS,      0,      "lo0"},
+	{"net.if.collisions",   CF_USEUPARAM,   NET_IF_COLLISIONS,      0,	"lo0"},
 
 	{"vm.memory.size",	CF_USEUPARAM,	VM_MEMORY_SIZE,		0,	"free"},
 
 	{"proc.num",		CF_USEUPARAM,	PROC_NUM,		0,	"inetd,,"},
 	{"proc.mem",		CF_USEUPARAM,	PROC_MEM,		0,	"inetd,,"},
 
-	{"system.cpu.switches", 0,              SYSTEM_CPU_SWITCHES,    0,      0},
-	{"system.cpu.intr",     0,              SYSTEM_CPU_INTR,        0,      0},
+	{"system.cpu.switches",	0,		SYSTEM_CPU_SWITCHES,	0,	0},
+	{"system.cpu.intr",	0,		SYSTEM_CPU_INTR,	0,	0},
 	{"system.cpu.util",	CF_USEUPARAM,	SYSTEM_CPU_UTIL,	0,	"all,user,avg1"},
 	{"system.cpu.load",	CF_USEUPARAM,	SYSTEM_CPU_LOAD,	0,	"all,avg1"},
 	{"system.cpu.num",	CF_USEUPARAM,	SYSTEM_CPU_NUM,		0,	"online"},
@@ -53,7 +55,7 @@ ZBX_METRIC	parameters_specific[] =
 	{"system.swap.size",	CF_USEUPARAM,	SYSTEM_SWAP_SIZE,	0,	"all,free"},
 
 	{"system.uptime",	0,		SYSTEM_UPTIME,		0,	0},
-	{"system.boottime",     0,      	SYSTEM_BOOTTIME,        0,      0},
+	{"system.boottime",	0,		SYSTEM_BOOTTIME,	0,	0},
 
 	{0}
 };
