@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,23 +9,24 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
 #include "sysinfo.h"
+#include "../common/common.h"
 
 static struct ifmibdata	ifmd;
 
-static int	get_ifmib_general(char *if_name)
+static int	get_ifmib_general(const char *if_name)
 {
-	int	ifcount, mib[6];
-	size_t	sz;
+	int	mib[6], ifcount;
+	size_t	len;
 
 	mib[0] = CTL_NET;
 	mib[1] = PF_LINK;
@@ -33,20 +34,23 @@ static int	get_ifmib_general(char *if_name)
 	mib[3] = IFMIB_SYSTEM;
 	mib[4] = IFMIB_IFCOUNT;
 
-	sz = sizeof(ifcount);
-	if (-1 == sysctl(mib, 5, &ifcount, &sz, 0, 0))
+	len = sizeof(ifcount);
+
+	if (-1 == sysctl(mib, 5, &ifcount, &len, NULL, 0))
 		return FAIL;
 
 	mib[3] = IFMIB_IFDATA;
 	mib[5] = IFDATA_GENERAL;
 
-	sz = sizeof(ifmd);
+	len = sizeof(ifmd);
+
 	for (mib[4] = 1; mib[4] <= ifcount; mib[4]++)
 	{
-		if (-1 == sysctl(mib, 6, &ifmd, &sz, 0, 0))
+		if (-1 == sysctl(mib, 6, &ifmd, &len, NULL, 0))
 		{
-			if (errno == ENOENT)
+			if (ENOENT == errno)
 				continue;
+
 			break;
 		}
 
@@ -61,7 +65,7 @@ int	NET_IF_IN(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *
 {
 	char	if_name[MAX_STRING_LEN], mode[32];
 
-	if (num_param(param) > 2)
+	if (2 < num_param(param))
 		return SYSINFO_RET_FAIL;
 
 	if (0 != get_param(param, 1, if_name, sizeof(if_name)))
@@ -74,21 +78,13 @@ int	NET_IF_IN(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *
 		return SYSINFO_RET_FAIL;
 
 	if ('\0' == *mode || 0 == strcmp(mode, "bytes"))	/* default parameter */
-	{
 		SET_UI64_RESULT(result, ifmd.ifmd_data.ifi_ibytes);
-	}
 	else if (0 == strcmp(mode, "packets"))
-	{
 		SET_UI64_RESULT(result, ifmd.ifmd_data.ifi_ipackets);
-	}
 	else if (0 == strcmp(mode, "errors"))
-	{
 		SET_UI64_RESULT(result, ifmd.ifmd_data.ifi_ierrors);
-	}
 	else if (0 == strcmp(mode, "dropped"))
-	{
 		SET_UI64_RESULT(result, ifmd.ifmd_data.ifi_iqdrops);
-	}
 	else
 		return SYSINFO_RET_FAIL;
 
@@ -99,7 +95,7 @@ int	NET_IF_OUT(const char *cmd, const char *param, unsigned flags, AGENT_RESULT
 {
 	char	if_name[MAX_STRING_LEN], mode[32];
 
-	if (num_param(param) > 2)
+	if (2 < num_param(param))
 		return SYSINFO_RET_FAIL;
 
 	if (0 != get_param(param, 1, if_name, sizeof(if_name)))
@@ -112,17 +108,11 @@ int	NET_IF_OUT(const char *cmd, const char *param, unsigned flags, AGENT_RESULT
 		return SYSINFO_RET_FAIL;
 
 	if ('\0' == *mode || 0 == strcmp(mode, "bytes"))	/* default parameter */
-	{
 		SET_UI64_RESULT(result, ifmd.ifmd_data.ifi_obytes);
-	}
 	else if (0 == strcmp(mode, "packets"))
-	{
 		SET_UI64_RESULT(result, ifmd.ifmd_data.ifi_opackets);
-	}
 	else if (0 == strcmp(mode, "errors"))
-	{
 		SET_UI64_RESULT(result, ifmd.ifmd_data.ifi_oerrors);
-	}
 	else
 		return SYSINFO_RET_FAIL;
 
@@ -133,7 +123,7 @@ int	NET_IF_TOTAL(const char *cmd, const char *param, unsigned flags, AGENT_RESUL
 {
 	char	if_name[MAX_STRING_LEN], mode[32];
 
-	if (num_param(param) > 2)
+	if (2 < num_param(param))
 		return SYSINFO_RET_FAIL;
 
 	if (0 != get_param(param, 1, if_name, sizeof(if_name)))
@@ -146,17 +136,11 @@ int	NET_IF_TOTAL(const char *cmd, const char *param, unsigned flags, AGENT_RESUL
 		return SYSINFO_RET_FAIL;
 
 	if ('\0' == *mode || 0 == strcmp(mode, "bytes"))	/* default parameter */
-	{
-		SET_UI64_RESULT(result, (zbx_uint64_t)ifmd.ifmd_data.ifi_ibytes + (zbx_uint64_t)ifmd.ifmd_data.ifi_obytes);
-	}
+		SET_UI64_RESULT(result, (zbx_uint64_t)ifmd.ifmd_data.ifi_ibytes + ifmd.ifmd_data.ifi_obytes);
 	else if (0 == strcmp(mode, "packets"))
-	{
-		SET_UI64_RESULT(result, (zbx_uint64_t)ifmd.ifmd_data.ifi_ipackets + (zbx_uint64_t)ifmd.ifmd_data.ifi_opackets);
-	}
+		SET_UI64_RESULT(result, (zbx_uint64_t)ifmd.ifmd_data.ifi_ipackets + ifmd.ifmd_data.ifi_opackets);
 	else if (0 == strcmp(mode, "errors"))
-	{
-		SET_UI64_RESULT(result, (zbx_uint64_t)ifmd.ifmd_data.ifi_ierrors + (zbx_uint64_t)ifmd.ifmd_data.ifi_oerrors);
-	}
+		SET_UI64_RESULT(result, (zbx_uint64_t)ifmd.ifmd_data.ifi_ierrors + ifmd.ifmd_data.ifi_oerrors);
 	else
 		return SYSINFO_RET_FAIL;
 
@@ -169,7 +153,7 @@ int     NET_TCP_LISTEN(const char *cmd, const char *param, unsigned flags, AGENT
 	unsigned short	port;
 	int		res;
 
-	if (num_param(param) > 1)
+	if (1 < num_param(param))
 		return SYSINFO_RET_FAIL;
 
 	if (0 != get_param(param, 1, tmp, sizeof(tmp)))
@@ -178,12 +162,38 @@ int     NET_TCP_LISTEN(const char *cmd, const char *param, unsigned flags, AGENT
 	if (FAIL == is_ushort(tmp, &port))
 		return SYSINFO_RET_FAIL;
 
-	zbx_snprintf(command, sizeof(command), "netstat -an | grep '*.%hu\\>' | wc -l", port);
+	zbx_snprintf(command, sizeof(command), "netstat -an | grep '^tcp.*\\.%hu[^.].*LISTEN' | wc -l", port);
 
 	if (SYSINFO_RET_FAIL == (res = EXECUTE_INT(NULL, command, flags, result)))
 		return res;
 
-	if (result->ui64 > 1)
+	if (1 < result->ui64)
+		result->ui64 = 1;
+
+	return res;
+}
+
+int     NET_UDP_LISTEN(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
+{
+	char		tmp[8], command[64];
+	unsigned short	port;
+	int		res;
+
+	if (1 < num_param(param))
+		return SYSINFO_RET_FAIL;
+
+	if (0 != get_param(param, 1, tmp, sizeof(tmp)))
+		return SYSINFO_RET_FAIL;
+
+	if (FAIL == is_ushort(tmp, &port))
+		return SYSINFO_RET_FAIL;
+
+	zbx_snprintf(command, sizeof(command), "netstat -an | grep '^udp.*\\.%hu[^.].*\\*\\.\\*' | wc -l", port);
+
+	if (SYSINFO_RET_FAIL == (res = EXECUTE_INT(NULL, command, flags, result)))
+		return res;
+
+	if (1 < result->ui64)
 		result->ui64 = 1;
 
 	return res;
@@ -191,9 +201,9 @@ int     NET_TCP_LISTEN(const char *cmd, const char *param, unsigned flags, AGENT
 
 int     NET_IF_COLLISIONS(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
 {
-	char	if_name[MAX_STRING_LEN], mode[32];
+	char	if_name[MAX_STRING_LEN];
 
-	if (num_param(param) > 1)
+	if (1 < num_param(param))
 		return SYSINFO_RET_FAIL;
 
 	if (0 != get_param(param, 1, if_name, sizeof(if_name)))
