@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,44 +9,52 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
 #include "sysinfo.h"
 
-static void	get_swapinfo(double *total, double *fr)
+/******************************************************************************
+ *                                                                            *
+ * Function: get_swapinfo                                                     *
+ *                                                                            *
+ * Purpose: get swap usage statistics                                         *
+ *                                                                            *
+ * Return value: SUCCEED if swap usage statistics retrieved successfully      *
+ *               FAIL otherwise                                               *
+ *                                                                            *
+ * Comments: we try to imitate "swap -l".                                     *
+ *                                                                            *
+ ******************************************************************************/
+static int	get_swapinfo(zbx_uint64_t *total, zbx_uint64_t *free1)
 {
-	register int cnt, i, page_size;
-/* Support for >2Gb */
-/*	register int t, f;*/
-	double	t, f;
-	struct swaptable *swt;
-	struct swapent *ste;
-	static char path[256];
+	int			i, cnt, cnt2, page_size, ret = SUCCEED;
+	struct swaptable	*swt = NULL;
+	struct swapent		*ste;
+	static char		path[256];
 
 	/* get total number of swap entries */
-	cnt = swapctl(SC_GETNSWP, 0);
+	if (-1 == (cnt = swapctl(SC_GETNSWP, 0)))
+		return FAIL;
 
-	/* allocate enough space to hold count + n swapents */
-	swt = (struct swaptable *)malloc(sizeof(int) +
-		cnt * sizeof(struct swapent));
-
-	if (swt == NULL)
+	if (0 == cnt)
 	{
-		*total = 0;
-		*fr = 0;
-		return;
+		*total = *free1 = 0;
+		return SUCCEED;
 	}
+
+	/* allocate space to hold count + n swapents */
+	swt = (struct swaptable *)zbx_malloc(swt, sizeof(struct swaptable) + (cnt - 1) * sizeof(struct swapent));
+
 	swt->swt_n = cnt;
 
-/* fill in ste_path pointers: we don't care about the paths, so we
-point them all to the same buffer */
+	/* fill in ste_path pointers: we don't care about the paths, so we point them all to the same buffer */
 	ste = &(swt->swt_ent[0]);
 	i = cnt;
 	while (--i >= 0)
@@ -55,162 +63,120 @@ point them all to the same buffer */
 	}
 
 	/* grab all swap info */
-	swapctl(SC_LIST, swt);
+	if (-1 == (cnt2 = swapctl(SC_LIST, swt)) || cnt != cnt2)
+	{
+		ret = FAIL;
+		goto finish;
+	}
 
 	/* walk through the structs and sum up the fields */
-	t = f = 0;
+	*total = *free1 = 0;
 	ste = &(swt->swt_ent[0]);
 	i = cnt;
 	while (--i >= 0)
 	{
 		/* don't count slots being deleted */
-		if (!(ste->ste_flags & ST_INDEL) &&
-		!(ste->ste_flags & ST_DOINGDEL))
+		if (0 == (ste->ste_flags & (ST_INDEL | ST_DOINGDEL)))
 		{
-			t += ste->ste_pages;
-			f += ste->ste_free;
+			*total += ste->ste_pages;
+			*free1 += ste->ste_free;
 		}
 		ste++;
 	}
 
-	page_size=getpagesize();
+	page_size = getpagesize();
 
 	/* fill in the results */
-	*total = page_size*t;
-	*fr = page_size*f;
-	free(swt);
+	*total *= page_size;
+	*free1 *= page_size;
+finish:
+	zbx_free(swt);
+
+	return ret;
 }
 
-int	SYSTEM_SWAP_FREE(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
+static int	SYSTEM_SWAP_TOTAL(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
 {
-	double	swaptotal, swapfree;
+	zbx_uint64_t	total, free1;
 
-	get_swapinfo(&swaptotal, &swapfree);
+	if (SUCCEED != get_swapinfo(&total, &free1))
+		return SYSINFO_RET_FAIL;
+
+	SET_UI64_RESULT(result, total);
 
-	SET_UI64_RESULT(result, swapfree);
 	return SYSINFO_RET_OK;
 }
 
-int	SYSTEM_SWAP_TOTAL(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
+static int	SYSTEM_SWAP_FREE(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
 {
-	double	swaptotal, swapfree;
+	zbx_uint64_t	total, free1;
 
-	get_swapinfo(&swaptotal, &swapfree);
-
-	SET_UI64_RESULT(result, swaptotal);
-	return SYSINFO_RET_OK;
-}
-
-static int	SYSTEM_SWAP_PFREE(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
-{
-	AGENT_RESULT	result_tmp;
-	zbx_uint64_t	tot_val = 0;
-	zbx_uint64_t	free_val = 0;
-
-        init_result(&result_tmp);
-
-	if(SYSTEM_SWAP_TOTAL(cmd, param, flags, &result_tmp) != SYSINFO_RET_OK || !(result_tmp.type & AR_UINT64))
+	if (SUCCEED != get_swapinfo(&total, &free1))
 		return SYSINFO_RET_FAIL;
-	tot_val = result_tmp.ui64;
 
-	/* Check for division by zero */
-	if(tot_val == 0)
-	{
-		free_result(&result_tmp);
-		return SYSINFO_RET_FAIL;
-	}
-
-	if(SYSTEM_SWAP_FREE(cmd, param, flags, &result_tmp) != SYSINFO_RET_OK || !(result_tmp.type & AR_UINT64))
-		return SYSINFO_RET_FAIL;
-	free_val = result_tmp.ui64;
-
-	free_result(&result_tmp);
-
-	SET_DBL_RESULT(result, (100.0 * (double)free_val) / (double)tot_val);
+	SET_UI64_RESULT(result, free1);
 
 	return SYSINFO_RET_OK;
 }
 
 static int	SYSTEM_SWAP_PUSED(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
 {
-	AGENT_RESULT	result_tmp;
-	zbx_uint64_t	tot_val = 0;
-	zbx_uint64_t	free_val = 0;
+	zbx_uint64_t	total, free1;
 
-        init_result(&result_tmp);
-
-	if(SYSTEM_SWAP_TOTAL(cmd, param, flags, &result_tmp) != SYSINFO_RET_OK || !(result_tmp.type & AR_UINT64))
+	if (SUCCEED != get_swapinfo(&total, &free1))
 		return SYSINFO_RET_FAIL;
-	tot_val = result_tmp.ui64;
 
-	/* Check for division by zero */
-	if(tot_val == 0)
-	{
-		free_result(&result_tmp);
+	if (0 == total)
 		return SYSINFO_RET_FAIL;
-	}
 
-	if(SYSTEM_SWAP_FREE(cmd, param, flags, &result_tmp) != SYSINFO_RET_OK || !(result_tmp.type & AR_UINT64))
+	SET_DBL_RESULT(result, 100.0 * (double)(total - free1) / (double)total);
+
+	return SYSINFO_RET_OK;
+}
+
+static int	SYSTEM_SWAP_PFREE(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
+{
+	zbx_uint64_t	total, free1;
+
+	if (SUCCEED != get_swapinfo(&total, &free1))
 		return SYSINFO_RET_FAIL;
-	free_val = result_tmp.ui64;
 
-	free_result(&result_tmp);
+	if (0 == total)
+		return SYSINFO_RET_FAIL;
 
-	SET_DBL_RESULT(result, 100.0 - (100.0 * (double)free_val) / (double)tot_val);
+	SET_DBL_RESULT(result, 100.0 * (double)free1 / (double)total);
 
-        return SYSINFO_RET_OK;
+	return SYSINFO_RET_OK;
 }
 
 int	SYSTEM_SWAP_SIZE(const char *cmd, const char *param, unsigned flags, AGENT_RESULT *result)
 {
-	MODE_FUNCTION fl[] =
+	const MODE_FUNCTION	fl[] =
 	{
 		{"total",	SYSTEM_SWAP_TOTAL},
 		{"free",	SYSTEM_SWAP_FREE},
-		{"pfree",       SYSTEM_SWAP_PFREE},
-		{"pused",       SYSTEM_SWAP_PUSED},
-		{0,		0}
+		{"pused",	SYSTEM_SWAP_PUSED},
+		{"pfree",	SYSTEM_SWAP_PFREE},
+		{NULL,		0}
 	};
 
-	char swapdev[MAX_STRING_LEN];
-	char mode[MAX_STRING_LEN];
-	int i;
+	char	swapdev[MAX_STRING_LEN], mode[MAX_STRING_LEN];
+	int	i;
 
-        if(num_param(param) > 2)
-        {
-                return SYSINFO_RET_FAIL;
-        }
-
-        if(get_param(param, 1, swapdev, sizeof(swapdev)) != 0)
-        {
-                return SYSINFO_RET_FAIL;
-        }
-
-        if(swapdev[0] == '\0')
-	{
-		/* default parameter */
-		zbx_snprintf(swapdev, sizeof(swapdev), "all");
-	}
-
-	if(strncmp(swapdev, "all", sizeof(swapdev)))
-	{
+	if (2 < num_param(param))
 		return SYSINFO_RET_FAIL;
-	}
 
-	if(get_param(param, 2, mode, sizeof(mode)) != 0)
-        {
-                mode[0] = '\0';
-        }
+	if (0 == get_param(param, 1, swapdev, sizeof(swapdev)) && '\0' != *swapdev && 0 != strcmp("all", swapdev))
+		return SYSINFO_RET_FAIL;	/* first parameter must be one of missing, empty or "all" */
 
-        if(mode[0] == '\0')
+	if (0 != get_param(param, 2, mode, sizeof(mode)) || '\0' == *mode)
+		strscpy(mode, "free");	/* default parameter */
+
+	for (i = 0; NULL != fl[i].mode; i++)
 	{
-		/* default parameter */
-		zbx_snprintf(mode, sizeof(mode), "free");
-	}
-
-	for(i=0; fl[i].mode!=0; i++)
-		if(strncmp(mode, fl[i].mode, MAX_STRING_LEN)==0)
+		if (0 == strcmp(mode, fl[i].mode))
 			return (fl[i].function)(cmd, param, flags, result);
+	}
 
 	return SYSINFO_RET_FAIL;
 }
@@ -220,7 +186,7 @@ int	SYSTEM_SWAP_SIZE(const char *cmd, const char *param, unsigned flags, AGENT_R
 #define	DO_SWP_OUT	3
 #define DO_PG_OUT	4
 
-int	get_swap_io(double *swapin, double *pgswapin, double *swapout, double *pgswapout)
+static int	get_swap_io(double *swapin, double *pgswapin, double *swapout, double *pgswapout)
 {
     kstat_ctl_t	    *kc;
     kstat_t	    *k;
