@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
@@ -23,58 +23,44 @@
 #include "cfg.h"
 #include "alias.h"
 
-#if defined(WITH_AGENT_METRICS)
+#ifdef WITH_AGENT_METRICS
 #	include "agent/agent.h"
 #endif
 
-#if defined(WITH_COMMON_METRICS)
+#ifdef WITH_COMMON_METRICS
 #	include "common/common.h"
 #endif
 
-#if defined(WITH_SIMPLE_METRICS)
+#ifdef WITH_SIMPLE_METRICS
 #	include "simple/simple.h"
 #endif
 
-#if defined(WITH_SPECIFIC_METRICS)
+#ifdef WITH_SPECIFIC_METRICS
 #	include "specsysinfo.h"
 #endif
 
-ZBX_METRIC	*commands = NULL;
+static ZBX_METRIC	*commands = NULL;
 
 void	add_metric(ZBX_METRIC *new)
 {
-	register int i;
+	int	i = 0;
 
 	assert(new);
 
-	if(new->key == NULL)
+	if (NULL == new->key)
 		return;
 
-	for(i=0;;i++)
-	{
-		if(commands[i].key == NULL)
-		{
+	while (NULL != commands[i].key)
+		i++;
 
-			commands[i].key = strdup(new->key);
-			commands[i].flags = new->flags;
+	commands[i].key = zbx_strdup(NULL, new->key);
+	commands[i].flags = new->flags;
+	commands[i].function = new->function;
+	commands[i].main_param = (NULL == new->main_param ? NULL : zbx_strdup(NULL, new->main_param));
+	commands[i].test_param = (NULL == new->test_param ? NULL : zbx_strdup(NULL, new->test_param));
 
-			commands[i].function=new->function;
-
-			if(new->main_param == NULL)
-				commands[i].main_param=NULL;
-			else
-				commands[i].main_param=strdup(new->main_param);
-
-			if(new->test_param == NULL)
-				commands[i].test_param=NULL;
-			else
-				commands[i].test_param=strdup(new->test_param);
-
-			commands = zbx_realloc(commands,(i+2)*sizeof(ZBX_METRIC));
-			memset(&commands[i+1], 0, sizeof(ZBX_METRIC));
-			break;
-		}
-	}
+	commands = zbx_realloc(commands, (i + 2) * sizeof(ZBX_METRIC));
+	memset(&commands[i + 1], 0, sizeof(ZBX_METRIC));
 }
 
 int	add_user_parameter(const char *key, char *command)
@@ -98,14 +84,14 @@ int	add_user_parameter(const char *key, char *command)
 		flag |= CF_USEUPARAM;
 	}
 
-	for (i = 0; ; i++)
+	for (i = 0;; i++)
 	{
 		/* add new parameters */
 		if (NULL == commands[i].key)
 		{
 			commands[i].key = zbx_strdup(NULL, usr_cmd);
 			commands[i].flags = flag;
-			commands[i].function = &EXECUTE_STR;
+			commands[i].function = &EXECUTE_USER_PARAMETER;
 			commands[i].main_param = zbx_strdup(NULL, command);
 			commands[i].test_param = 0;
 
@@ -127,9 +113,9 @@ int	add_user_parameter(const char *key, char *command)
 
 void	init_metrics()
 {
-	register int	i;
+	int	i;
 
-	commands = malloc(sizeof(ZBX_METRIC));
+	commands = zbx_malloc(commands, sizeof(ZBX_METRIC));
 	commands[0].key = NULL;
 
 #ifdef WITH_AGENT_METRICS
@@ -170,36 +156,6 @@ void	free_metrics()
 	}
 }
 
-void    escape_string(char *from, char *to, int maxlen)
-{
-	register int     i,ptr;
-	char    *f;
-
-	ptr=0;
-	f=(char *)strdup(from);
-	for(i=0;f[i]!=0;i++)
-	{
-		if( (f[i]=='\'') || (f[i]=='\\'))
-		{
-			if(ptr>maxlen-1)        break;
-			to[ptr]='\\';
-			if(ptr+1>maxlen-1)      break;
-			to[ptr+1]=f[i];
-			ptr+=2;
-		}
-		else
-		{
-			if(ptr>maxlen-1)        break;
-			to[ptr]=f[i];
-			ptr++;
-		}
-	}
-	free(f);
-
-	to[ptr]=0;
-	to[maxlen-1]=0;
-}
-
 void	init_result(AGENT_RESULT *result)
 {
 	result->type = 0;
@@ -227,57 +183,48 @@ void	free_result(AGENT_RESULT *result)
  */
 int	parse_command(const char *command, char *cmd, size_t cmd_max_len, char *param, size_t param_max_len)
 {
-	char	*pl, *pr;
-	size_t	sz;
+	const char	*pl, *pr;
+	size_t		sz;
 
-	pl = strchr(command, '[');
-	pr = strrchr(command, ']');
+	for (pl = command; SUCCEED == is_key_char(*pl); pl++)
+		;
 
-	if (pl > pr)
+	if (pl == command)
 		return 0;
 
-	if (NULL != pl && NULL == pr)
-		return 0;
-
-	if (NULL == pl && NULL != pr)
-		return 0;
-
-	if (NULL == pl && NULL == pr) /* simple check? */
-	{
-		if (NULL != (pl = strchr(command, ',')))
-		{
-			for (pr = pl + 1; '\0' != *pr; pr++);
-		}
-	}
-
 	if (NULL != cmd)
 	{
-		if (NULL != pl)
-		{
-			if (cmd_max_len < (sz = (size_t)(pl - command) + 1))
-				sz = cmd_max_len;
-			memcpy(cmd, command, sz - 1);
-			cmd[sz - 1] = '\0';
-		}
-		else
-			zbx_strlcpy(cmd, command, cmd_max_len);
+		if (cmd_max_len <= (sz = (size_t)(pl - command)))
+			return 0;
+
+		memcpy(cmd, command, sz);
+		cmd[sz] = '\0';
 	}
 
-	if (NULL != param)
-		*param = '\0';
-
-	if (NULL != pl && NULL != pr)
+	if ('\0' == *pl)	/* no parameters specified */
 	{
 		if (NULL != param)
-		{
-			if (param_max_len < (sz = (size_t)(pr - pl)))
-				sz = param_max_len;
-			memcpy(param, pl + 1, sz - 1);
-			param[sz - 1] = '\0';
-		}
-	}
-	else
+			*param = '\0';
 		return 1;
+	}
+
+	if ('[' != *pl)		/* unsupported character */
+		return 0;
+
+	for (pr = ++pl; '\0' != *pr; pr++)
+		;
+
+	if (']' != *--pr)
+		return 0;
+
+	if (NULL != param)
+	{
+		if (param_max_len <= (sz = (size_t)(pr - pl)))
+			return 0;
+
+		memcpy(param, pl, sz);
+		param[sz] = '\0';
+	}
 
 	return 2;
 }
@@ -290,19 +237,19 @@ void	test_parameter(const char *key, unsigned flags)
 
 	process(key, flags, &result);
 
-	if (result.type & AR_UINT64)
+	if (ISSET_UI64(&result))
 		printf(" [u|" ZBX_FS_UI64 "]", result.ui64);
 
-	if (result.type & AR_DOUBLE)
+	if (ISSET_DBL(&result))
 		printf(" [d|" ZBX_FS_DBL "]", result.dbl);
 
-	if (result.type & AR_STRING)
+	if (ISSET_STR(&result))
 		printf(" [s|%s]", result.str);
 
-	if (result.type & AR_TEXT)
+	if (ISSET_TEXT(&result))
 		printf(" [t|%s]", result.text);
 
-	if (result.type & AR_MESSAGE)
+	if (ISSET_MSG(&result))
 		printf(" [m|%s]", result.msg);
 
 	free_result(&result);
@@ -316,58 +263,61 @@ void	test_parameters()
 {
 	int	i;
 
-	for (i = 0; 0 != commands[i].key; i++)
+	for (i = 0; NULL != commands[i].key; i++)
 		test_parameter(commands[i].key, PROCESS_TEST | PROCESS_USE_TEST_PARAM);
 }
 
-static int	replace_param(const char *cmd, const char *param, char *out, int outlen, char *error, int max_err_len)
+static int	replace_param(const char *cmd, const char *param, char *out, int outlen, char *error, int max_error_len)
 {
-	int ret = SUCCEED;
-	char buf[MAX_STRING_LEN];
-	char command[MAX_STRING_LEN];
-	register char *pl, *pr;
-	const char	suppressed_chars[] = "\\'\"`*?[]{}~$!&;()<>|#@\0", *c;
+	int		ret = SUCCEED;
+	char		buf[MAX_STRING_LEN];
+	char		command[MAX_STRING_LEN];
+	char		*pl, *pr;
+	const char	suppressed_chars[] = "\\'\"`*?[]{}~$!&;()<>|#@", *c;
 
 	assert(out);
 
 	out[0] = '\0';
 
-	if(!cmd && !param)
+	if (NULL == cmd && NULL == param)
 		return ret;
 
-	zbx_strlcpy(command, cmd, MAX_STRING_LEN);
+	strscpy(command, cmd);
 
 	pl = command;
-	while((pr = strchr(pl, '$')) && outlen > 0)
+
+	while (NULL != (pr = strchr(pl, '$')) && outlen > 0)
 	{
 		pr[0] = '\0';
 		zbx_strlcat(out, pl, outlen);
 		outlen -= MIN((int)strlen(pl), (int)outlen);
 		pr[0] = '$';
 
-		if (pr[1] >= '0' && pr[1] <= '9')
+		if ('0' <= pr[1] && pr[1] <= '9')
 		{
 			buf[0] = '\0';
 
-			if(pr[1] == '0')
+			if ('0' == pr[1])
 			{
-				zbx_strlcpy(buf, command, MAX_STRING_LEN);
+				strscpy(buf, command);
 			}
 			else
 			{
-				get_param(param, (int)(pr[1] - '0'), buf, MAX_STRING_LEN);
+				get_param(param, (int)(pr[1] - '0'), buf, sizeof(buf));
 
 				if (0 == CONFIG_UNSAFE_USER_PARAMETERS)
 				{
 					for (c = suppressed_chars; '\0' != *c; c++)
+					{
 						if (NULL != strchr(buf, *c))
 						{
-							zbx_snprintf(error, max_err_len, "Special characters '%s'"
+							zbx_snprintf(error, max_error_len, "Special characters '%s'"
 									" are not allowed in the parameters",
 									suppressed_chars);
 							ret = FAIL;
 							break;
 						}
+					}
 				}
 			}
 
@@ -379,15 +329,17 @@ static int	replace_param(const char *cmd, const char *param, char *out, int outl
 
 			pl = pr + 2;
 			continue;
-		} else if(pr[1] == '$')
+		}
+		else if ('$' == pr[1])
 		{
-			pr++; /* remove second '$' symbol */
+			pr++;	/* remove second '$' symbol */
 		}
 
 		pl = pr + 1;
 		zbx_strlcat(out, "$", outlen);
 		outlen -= 1;
 	}
+
 	zbx_strlcat(out, pl, outlen);
 	outlen -= MIN((int)strlen(pl), (int)outlen);
 
@@ -396,128 +348,90 @@ static int	replace_param(const char *cmd, const char *param, char *out, int outl
 
 int	process(const char *in_command, unsigned flags, AGENT_RESULT *result)
 {
-	char	*p;
-	int	i = 0;
+	int		rc, ret = NOTSUPPORTED;
+	char		usr_cmd[MAX_STRING_LEN];
+	char		usr_param[MAX_STRING_LEN];
+	char		usr_command[MAX_STRING_LEN];
+	char		param[MAX_STRING_LEN], error[MAX_STRING_LEN];
 
-	int	(*function)() = NULL;
-	int	ret = SUCCEED;
-	int	err = SYSINFO_RET_OK;
-
-	char	usr_cmd[MAX_STRING_LEN];
-	char	usr_param[MAX_STRING_LEN];
-
-	char	usr_command[MAX_STRING_LEN];
-	int	usr_command_len;
-
-	char	param[MAX_STRING_LEN], error[MAX_STRING_LEN];
+	ZBX_METRIC	*command = NULL;
 
 	assert(result);
 	init_result(result);
 
-	*error = '\0';
-
 	alias_expand(in_command, usr_command, sizeof(usr_command));
 
-	usr_command_len = (int)strlen(usr_command);
+	if (0 == (rc = parse_command(usr_command, usr_cmd, sizeof(usr_cmd), usr_param, sizeof(usr_param))))
+		goto notsupported;
 
-	for (p = usr_command + usr_command_len - 1; p > usr_command && NULL != strchr(ZBX_WHITESPACE, *p); --p)
-		;
-
-	if (NULL != strchr(ZBX_WHITESPACE, p[1]))
-		p[1] = '\0';
-
-	if (0 != parse_command(usr_command, usr_cmd, sizeof(usr_cmd), usr_param, sizeof(usr_param)))
+	for (command = commands; NULL != command->key; command++)
 	{
-		for (i = 0; NULL != commands[i].key; i++)
-		{
-			if (0 == strcmp(commands[i].key, usr_cmd))
-			{
-				function = commands[i].function;
-				break;
-			}
-		}
+		if (0 == strcmp(command->key, usr_cmd))
+			break;
 	}
 
-	param[0] = '\0';
-
-	if (NULL != function)
+	if (NULL != command->key)
 	{
-		if (0 != (commands[i].flags & CF_USEUPARAM))
+		if (0 != (command->flags & CF_USEUPARAM))
 		{
-			if (0 != (flags & PROCESS_TEST) &&
-					0 != (flags & PROCESS_USE_TEST_PARAM) &&
-					NULL != commands[i].test_param)
-			{
-				zbx_strlcpy(usr_param, commands[i].test_param, sizeof(usr_param));
-			}
+			if (0 != (flags & PROCESS_USE_TEST_PARAM) && NULL != command->test_param)
+				strscpy(usr_param, command->test_param);
 		}
-		else
-			usr_param[0] = '\0';
+		else if (2 == rc)
+			goto notsupported;
 
-		if (NULL != commands[i].main_param)
+		*param = '\0';
+		*error = '\0';
+
+		if (NULL != command->main_param)
 		{
-			if (0 != (commands[i].flags & CF_USEUPARAM))
+			if (0 != (command->flags & CF_USEUPARAM))
 			{
-				err = replace_param(commands[i].main_param, usr_param,
-						param, sizeof(param), error, sizeof(error));
+				if (FAIL == replace_param(command->main_param, usr_param,
+						param, sizeof(param), error, sizeof(error)))
+				{
+					if ('\0' != *error)
+						zabbix_log(LOG_LEVEL_WARNING, "item [%s] error: %s", in_command, error);
+					goto notsupported;
+				}
 			}
 			else
-				zbx_snprintf(param, sizeof(param), "%s", commands[i].main_param);
+				strscpy(param, command->main_param);
 		}
 		else
-			zbx_snprintf(param, sizeof(param), "%s", usr_param);
+			strscpy(param, usr_param);
 
-		if (FAIL != err)
+		if (SYSINFO_RET_OK != command->function(usr_command, param, flags, result))
 		{
-			err = function(usr_command, param, flags, result);
-
-			if (SYSINFO_RET_FAIL == err)
-				err = NOTSUPPORTED;
-		}
-		else
-		{
-			err = NOTSUPPORTED;
-			if ('\0' != *error)
-				zabbix_log(LOG_LEVEL_WARNING, "item [%s] error: %s", in_command, error);
+			/* "return NOTSUPPORTED;" would be more appropriate here for preserving original error */
+			/* message in "result" but would break things relying on ZBX_NOTSUPPORTED message. */
+			goto notsupported;
 		}
 	}
 	else
-		err = NOTSUPPORTED;
+		goto notsupported;
+
+	ret = SUCCEED;
+
+notsupported:
 
 	if (0 != (flags & PROCESS_TEST))
 	{
-		printf("%s", usr_cmd);
+#define	COL_WIDTH	45
 
-		if (0 != (commands[i].flags & CF_USEUPARAM))
-		{
-			printf("[%s]", usr_param);
-			i = 2 + (int)strlen(usr_param);
-		}
-		else
-			i = 0;
+		int	n1, n2 = 0;
 
-		i += (int)strlen(usr_cmd);
+		n1 = printf("%s", usr_cmd);
 
-#define COLUMN_2_X 45	/* max of spaces count */
-		i = i > COLUMN_2_X ? 1 : COLUMN_2_X - i;
+		if (0 < n1 && '\0' != *param)
+			n2 = printf("[%s]", param);
 
-		printf("%-*.*s", i, i, " ");	/* print spaces */
+		if (0 < n1 && 0 <= n2 && COL_WIDTH > n1 + n2)
+			printf("%-*s", COL_WIDTH - n1 - n2, " ");
 	}
 
-	if (NOTSUPPORTED == err)
-	{
-		if (!ISSET_MSG(result))
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "ZBX_NOTSUPPORTED"));
-
-		ret = NOTSUPPORTED;
-	}
-	else if (TIMEOUT_ERROR == err)
-	{
-		if (!ISSET_MSG(result))
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "ZBX_ERROR"));
-
-		ret = TIMEOUT_ERROR;
-	}
+	if (NOTSUPPORTED == ret)
+		SET_MSG_RESULT(result, zbx_strdup(NULL, ZBX_NOTSUPPORTED));
 
 	return ret;
 }
@@ -539,6 +453,13 @@ int	set_result_type(AGENT_RESULT *result, int value_type, int data_type, char *c
 
 			switch (data_type)
 			{
+				case ITEM_DATA_TYPE_BOOLEAN:
+					if (SUCCEED == is_boolean(c, &value_uint64))
+					{
+						SET_UI64_RESULT(result, value_uint64);
+						ret = SUCCEED;
+					}
+					break;
 				case ITEM_DATA_TYPE_OCTAL:
 					if (SUCCEED == is_uoct(c))
 					{
@@ -547,6 +468,13 @@ int	set_result_type(AGENT_RESULT *result, int value_type, int data_type, char *c
 						ret = SUCCEED;
 					}
 					break;
+				case ITEM_DATA_TYPE_DECIMAL:
+					if (SUCCEED == is_uint64(c, &value_uint64))
+					{
+						SET_UI64_RESULT(result, value_uint64);
+						ret = SUCCEED;
+					}
+					break;
 				case ITEM_DATA_TYPE_HEXADECIMAL:
 					if (SUCCEED == is_uhex(c))
 					{
@@ -562,14 +490,10 @@ int	set_result_type(AGENT_RESULT *result, int value_type, int data_type, char *c
 						ret = SUCCEED;
 					}
 					break;
-				default:	/* ITEM_DATA_TYPE_DECIMAL */
-					if (SUCCEED == is_uint64(c, &value_uint64))
-					{
-						SET_UI64_RESULT(result, value_uint64);
-						ret = SUCCEED;
-					}
+				default:
+					THIS_SHOULD_NEVER_HAPPEN;
+					break;
 			}
-
 			break;
 		case ITEM_VALUE_TYPE_FLOAT:
 			zbx_rtrim(c, " \"");
@@ -597,30 +521,41 @@ int	set_result_type(AGENT_RESULT *result, int value_type, int data_type, char *c
 
 	if (SUCCEED != ret)
 	{
+		char	*error = NULL;
+
 		zbx_remove_chars(c, "\r\n");
 		zbx_replace_invalid_utf8(c);
-		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "Type of received value [%s] is not suitable for value type [%s]",
-				c, zbx_item_value_type_string(value_type)));
+
+		if (ITEM_VALUE_TYPE_UINT64 == value_type)
+			error = zbx_dsprintf(error,
+					"Received value [%s] is not suitable for value type [%s] and data type [%s]",
+					c, zbx_item_value_type_string(value_type), zbx_item_data_type_string(data_type));
+		else
+			error = zbx_dsprintf(error,
+					"Received value [%s] is not suitable for value type [%s]",
+					c, zbx_item_value_type_string(value_type));
+
+		SET_MSG_RESULT(result, error);
 	}
 
 	return ret;
 }
 
-static zbx_uint64_t* get_result_ui64_value(AGENT_RESULT *result)
+static zbx_uint64_t	*get_result_ui64_value(AGENT_RESULT *result)
 {
 	zbx_uint64_t	value;
 
 	assert(result);
 
-	if(ISSET_UI64(result))
+	if (ISSET_UI64(result))
 	{
 		/* nothing to do */
 	}
-	else if(ISSET_DBL(result))
+	else if (ISSET_DBL(result))
 	{
 		SET_UI64_RESULT(result, result->dbl);
 	}
-	else if(ISSET_STR(result))
+	else if (ISSET_STR(result))
 	{
 		zbx_rtrim(result->str, " \"");
 		zbx_ltrim(result->str, " \"+");
@@ -631,7 +566,7 @@ static zbx_uint64_t* get_result_ui64_value(AGENT_RESULT *result)
 
 		SET_UI64_RESULT(result, value);
 	}
-	else if(ISSET_TEXT(result))
+	else if (ISSET_TEXT(result))
 	{
 		zbx_rtrim(result->text, " \"");
 		zbx_ltrim(result->text, " \"+");
@@ -644,29 +579,27 @@ static zbx_uint64_t* get_result_ui64_value(AGENT_RESULT *result)
 	}
 	/* skip AR_MESSAGE - it is information field */
 
-	if(ISSET_UI64(result))
-	{
+	if (ISSET_UI64(result))
 		return &result->ui64;
-	}
 
 	return NULL;
 }
 
-static double* get_result_dbl_value(AGENT_RESULT *result)
+static double	*get_result_dbl_value(AGENT_RESULT *result)
 {
 	double	value;
 
 	assert(result);
 
-	if(ISSET_DBL(result))
+	if (ISSET_DBL(result))
 	{
 		/* nothing to do */
 	}
-	else if(ISSET_UI64(result))
+	else if (ISSET_UI64(result))
 	{
 		SET_DBL_RESULT(result, result->ui64);
 	}
-	else if(ISSET_STR(result))
+	else if (ISSET_STR(result))
 	{
 		zbx_rtrim(result->str, " \"");
 		zbx_ltrim(result->str, " \"+");
@@ -677,7 +610,7 @@ static double* get_result_dbl_value(AGENT_RESULT *result)
 
 		SET_DBL_RESULT(result, value);
 	}
-	else if(ISSET_TEXT(result))
+	else if (ISSET_TEXT(result))
 	{
 		zbx_rtrim(result->text, " \"");
 		zbx_ltrim(result->text, " \"+");
@@ -690,78 +623,71 @@ static double* get_result_dbl_value(AGENT_RESULT *result)
 	}
 	/* skip AR_MESSAGE - it is information field */
 
-	if(ISSET_DBL(result))
-	{
+	if (ISSET_DBL(result))
 		return &result->dbl;
-	}
 
 	return NULL;
 }
 
-static char** get_result_str_value(AGENT_RESULT *result)
+static char	**get_result_str_value(AGENT_RESULT *result)
 {
-	register char *p, tmp;
+	char	*p, tmp;
 
 	assert(result);
 
-	if(ISSET_STR(result))
+	if (ISSET_STR(result))
 	{
 		/* nothing to do */
 	}
-	else if(ISSET_TEXT(result))
+	else if (ISSET_TEXT(result))
 	{
 		/* NOTE: copy only line */
-		for(p = result->text; *p != '\0' && *p != '\r' && *p != '\n'; p++);
+		for (p = result->text; '\0' != *p && '\r' != *p && '\n' != *p; p++);
 		tmp = *p; /* remember result->text character */
-		*p = '\0'; /* replace to EOL */
-		SET_STR_RESULT(result, strdup(result->text)); /* copy line */
+		*p = '\0'; /* replace to NUL */
+		SET_STR_RESULT(result, zbx_strdup(NULL, result->text)); /* copy line */
 		*p = tmp; /* restore result->text character */
-
 	}
-	else if(ISSET_UI64(result))
+	else if (ISSET_UI64(result))
 	{
 		SET_STR_RESULT(result, zbx_dsprintf(NULL, ZBX_FS_UI64, result->ui64));
 	}
-	else if(ISSET_DBL(result))
+	else if (ISSET_DBL(result))
 	{
 		SET_STR_RESULT(result, zbx_dsprintf(NULL, ZBX_FS_DBL, result->dbl));
 	}
 	/* skip AR_MESSAGE - it is information field */
 
-	if(ISSET_STR(result))
-	{
+	if (ISSET_STR(result))
 		return &result->str;
-	}
 
 	return NULL;
 }
 
-static char** get_result_text_value(AGENT_RESULT *result)
+static char	**get_result_text_value(AGENT_RESULT *result)
 {
 	assert(result);
 
-	if(ISSET_TEXT(result))
+	if (ISSET_TEXT(result))
 	{
 		/* nothing to do */
 	}
-	else if(ISSET_STR(result))
+	else if (ISSET_STR(result))
 	{
-		SET_TEXT_RESULT(result, strdup(result->str));
+		SET_TEXT_RESULT(result, zbx_strdup(NULL, result->str));
 	}
-	else if(ISSET_UI64(result))
+	else if (ISSET_UI64(result))
 	{
 		SET_TEXT_RESULT(result, zbx_dsprintf(NULL, ZBX_FS_UI64, result->ui64));
 	}
-	else if(ISSET_DBL(result))
+	else if (ISSET_DBL(result))
 	{
 		SET_TEXT_RESULT(result, zbx_dsprintf(NULL, ZBX_FS_DBL, result->dbl));
 	}
 	/* skip AR_MESSAGE - it is information field */
 
-	if(ISSET_TEXT(result))
-	{
+	if (ISSET_TEXT(result))
 		return &result->text;
-	}
 
 	return NULL;
 }
@@ -796,16 +722,12 @@ void	*get_result_value_by_type(AGENT_RESULT *result, int require_type)
 	{
 		case AR_UINT64:
 			return (void *)get_result_ui64_value(result);
-			break;
 		case AR_DOUBLE:
 			return (void *)get_result_dbl_value(result);
-			break;
 		case AR_STRING:
 			return (void *)get_result_str_value(result);
-			break;
 		case AR_TEXT:
 			return (void *)get_result_text_value(result);
-			break;
 		case AR_MESSAGE:
 			if (ISSET_MSG(result))
 				return (void *)(&result->msg);
