@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -70,6 +70,7 @@ static void	init_active_metrics()
 		buffer.count = 0;
 		buffer.pcount = 0;
 		buffer.lastsent = (int)time(NULL);
+		buffer.first_error = 0;
 	}
 }
 
@@ -83,6 +84,7 @@ static void	disable_all_metrics()
 		active_metrics[i].status = ITEM_STATUS_NOTSUPPORTED;
 }
 
+#ifdef _WINDOWS
 static void	free_active_metrics()
 {
 	int	i;
@@ -98,6 +100,7 @@ static void	free_active_metrics()
 
 	zbx_free(regexps);
 }
+#endif
 
 static int	get_min_nextcheck()
 {
@@ -218,10 +221,9 @@ static int	parse_list_of_checks(char *str)
 	if (0 != strcmp(tmp, ZBX_PROTO_VALUE_SUCCESS))
 	{
 		if (SUCCEED == zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_INFO, tmp, sizeof(tmp)))
-			zabbix_log(LOG_LEVEL_WARNING, "No active checks on server: %s",
-					tmp);
+			zabbix_log(LOG_LEVEL_WARNING, "no active checks on server: %s", tmp);
 		else
-			zabbix_log(LOG_LEVEL_WARNING, "No active checks on server");
+			zabbix_log(LOG_LEVEL_WARNING, "no active checks on server");
 		return FAIL;
 	}
 
@@ -238,8 +240,7 @@ static int	parse_list_of_checks(char *str)
 
 		if (SUCCEED != zbx_json_value_by_name(&jp_row, ZBX_PROTO_TAG_KEY, name, sizeof(name)) || '\0' == *name)
 		{
-			zabbix_log(LOG_LEVEL_WARNING, "Unable to retrieve value of tag \"%s\"",
-					ZBX_PROTO_TAG_KEY);
+			zabbix_log(LOG_LEVEL_WARNING, "unable to retrieve value of tag \"%s\"", ZBX_PROTO_TAG_KEY);
 			continue;
 		}
 
@@ -248,8 +249,7 @@ static int	parse_list_of_checks(char *str)
 
 		if (SUCCEED != zbx_json_value_by_name(&jp_row, ZBX_PROTO_TAG_DELAY, tmp, sizeof(tmp)) || '\0' == *tmp)
 		{
-			zabbix_log(LOG_LEVEL_WARNING, "Unable to retrieve value of tag \"%s\"",
-					ZBX_PROTO_TAG_DELAY);
+			zabbix_log(LOG_LEVEL_WARNING, "unable to retrieve value of tag \"%s\"", ZBX_PROTO_TAG_DELAY);
 			continue;
 		}
 
@@ -258,15 +258,14 @@ static int	parse_list_of_checks(char *str)
 		if (SUCCEED != zbx_json_value_by_name(&jp_row, ZBX_PROTO_TAG_LOGLASTSIZE, tmp, sizeof(tmp)) ||
 				SUCCEED != is_uint64(tmp, &lastlogsize))
 		{
-			zabbix_log(LOG_LEVEL_WARNING, "Unable to retrieve value of tag \"%s\"",
+			zabbix_log(LOG_LEVEL_WARNING, "unable to retrieve value of tag \"%s\"",
 					ZBX_PROTO_TAG_LOGLASTSIZE);
 			continue;
 		}
 
 		if (SUCCEED != zbx_json_value_by_name(&jp_row, ZBX_PROTO_TAG_MTIME, tmp, sizeof(tmp)) || '\0' == *tmp)
 		{
-			zabbix_log(LOG_LEVEL_WARNING, "Unable to retrieve value of tag \"%s\"",
-					ZBX_PROTO_TAG_MTIME);
+			zabbix_log(LOG_LEVEL_WARNING, "unable to retrieve value of tag \"%s\"", ZBX_PROTO_TAG_MTIME);
 			mtime = 0;
 		}
 		else
@@ -291,19 +290,20 @@ static int	parse_list_of_checks(char *str)
 
 			if (SUCCEED != zbx_json_value_by_name(&jp_row, "name", name, sizeof(name)))
 			{
-				zabbix_log(LOG_LEVEL_WARNING, "Unable to retrieve value of tag \"%s\"", "name");
+				zabbix_log(LOG_LEVEL_WARNING, "unable to retrieve value of tag \"%s\"", "name");
 				continue;
 			}
 
 			if (SUCCEED != zbx_json_value_by_name(&jp_row, "expression", expression, sizeof(expression)) || '\0' == *expression)
 			{
-				zabbix_log(LOG_LEVEL_WARNING, "Unable to retrieve value of tag \"%s\"",	"expression");
+				zabbix_log(LOG_LEVEL_WARNING, "unable to retrieve value of tag \"%s\"",	"expression");
 				continue;
 			}
 
 			if (SUCCEED != zbx_json_value_by_name(&jp_row, "expression_type", tmp, sizeof(tmp)) || *tmp == '\0')
 			{
-				zabbix_log(LOG_LEVEL_WARNING, "Unable to retrieve value of tag \"%s\"",	"expression_type");
+				zabbix_log(LOG_LEVEL_WARNING, "unable to retrieve value of tag \"%s\"",
+						"expression_type");
 				continue;
 			}
 
@@ -311,7 +311,8 @@ static int	parse_list_of_checks(char *str)
 
 			if (SUCCEED != zbx_json_value_by_name(&jp_row, "exp_delimiter", tmp, sizeof(tmp)))
 			{
-				zabbix_log(LOG_LEVEL_WARNING, "Unable to retrieve value of tag \"%s\"",	"exp_delimiter");
+				zabbix_log(LOG_LEVEL_WARNING, "unable to retrieve value of tag \"%s\"",
+						"exp_delimiter");
 				continue;
 			}
 
@@ -319,7 +320,7 @@ static int	parse_list_of_checks(char *str)
 
 			if (SUCCEED != zbx_json_value_by_name(&jp_row, "case_sensitive", tmp, sizeof(tmp)) || '\0' == *tmp)
 			{
-				zabbix_log(LOG_LEVEL_WARNING, "Unable to retrieve value of tag \"%s\"",
+				zabbix_log(LOG_LEVEL_WARNING, "unable to retrieve value of tag \"%s\"",
 						"case_sensitive");
 				continue;
 			}
@@ -361,8 +362,9 @@ static int	refresh_active_checks(const char *host, unsigned short port)
 	char		*buf;
 	int		ret;
 	struct zbx_json	json;
+	static int	last_ret = SUCCEED;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "refresh_active_checks('%s',%u)", host, port);
+	zabbix_log(LOG_LEVEL_DEBUG, "refresh_active_checks() host:'%s' port:%hu", host, port);
 
 	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
 
@@ -387,15 +389,21 @@ static int	refresh_active_checks(const char *host, unsigned short port)
 
 	if (SUCCEED == (ret = zbx_tcp_connect(&s, CONFIG_SOURCE_IP, host, port, CONFIG_TIMEOUT)))
 	{
-		zabbix_log(LOG_LEVEL_DEBUG, "Sending [%s]", json.buffer);
+		zabbix_log(LOG_LEVEL_DEBUG, "sending [%s]", json.buffer);
 
 		if (SUCCEED == (ret = zbx_tcp_send(&s, json.buffer)))
 		{
-			zabbix_log(LOG_LEVEL_DEBUG, "Before read");
+			zabbix_log(LOG_LEVEL_DEBUG, "before read");
 
 			if (SUCCEED == (ret = SUCCEED_OR_FAIL(zbx_tcp_recv_ext(&s, &buf, ZBX_TCP_READ_UNTIL_CLOSE, 0))))
 			{
-				zabbix_log(LOG_LEVEL_DEBUG, "Got [%s]", buf);
+				zabbix_log(LOG_LEVEL_DEBUG, "got [%s]", buf);
+
+				if (SUCCEED != last_ret)
+				{
+					zabbix_log(LOG_LEVEL_WARNING, "active check configuration update from [%s:%hu]"
+							" is working again", host, port);
+				}
 				parse_list_of_checks(buf);
 			}
 		}
@@ -403,8 +411,14 @@ static int	refresh_active_checks(const char *host, unsigned short port)
 		zbx_tcp_close(&s);
 	}
 
-	if (SUCCEED != ret)
-		zabbix_log(LOG_LEVEL_DEBUG, "Get active checks error: %s", zbx_tcp_strerror());
+	if (SUCCEED != ret && SUCCEED == last_ret)
+	{
+		zabbix_log(LOG_LEVEL_WARNING,
+				"active check configuration update from [%s:%hu] started to fail (%s)",
+				host, port, zbx_tcp_strerror());
+	}
+
+	last_ret = ret;
 
 	zbx_json_free(&json);
 
@@ -480,6 +494,7 @@ static int	send_buffer(const char *host, unsigned short port)
 	char				*buf = NULL;
 	int				ret = SUCCEED, i, now;
 	zbx_timespec_t			ts;
+	const char			*err_send_step = "";
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() host:'%s' port:%d values:%d/%d",
 			__function_name, host, port, buffer.count, CONFIG_BUFFER_SIZE);
@@ -549,15 +564,15 @@ static int	send_buffer(const char *host, unsigned short port)
 					zabbix_log(LOG_LEVEL_DEBUG, "OK");
 			}
 			else
-				zabbix_log(LOG_LEVEL_DEBUG, "Send value error: [recv] %s", zbx_tcp_strerror());
+				err_send_step = "[recv] ";
 		}
 		else
-			zabbix_log(LOG_LEVEL_DEBUG, "Send value error: [send] %s", zbx_tcp_strerror());
+			err_send_step = "[send] ";
 
 		zbx_tcp_close(&s);
 	}
 	else
-		zabbix_log(LOG_LEVEL_DEBUG, "Send value error: [connect] %s", zbx_tcp_strerror());
+		err_send_step = "[connect] ";
 
 	zbx_json_free(&json);
 
@@ -576,6 +591,22 @@ static int	send_buffer(const char *host, unsigned short port)
 		buffer.count = 0;
 		buffer.pcount = 0;
 		buffer.lastsent = now;
+		if (0 != buffer.first_error)
+		{
+			zabbix_log(LOG_LEVEL_WARNING, "active check data upload to [%s:%hu] is working again",
+					host, port);
+			buffer.first_error = 0;
+		}
+	}
+	else
+	{
+		if (0 == buffer.first_error)
+		{
+			zabbix_log(LOG_LEVEL_WARNING, "active check data upload to [%s:%hu] started to fail (%s%s)",
+					host, port, err_send_step, zbx_tcp_strerror());
+			buffer.first_error = now;
+		}
+		zabbix_log(LOG_LEVEL_DEBUG, "send value error: %s %s", err_send_step, zbx_tcp_strerror());
 	}
 ret:
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
@@ -844,7 +875,7 @@ static void	process_active_checks(char *server, unsigned short port)
 			if (FAIL == ret)
 			{
 				active_metrics[i].status = ITEM_STATUS_NOTSUPPORTED;
-				zabbix_log(LOG_LEVEL_WARNING, "Active check [%s] is not supported. Disabled.",
+				zabbix_log(LOG_LEVEL_WARNING, "active check \"%s\" is not supported",
 						active_metrics[i].key);
 
 				process_value(server, port, CONFIG_HOSTNAME,
@@ -949,7 +980,7 @@ static void	process_active_checks(char *server, unsigned short port)
 			if (FAIL == ret)
 			{
 				active_metrics[i].status = ITEM_STATUS_NOTSUPPORTED;
-				zabbix_log(LOG_LEVEL_WARNING, "Active check [%s] is not supported. Disabled.",
+				zabbix_log(LOG_LEVEL_WARNING, "active check \"%s\" is not supported",
 						active_metrics[i].key);
 
 				process_value(server, port, CONFIG_HOSTNAME,
@@ -1089,7 +1120,7 @@ static void	process_active_checks(char *server, unsigned short port)
 			if (FAIL == ret)
 			{
 				active_metrics[i].status = ITEM_STATUS_NOTSUPPORTED;
-				zabbix_log(LOG_LEVEL_WARNING, "Active check [%s] is not supported. Disabled.",
+				zabbix_log(LOG_LEVEL_WARNING, "active check \"%s\" is not supported",
 						active_metrics[i].key);
 
 				process_value(server, port, CONFIG_HOSTNAME,
@@ -1115,7 +1146,7 @@ static void	process_active_checks(char *server, unsigned short port)
 				if (0 == strcmp(*pvalue, ZBX_NOTSUPPORTED))
 				{
 					active_metrics[i].status = ITEM_STATUS_NOTSUPPORTED;
-					zabbix_log(LOG_LEVEL_WARNING, "Active check [%s] is not supported. Disabled.",
+					zabbix_log(LOG_LEVEL_WARNING, "active check \"%s\" is not supported",
 							active_metrics[i].key);
 				}
 			}
@@ -1188,6 +1219,7 @@ ZBX_THREAD_ENTRY(active_checks_thread, args)
 		}
 	}
 
+#ifdef _WINDOWS
 	zbx_free(activechk_args.host);
 	free_active_metrics();
 
@@ -1196,4 +1228,5 @@ ZBX_THREAD_ENTRY(active_checks_thread, args)
 	ZBX_DO_EXIT();
 
 	zbx_thread_exit(0);
+#endif
 }
