@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,24 +9,23 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #ifndef ZABBIX_EVENTLOG_H
 #define ZABBIX_EVENTLOG_H
 
-int process_eventlog(
-	const char	*source,
-	long		*lastlogsize,
-	unsigned long	*out_timestamp,
-	char		**out_source,
-	unsigned short	*out_severity,
-	char		**out_message,
-	unsigned long	*out_eventid);
+#ifndef _WINDOWS
+#	error "This module is only available for Windows OS"
+#endif
+
+int	process_eventlog(const char *source, zbx_uint64_t *lastlogsize, unsigned long *out_timestamp,
+		char **out_source, unsigned short *out_severity, char **out_message, unsigned long *out_eventid,
+		unsigned char skip_old_data);
 
 #endif /* ZABBIX_EVENTLOG_H */
