@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -24,10 +24,8 @@
 #include "log.h"
 #include "mutexs.h"
 
-#ifdef _WINDOWS
-
-static ZBX_PERF_STAT_DATA	*ppsd = NULL;
-static ZBX_MUTEX		perfstat_access;
+ZBX_PERF_STAT_DATA	ppsd;
+static ZBX_MUTEX	perfstat_access = ZBX_MUTEX_NULL;
 
 /******************************************************************************
  *                                                                            *
@@ -36,6 +34,8 @@ static ZBX_MUTEX		perfstat_access;
  ******************************************************************************/
 static void	deactivate_perf_counter(PERF_COUNTER_DATA *counter)
 {
+	zabbix_log(LOG_LEVEL_DEBUG, "deactivate_perf_counter() counterpath:'%s'", counter->counterpath);
+
 	counter->status = PERF_COUNTER_NOTSUPPORTED;
 	counter->value_count = 0;
 	counter->value_current = -1;
@@ -61,7 +61,7 @@ PERF_COUNTER_DATA	*add_perf_counter(const char *name, const char *counterpath, i
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() counter:'%s' interval:%d", __function_name, counterpath, interval);
 
-	if (NULL == ppsd->pdh_query)
+	if (SUCCEED != perf_collector_started())
 	{
 		zabbix_log(LOG_LEVEL_WARNING, "PerfCounter '%s' FAILED: collector is not started!", counterpath);
 		return NULL;
@@ -73,7 +73,7 @@ PERF_COUNTER_DATA	*add_perf_counter(const char *name, const char *counterpath, i
 		return NULL;
 	}
 
-	for (cptr = ppsd->pPerfCounterList; ; cptr = cptr->next)
+	for (cptr = ppsd.pPerfCounterList; ; cptr = cptr->next)
 	{
 		/* add new parameters */
 		if (NULL == cptr)
@@ -90,16 +90,17 @@ PERF_COUNTER_DATA	*add_perf_counter(const char *name, const char *counterpath, i
 			cptr->value_array = (double *)zbx_malloc(cptr->value_array, sizeof(double) * interval);
 
 			/* add the counter to the query */
-			pdh_status = zbx_PdhAddCounter(__function_name, cptr, ppsd->pdh_query, counterpath, &cptr->handle);
+			pdh_status = zbx_PdhAddCounter(__function_name, cptr, ppsd.pdh_query, counterpath, &cptr->handle);
 
 			zbx_mutex_lock(&perfstat_access);
-			cptr->next = ppsd->pPerfCounterList;
-			ppsd->pPerfCounterList = cptr;
+			cptr->next = ppsd.pPerfCounterList;
+			ppsd.pPerfCounterList = cptr;
 			zbx_mutex_unlock(&perfstat_access);
 
 			if (ERROR_SUCCESS != pdh_status && PDH_CSTATUS_NO_INSTANCE != pdh_status)
 			{
-				zabbix_log(LOG_LEVEL_WARNING, "PerfCounter '%s' FAILED: invalid format", counterpath);
+				zabbix_log(LOG_LEVEL_WARNING, "cannot add performance counter \"%s\": invalid format",
+						counterpath);
 				cptr = NULL;	/* indicate a failure */
 			}
 
@@ -116,12 +117,12 @@ PERF_COUNTER_DATA	*add_perf_counter(const char *name, const char *counterpath, i
 
 	if (FAIL == result)
 	{
-		zabbix_log(LOG_LEVEL_DEBUG, "%s() counter '%s' (interval: %d) already exists", __function_name, counterpath, interval);
+		zabbix_log(LOG_LEVEL_DEBUG, "%s() counter '%s' already exists", __function_name, counterpath);
 	}
 	else if (NULL != name)
 	{
 		alias_name = zbx_dsprintf(NULL, "__UserPerfCounter[%s]", name);
-		result = add_alias(name, alias_name);
+		add_alias(name, alias_name);
 		zbx_free(alias_name);
 	}
 
@@ -138,18 +139,18 @@ void	remove_perf_counter(PERF_COUNTER_DATA *counter)
 {
 	PERF_COUNTER_DATA	*cptr;
 
-	if (NULL == counter || NULL == ppsd->pPerfCounterList)
+	if (NULL == counter || NULL == ppsd.pPerfCounterList)
 		return;
 
 	zbx_mutex_lock(&perfstat_access);
 
-	if (counter == ppsd->pPerfCounterList)
+	if (counter == ppsd.pPerfCounterList)
 	{
-		ppsd->pPerfCounterList = counter->next;
+		ppsd.pPerfCounterList = counter->next;
 	}
 	else
 	{
-		for (cptr = ppsd->pPerfCounterList; ; cptr = cptr->next)
+		for (cptr = ppsd.pPerfCounterList; ; cptr = cptr->next)
 		{
 			if (cptr->next == counter)
 			{
@@ -174,10 +175,10 @@ static void	free_perf_counter_list()
 
 	zbx_mutex_lock(&perfstat_access);
 
-	while (NULL != ppsd->pPerfCounterList)
+	while (NULL != ppsd.pPerfCounterList)
 	{
-		cptr = ppsd->pPerfCounterList;
-		ppsd->pPerfCounterList = cptr->next;
+		cptr = ppsd.pPerfCounterList;
+		ppsd.pPerfCounterList = cptr->next;
 
 		zbx_free(cptr->name);
 		zbx_free(cptr->counterpath);
@@ -216,38 +217,47 @@ double	compute_average_value(PERF_COUNTER_DATA *counter, int interval)
 	return sum / (double)count;
 }
 
-int	init_perf_collector(ZBX_PERF_STAT_DATA *pperf)
+int	init_perf_collector(int multithreaded)
 {
 	const char	*__function_name = "init_perf_collector";
+	int		ret = FAIL;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	ppsd = pperf;
-
-	if (ZBX_MUTEX_ERROR == zbx_mutex_create_force(&perfstat_access, ZBX_MUTEX_PERFSTAT))
+	if (0 != multithreaded)
 	{
-		zbx_error("unable to create mutex for performance counters");
-		exit(FAIL);
+		if (ZBX_MUTEX_ERROR == zbx_mutex_create_force(&perfstat_access, ZBX_MUTEX_PERFSTAT))
+		{
+			zbx_error("cannot create mutex for performance counters");
+			exit(EXIT_FAILURE);
+		}
 	}
 
-	if (ERROR_SUCCESS != zbx_PdhOpenQuery(__function_name, &ppsd->pdh_query))
-		return FAIL;
+	if (ERROR_SUCCESS != zbx_PdhOpenQuery(__function_name, &ppsd.pdh_query))
+		goto out;
 
-	ppsd->nextcheck = time(NULL) + UNSUPPORTED_REFRESH_PERIOD;
+	ppsd.nextcheck = time(NULL) + UNSUPPORTED_REFRESH_PERIOD;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+	ret = SUCCEED;
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
 
-	return SUCCEED;
+	return ret;
+}
+
+int	perf_collector_started()
+{
+	return (NULL != ppsd.pdh_query ? SUCCEED : FAIL);
 }
 
 void	free_perf_collector()
 {
 	PERF_COUNTER_DATA	*cptr;
 
-	if (NULL == ppsd->pdh_query)
+	if (SUCCEED != perf_collector_started())
 		return;
 
-	for (cptr = ppsd->pPerfCounterList; cptr != NULL; cptr = cptr->next)
+	for (cptr = ppsd.pPerfCounterList; cptr != NULL; cptr = cptr->next)
 	{
 		if (NULL != cptr->handle)
 		{
@@ -256,8 +266,8 @@ void	free_perf_collector()
 		}
 	}
 
-	PdhCloseQuery(ppsd->pdh_query);
-	ppsd->pdh_query = NULL;
+	PdhCloseQuery(ppsd.pdh_query);
+	ppsd.pdh_query = NULL;
 
 	free_perf_counter_list();
 
@@ -272,10 +282,10 @@ void	collect_perfstat()
 	time_t			now;
 	PDH_FMT_COUNTERVALUE	value;
 
-	if (NULL == ppsd->pdh_query)	/* collector is not started */
+	if (SUCCEED != perf_collector_started())
 		return;
 
-	if (NULL == ppsd->pPerfCounterList)	/* no counters */
+	if (NULL == ppsd.pPerfCounterList)	/* no counters */
 		return;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
@@ -283,36 +293,36 @@ void	collect_perfstat()
 	now = time(NULL);
 
 	/* refresh unsupported counters */
-	if (ppsd->nextcheck <= now)
+	if (ppsd.nextcheck <= now)
 	{
-		for (cptr = ppsd->pPerfCounterList; NULL != cptr; cptr = cptr->next)
+		for (cptr = ppsd.pPerfCounterList; NULL != cptr; cptr = cptr->next)
  		{
 			if (PERF_COUNTER_NOTSUPPORTED != cptr->status)
 				continue;
 
-			zbx_PdhAddCounter(__function_name, cptr, ppsd->pdh_query, cptr->counterpath, &cptr->handle);
+			zbx_PdhAddCounter(__function_name, cptr, ppsd.pdh_query, cptr->counterpath, &cptr->handle);
 		}
 
-		ppsd->nextcheck = now + UNSUPPORTED_REFRESH_PERIOD;
+		ppsd.nextcheck = now + UNSUPPORTED_REFRESH_PERIOD;
 	}
 
 	/* query for new data */
-	if (ERROR_SUCCESS != (pdh_status = PdhCollectQueryData(ppsd->pdh_query)))
+	if (ERROR_SUCCESS != (pdh_status = PdhCollectQueryData(ppsd.pdh_query)))
 	{
-		for (cptr = ppsd->pPerfCounterList; NULL != cptr; cptr = cptr->next)
+		for (cptr = ppsd.pPerfCounterList; NULL != cptr; cptr = cptr->next)
 		{
 			if (PERF_COUNTER_NOTSUPPORTED != cptr->status)
 				deactivate_perf_counter(cptr);
 		}
 
-		zabbix_log(LOG_LEVEL_DEBUG, "call to PdhCollectQueryData() failed: %s",
-				strerror_from_module(pdh_status, L"PDH.DLL"));
+		zabbix_log(LOG_LEVEL_DEBUG, "%s() call to PdhCollectQueryData() failed: %s",
+				__function_name, strerror_from_module(pdh_status, L"PDH.DLL"));
 
-		return;
+		goto out;
 	}
 
 	/* get the raw values */
-	for (cptr = ppsd->pPerfCounterList; NULL != cptr; cptr = cptr->next)
+	for (cptr = ppsd.pPerfCounterList; NULL != cptr; cptr = cptr->next)
 	{
 		if (PERF_COUNTER_NOTSUPPORTED == cptr->status)
 			continue;
@@ -324,17 +334,68 @@ void	collect_perfstat()
 			continue;
 		}
 
-		cptr->olderRawValue = (cptr->olderRawValue + 1) % 2;
+		if (PERF_COUNTER_INITIALIZED < cptr->status)
+		{
+			zabbix_log(LOG_LEVEL_DEBUG, "%s() counterpath:'%s' old first:%I64d second:%I64d",
+					__function_name, cptr->counterpath,
+					cptr->rawValues[(cptr->olderRawValue + 1) & 1].FirstValue,
+					cptr->rawValues[(cptr->olderRawValue + 1) & 1].SecondValue);
 
-		pdh_status = PdhCalculateCounterFromRawValue(
-				cptr->handle,
-				PDH_FMT_DOUBLE,
-				&cptr->rawValues[(cptr->olderRawValue + 1) % 2],						/* the new value */
-				(PERF_COUNTER_INITIALIZED < cptr->status ? &cptr->rawValues[cptr->olderRawValue] : NULL),	/* the older value */
+			zabbix_log(LOG_LEVEL_DEBUG, "%s() counterpath:'%s' new first:%I64d second:%I64d",
+					__function_name, cptr->counterpath,
+					cptr->rawValues[cptr->olderRawValue].FirstValue,
+					cptr->rawValues[cptr->olderRawValue].SecondValue);
+		}
+
+		cptr->olderRawValue = (cptr->olderRawValue + 1) & 1;
+
+		pdh_status = PdhCalculateCounterFromRawValue(cptr->handle, PDH_FMT_DOUBLE | PDH_FMT_NOCAP100,
+				&cptr->rawValues[(cptr->olderRawValue + 1) & 1],
+				(PERF_COUNTER_INITIALIZED < cptr->status ? &cptr->rawValues[cptr->olderRawValue] : NULL),
 				&value);
 
+		if (ERROR_SUCCESS == pdh_status && PDH_CSTATUS_VALID_DATA != value.CStatus &&
+				PDH_CSTATUS_NEW_DATA != value.CStatus)
+		{
+			pdh_status = value.CStatus;
+		}
+
+		if (PDH_CSTATUS_INVALID_DATA == pdh_status)
+		{
+			/* some (e.g., rate) counters require two raw values, MSDN lacks documentation */
+			/* about what happens but tests show that PDH_CSTATUS_INVALID_DATA is returned */
+
+			cptr->status = PERF_COUNTER_GET_SECOND_VALUE;
+			continue;
+		}
+
+		if (PDH_CALC_NEGATIVE_DENOMINATOR == pdh_status)
+		{
+			/* This counter type shows the average percentage of active time observed during the sample   */
+			/* interval. This is an inverse counter. Inverse counters are calculated by monitoring the    */
+			/* percentage of time that the service was inactive and then subtracting that value from 100  */
+			/* percent. The formula used to calculate the counter value is:                               */
+			/* (1 - (inactive time delta) / (total time delta)) x 100                                     */
+			/* For some unknown reason sometimes the collected row values indicate that inactive delta is */
+			/* greater than total time delta. There must be some kind of bug in how performance counters  */
+			/* work. When this happens function PdhCalculateCounterFromRawValue() returns error           */
+			/* PDH_CALC_NEGATIVE_DENOMINATOR. Basically this means that an item was inactive all the time */
+			/* so we set the return value to 0.0 and change status to indicate success.                   */
+			/* More info: technet.microsoft.com/en-us/library/cc757283(WS.10).aspx                        */
+
+			zabbix_log(LOG_LEVEL_DEBUG, "%s() counterpath:'%s'"
+					" negative denominator error is treated as 0.0",
+					__function_name, cptr->counterpath);
+
+			pdh_status = ERROR_SUCCESS;
+			value.doubleValue = 0.0;	/* 100% inactive */
+		}
+
 		if (ERROR_SUCCESS == pdh_status)
 		{
+			zabbix_log(LOG_LEVEL_DEBUG, "%s() '%s' calculated value:" ZBX_FS_DBL, __function_name,
+					cptr->counterpath, value.doubleValue);
+
 			cptr->status = PERF_COUNTER_ACTIVE;
 			cptr->value_current = (cptr->value_current + 1) % cptr->interval;
 			if (cptr->value_count == cptr->interval)
@@ -344,16 +405,14 @@ void	collect_perfstat()
 			if (cptr->value_count < cptr->interval)
 				cptr->value_count++;
 		}
-		else if (PDH_CSTATUS_INVALID_DATA == pdh_status)
-		{
-			/* some (e.g., rate) counters require two raw values, MSDN lacks documentation */
-			/* about what happens but tests show that PDH_CSTATUS_INVALID_DATA is returned */
-
-			cptr->status = PERF_COUNTER_GET_SECOND_VALUE;
-		}
 		else
-			deactivate_perf_counter(cptr);
-	}
-}
+		{
+			zabbix_log(LOG_LEVEL_WARNING, "cannot calculate performance counter value \"%s\": %s",
+					cptr->counterpath, strerror_from_module(pdh_status, L"PDH.DLL"));
 
-#endif	/* _WINDOWS */
+			deactivate_perf_counter(cptr);
+		}
+	}
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+}
