@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,20 +9,23 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #ifndef ZABBIX_PERFSTAT_H
 #define ZABBIX_PERFSTAT_H
 
-#ifdef _WINDOWS
+#ifndef _WINDOWS
+#	error "This module is only available for Windows OS"
+#endif
+
+#include "perfmon.h"
 
-#define PERF_COLLECTOR_STARTED(collector)	((collector) && (collector)->perfs.pdh_query)
 #define UNSUPPORTED_REFRESH_PERIOD		600
 #define USE_DEFAULT_INTERVAL			0
 
@@ -34,15 +37,16 @@ typedef struct
 }
 ZBX_PERF_STAT_DATA;
 
+extern ZBX_PERF_STAT_DATA	ppsd;
+
 PERF_COUNTER_DATA	*add_perf_counter(const char *name, const char *counterpath, int interval);
 void			remove_perf_counter(PERF_COUNTER_DATA *counter);
 
-double	compute_average_value(const char *function, PERF_COUNTER_DATA *counter, int interval);
+double	compute_average_value(PERF_COUNTER_DATA *counter, int interval);
 
-int	init_perf_collector(ZBX_PERF_STAT_DATA *pperf);
+int	init_perf_collector(int multithreaded);
 void	free_perf_collector();
+int	perf_collector_started();
 void	collect_perfstat();
 
-#endif /* _WINDOWS */
-
-#endif /* ZABBIX_PERFSTAT_H */
+#endif
