@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
@@ -28,7 +28,9 @@
 #include "alias.h"
 
 #include "stats.h"
-#include "perfstat.h"
+#ifdef _WINDOWS
+#	include "perfstat.h"
+#endif
 #include "active.h"
 #include "listener.h"
 
@@ -46,7 +48,7 @@ const char	*progname = NULL;
 #ifdef _WINDOWS
 	static char	DEFAULT_CONFIG_FILE[]	= "C:\\zabbix_agentd.conf";
 #else
-	static char	DEFAULT_CONFIG_FILE[]	= "/etc/zabbix/zabbix_agentd.conf";
+	static char	DEFAULT_CONFIG_FILE[]	= SYSCONFDIR "/zabbix_agentd.conf";
 #endif
 
 /* application TITLE */
@@ -77,22 +79,22 @@ const char	usage_message[] =
 const char	*help_message[] = {
 	"Options:",
 	"",
-	"  -c --config <config-file>  absolute path to the configuration file",
-	"  -h --help                  give this help",
-	"  -V --version               display version number",
-	"  -p --print                 print known items and exit",
-	"  -t --test <item key>       test specified item and exit",
+	"  -c --config <config-file>  Absolute path to the configuration file",
+	"  -p --print                 Print known items and exit",
+	"  -t --test <item key>       Test specified item and exit",
+	"  -h --help                  Give this help",
+	"  -V --version               Display version number",
 #ifdef _WINDOWS
 	"",
 	"Functions:",
 	"",
-	"  -i --install          install Zabbix agent as service",
-	"  -d --uninstall        uninstall Zabbix agent from service",
+	"  -i --install          Install Zabbix agent as service",
+	"  -d --uninstall        Uninstall Zabbix agent from service",
 
-	"  -s --start            start Zabbix agent service",
-	"  -x --stop             stop Zabbix agent service",
+	"  -s --start            Start Zabbix agent service",
+	"  -x --stop             Stop Zabbix agent service",
 
-	"  -m --multiple-agents  service name will include hostname",
+	"  -m --multiple-agents  Service name will include hostname",
 #endif
 	NULL	/* end of text */
 };
@@ -136,9 +138,7 @@ unsigned char	process_type = 255;	/* ZBX_PROCESS_TYPE_UNKNOWN */
 
 ZBX_THREAD_ACTIVECHK_ARGS	*CONFIG_ACTIVE_ARGS = NULL;
 int				CONFIG_ACTIVE_FORKS = 0;
-int				CONFIG_ACTIVE_DISABLE = 0;
 int				CONFIG_PASSIVE_FORKS = 3;	/* number of listeners for processing passive checks */
-int				CONFIG_PASSIVE_DISABLE = 0;
 
 static void	parse_commandline(int argc, char **argv, ZBX_TASK_EX *t)
 {
@@ -276,7 +276,7 @@ static void	zbx_validate_config()
 	}
 
 	/* make sure active or passive check is enabled */
-	if (1 == CONFIG_ACTIVE_DISABLE && 1 == CONFIG_PASSIVE_DISABLE)
+	if (0 == CONFIG_ACTIVE_FORKS && 0 == CONFIG_PASSIVE_FORKS)
 	{
 		zabbix_log(LOG_LEVEL_CRIT, "either active or passive checks must be enabled");
 		exit(FAIL);
@@ -321,7 +321,7 @@ static void	get_serveractive_hosts(char *active_hosts)
 		if (NULL != (r = strchr(l, ',')))
 			*r = '\0';
 
-		if (SUCCEED != parse_serveractive_element(l, &host, &port, (unsigned short)CONFIG_SERVER_PORT))
+		if (SUCCEED != parse_serveractive_element(l, &host, &port, (unsigned short)ZBX_DEFAULT_SERVER_PORT))
 			goto fail;
 
 		rc = add_activechk_host(host, port);
@@ -363,14 +363,14 @@ fail:
  ******************************************************************************/
 static void	zbx_load_config(int requirement)
 {
-	char	*p, *active_hosts = NULL;
+	char	*active_hosts = NULL;
 
 	struct cfg_line	cfg[] =
 	{
 		/* PARAMETER,			VAR,					TYPE,
 			MANDATORY,	MIN,			MAX */
 		{"Server",			&CONFIG_HOSTS_ALLOWED,			TYPE_STRING,
-			PARM_MAND,	0,			0},
+			PARM_OPT,	0,			0},
 		{"ServerActive",		&active_hosts,				TYPE_STRING,
 			PARM_OPT,	0,			0},
 		{"Hostname",			&CONFIG_HOSTNAME,			TYPE_STRING,
@@ -389,16 +389,10 @@ static void	zbx_load_config(int requirement)
 			PARM_OPT,	0,			0},
 		{"LogFileSize",			&CONFIG_LOG_FILE_SIZE,			TYPE_INT,
 			PARM_OPT,	0,			1024},
-		{"DisableActive",		&CONFIG_ACTIVE_DISABLE,			TYPE_INT,
-			PARM_OPT,	0,			1},
-		{"DisablePassive",		&CONFIG_PASSIVE_DISABLE,		TYPE_INT,
-			PARM_OPT,	0,			1},
 		{"Timeout",			&CONFIG_TIMEOUT,			TYPE_INT,
 			PARM_OPT,	1,			30},
 		{"ListenPort",			&CONFIG_LISTEN_PORT,			TYPE_INT,
 			PARM_OPT,	1024,			32767},
-		{"ServerPort",			&CONFIG_SERVER_PORT,			TYPE_INT,
-			PARM_OPT,	1024,			32767},
 		{"ListenIP",			&CONFIG_LISTEN_IP,			TYPE_STRING,
 			PARM_OPT,	0,			0},
 		{"SourceIP",			&CONFIG_SOURCE_IP,			TYPE_STRING,
@@ -406,7 +400,7 @@ static void	zbx_load_config(int requirement)
 		{"DebugLevel",			&CONFIG_LOG_LEVEL,			TYPE_INT,
 			PARM_OPT,	0,			4},
 		{"StartAgents",			&CONFIG_PASSIVE_FORKS,			TYPE_INT,
-			PARM_OPT,	1,			100},
+			PARM_OPT,	0,			100},
 		{"RefreshActiveChecks",		&CONFIG_REFRESH_ACTIVE_CHECKS,		TYPE_INT,
 			PARM_OPT,	SEC_PER_MIN,		SEC_PER_HOUR},
 		{"MaxLinesPerSecond",		&CONFIG_MAX_LINES_PER_SECOND,		TYPE_INT,
@@ -441,29 +435,19 @@ static void	zbx_load_config(int requirement)
 
 	set_defaults();
 
-	if (ZBX_CFG_FILE_REQUIRED == requirement)
-		zbx_validate_config();
-
-	if (0 != CONFIG_PASSIVE_DISABLE)
-		CONFIG_PASSIVE_FORKS = 0;	/* listeners are not needed for passive checks */
-
-	if (0 == CONFIG_ACTIVE_DISABLE && ZBX_CFG_FILE_REQUIRED == requirement)
+	if (ZBX_CFG_FILE_REQUIRED == requirement && NULL == CONFIG_HOSTS_ALLOWED && 0 != CONFIG_PASSIVE_FORKS)
 	{
-		if (NULL == active_hosts || '\0' == *active_hosts)
-		{
-			if (NULL != (p = strchr(CONFIG_HOSTS_ALLOWED, ',')))
-				*p = '\0';
-
-			add_activechk_host(CONFIG_HOSTS_ALLOWED, (unsigned short)CONFIG_SERVER_PORT);
-
-			if (NULL != p)
-				*p = ',';
-		}
-		else
-			get_serveractive_hosts(active_hosts);
+		zbx_error("StartAgents is not 0, parameter Server must be defined");
+		exit(EXIT_FAILURE);
 	}
 
+	if (NULL != active_hosts && '\0' != *active_hosts)
+		get_serveractive_hosts(active_hosts);
+
 	zbx_free(active_hosts);
+
+	if (ZBX_CFG_FILE_REQUIRED == requirement)
+		zbx_validate_config();
 }
 
 /******************************************************************************
@@ -529,7 +513,7 @@ int	MAIN_ZABBIX_ENTRY()
 	zabbix_log(LOG_LEVEL_INFORMATION, "Starting Zabbix Agent [%s]. Zabbix %s (revision %s).",
 			CONFIG_HOSTNAME, ZABBIX_VERSION, ZABBIX_REVISION);
 
-	if (0 == CONFIG_PASSIVE_DISABLE)
+	if (0 != CONFIG_PASSIVE_FORKS)
 	{
 		if (FAIL == zbx_tcp_listen(&listen_sock, CONFIG_LISTEN_IP, (unsigned short)CONFIG_LISTEN_PORT))
 		{
@@ -538,10 +522,10 @@ int	MAIN_ZABBIX_ENTRY()
 		}
 	}
 
-	/* collector data must be initiated by user 'zabbix' */
 	init_collector_data();
 
 #ifdef _WINDOWS
+	init_perf_collector(1);
 	load_perf_counters(CONFIG_PERF_COUNTERS);
 #endif
 	load_user_parameters(CONFIG_USER_PARAMETERS);
@@ -650,11 +634,7 @@ void	zbx_on_exit()
 		zbx_free(threads);
 	}
 
-#ifdef USE_PID_FILE
-	daemon_stop();
-#endif
-
-#if !defined(_WINDOWS)
+#ifndef _WINDOWS
 	zbx_sleep(2);	/* wait for all processes to exit */
 #endif
 
@@ -666,6 +646,9 @@ void	zbx_on_exit()
 	free_metrics();
 	alias_list_free();
 	free_collector_data();
+#ifdef _WINDOWS
+	free_perf_collector();
+#endif
 
 	exit(SUCCEED);
 }
@@ -733,7 +716,7 @@ int	main(int argc, char **argv)
 		case ZBX_TASK_PRINT_SUPPORTED:
 			zbx_load_config(ZBX_CFG_FILE_OPTIONAL);
 #ifdef _WINDOWS
-			init_collector_data();	/* required for reading PerfCounter */
+			init_perf_collector(0);
 			load_perf_counters(CONFIG_PERF_COUNTERS);
 #endif
 			load_user_parameters(CONFIG_USER_PARAMETERS);
@@ -744,7 +727,7 @@ int	main(int argc, char **argv)
 			else
 				test_parameters();
 #ifdef _WINDOWS
-			free_collector_data();
+			free_perf_collector();	/* cpu_collector must be freed before perf_collector is freed */
 #endif
 			free_metrics();
 			alias_list_free();
