@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -104,7 +104,8 @@ static void	send_signal_handler(int sig)
 	if (SIGALRM == sig)
 		zabbix_log(LOG_LEVEL_WARNING, "timeout while executing operation");
 
-	exit(FAIL);
+	/* Calling _exit() to terminate the process immediately is important. See ZBX-5732 for details. */
+	_exit(FAIL);
 }
 #endif
 
@@ -147,8 +148,11 @@ static int	check_response(char *response)
 	if (SUCCEED == ret && 0 != strcmp(value, ZBX_PROTO_VALUE_SUCCESS))
 		ret = FAIL;
 
-	if (SUCCEED == zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_INFO, info, sizeof(info)))
+	if (SUCCEED == ret && SUCCEED == zbx_json_value_by_name(&jp, ZBX_PROTO_TAG_INFO, info, sizeof(info)))
+	{
 		printf("info from server: \"%s\"\n", info);
+		fflush(stdout);
+	}
 
 	return ret;
 }
@@ -197,8 +201,7 @@ static	ZBX_THREAD_ENTRY(send_value, args)
 
 static void    zbx_load_config(const char *config_file)
 {
-	char	*cfg_source_ip = NULL, *cfg_server = NULL, *cfg_hostname = NULL, *c = NULL;
-	int	cfg_server_port = 0;
+	char	*cfg_source_ip = NULL, *cfg_active_hosts = NULL, *cfg_hostname = NULL, *r = NULL;
 
 	struct cfg_line	cfg[] =
 	{
@@ -206,10 +209,8 @@ static void    zbx_load_config(const char *config_file)
 			MANDATORY,	MIN,			MAX */
 		{"SourceIP",			&cfg_source_ip,				TYPE_STRING,
 			PARM_OPT,	0,			0},
-		{"Server",			&cfg_server,				TYPE_STRING,
+		{"ServerActive",		&cfg_active_hosts,			TYPE_STRING,
 			PARM_OPT,	0,			0},
-		{"ServerPort",			&cfg_server_port,			TYPE_INT,
-			PARM_OPT,	MIN_ZABBIX_PORT,	MAX_ZABBIX_PORT},
 		{"Hostname",			&cfg_hostname,				TYPE_STRING,
 			PARM_OPT,	0,			0},
 		{NULL}
@@ -229,24 +230,28 @@ static void    zbx_load_config(const char *config_file)
 			zbx_free(cfg_source_ip);
 		}
 
-		if (NULL != cfg_server)
+		if (NULL == ZABBIX_SERVER)
 		{
-			if (NULL == ZABBIX_SERVER)
+			if (NULL != cfg_active_hosts && '\0' != *cfg_active_hosts)
 			{
-				/* get only first server */
-				if (NULL != (c = strchr(cfg_server, ',')))
-				{
-					*c = '\0';
-				}
-				ZABBIX_SERVER = zbx_strdup(ZABBIX_SERVER, cfg_server);
-			}
-			zbx_free(cfg_server);
-		}
+				unsigned short	cfg_server_port = 0;
 
-		if (0 == ZABBIX_SERVER_PORT && 0 != cfg_server_port)
-		{
-			ZABBIX_SERVER_PORT = cfg_server_port;
+				if (NULL != (r = strchr(cfg_active_hosts, ',')))
+					*r = '\0';
+
+				if (SUCCEED != parse_serveractive_element(cfg_active_hosts, &ZABBIX_SERVER,
+						&cfg_server_port, 0))
+				{
+					zbx_error("error parsing a \"ServerActive\" option: address \"%s\" is invalid",
+							cfg_active_hosts);
+					exit(EXIT_FAILURE);
+				}
+
+				if (0 == ZABBIX_SERVER_PORT && 0 != cfg_server_port)
+					ZABBIX_SERVER_PORT = cfg_server_port;
+			}
 		}
+		zbx_free(cfg_active_hosts);
 
 		if (NULL != cfg_hostname)
 		{
@@ -382,7 +387,7 @@ int	main(int argc, char **argv)
 			if (1 == REAL_TIME)
 			{
 				/* set line buffering on stdin */
-				setvbuf(stdin, (char *)NULL, _IOLBF, 0);
+				setvbuf(stdin, (char *)NULL, _IOLBF, 1024);
 			}
 		}
 		else if (NULL == (in = fopen(INPUT_FILE, "r")) )
