@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -228,7 +228,8 @@ static int	check_trigger_condition(DB_EVENT *event, DB_CONDITION *condition)
 	{
 		tmp_str = zbx_strdup(tmp_str, event->trigger.description);
 
-		substitute_simple_macros(event, NULL, NULL, NULL, &tmp_str, MACRO_TYPE_TRIGGER_DESCRIPTION, NULL, 0);
+		substitute_simple_macros(event, NULL, NULL, NULL, NULL, NULL,
+				&tmp_str, MACRO_TYPE_TRIGGER_DESCRIPTION, NULL, 0);
 
 		switch (condition->operator)
 		{
@@ -1170,24 +1171,28 @@ static void	execute_operations(DB_EVENT *event, zbx_uint64_t actionid)
 	if (0 != lnk_templateids.values_num)
 	{
 		zbx_vector_uint64_sort(&lnk_templateids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
+		zbx_vector_uint64_uniq(&lnk_templateids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
 		op_template_add(event, &lnk_templateids);
 	}
 
 	if (0 != del_templateids.values_num)
 	{
 		zbx_vector_uint64_sort(&del_templateids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
+		zbx_vector_uint64_uniq(&del_templateids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
 		op_template_del(event, &del_templateids);
 	}
 
 	if (0 != new_groupids.values_num)
 	{
 		zbx_vector_uint64_sort(&new_groupids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
+		zbx_vector_uint64_uniq(&new_groupids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
 		op_groups_add(event, &new_groupids);
 	}
 
 	if (0 != del_groupids.values_num)
 	{
 		zbx_vector_uint64_sort(&del_groupids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
+		zbx_vector_uint64_uniq(&del_groupids, ZBX_DEFAULT_UINT64_COMPARE_FUNC);
 		op_groups_del(event, &del_groupids);
 	}
 
@@ -1241,7 +1246,30 @@ void	process_actions(DB_EVENT *event)
 				execute_operations(event, actionid);
 		}
 		else if (EVENT_SOURCE_TRIGGERS == event->source)
-			DBstop_escalation(actionid, event->objectid, event->eventid);
+		{
+			/* stop escalation only if it was started before */
+			DB_RESULT	result2;
+			char		*sql = NULL;
+			size_t		sql_alloc = 128, sql_offset = 0;
+
+			sql = zbx_malloc(sql, sql_alloc);
+
+			zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+					"select null"
+					" from escalations"
+					" where actionid=" ZBX_FS_UI64
+						" and triggerid=" ZBX_FS_UI64
+						" and eventid is not null",
+					actionid, event->objectid);
+
+			result2 = DBselectN(sql, 1);
+			zbx_free(sql);
+
+			if (NULL != DBfetch(result2))
+				DBstop_escalation(actionid, event->objectid, event->eventid);
+
+			DBfree_result(result2);
+		}
 	}
 	DBfree_result(result);
 
