@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
@@ -38,10 +38,6 @@ extern int		process_num;
  *                                                                            *
  * Purpose: periodically synchronises data in memory cache with database      *
  *                                                                            *
- * Parameters:                                                                *
- *                                                                            *
- * Return value:                                                              *
- *                                                                            *
  * Author: Alexei Vladishev                                                   *
  *                                                                            *
  * Comments: never returns                                                    *
@@ -72,19 +68,19 @@ void	main_dbsyncer_loop()
 		zabbix_log(LOG_LEVEL_DEBUG, "%s #%d spent " ZBX_FS_DBL " seconds while processing %d items",
 				get_process_type_string(process_type), process_num, sec, num);
 
-		if (last_sleeptime == -1)
+		if (-1 == last_sleeptime)
 		{
 			sleeptime = num ? ZBX_SYNC_MAX / num : CONFIG_HISTSYNCER_FREQUENCY;
 		}
 		else
 		{
 			sleeptime = last_sleeptime;
-			if (num > ZBX_SYNC_MAX)
+			if (ZBX_SYNC_MAX < num)
 			{
 				retry_up = 0;
 				retry_dn++;
 			}
-			else if (num < ZBX_SYNC_MAX / 2)
+			else if (ZBX_SYNC_MAX / 2 > num)
 			{
 				retry_up++;
 				retry_dn = 0;
@@ -92,22 +88,22 @@ void	main_dbsyncer_loop()
 			else
 				retry_up = retry_dn = 0;
 
-			if (retry_dn >= 3)
+			if (2 < retry_dn)
 			{
 				sleeptime--;
 				retry_dn = 0;
 			}
 
-			if (retry_up >= 3)
+			if (2 < retry_up)
 			{
 				sleeptime++;
 				retry_up = 0;
 			}
 		}
 
-		if (sleeptime < 0)
+		if (0 > sleeptime)
 			sleeptime = 0;
-		else if (sleeptime > CONFIG_HISTSYNCER_FREQUENCY)
+		else if (CONFIG_HISTSYNCER_FREQUENCY < sleeptime)
 			sleeptime = CONFIG_HISTSYNCER_FREQUENCY;
 
 		last_sleeptime = sleeptime;
