@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,304 +9,27 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <netinet/in.h>
-#include <netdb.h>
-
-#include <signal.h>
-
-#include <string.h>
-
-#include <time.h>
-
 #include "common.h"
 #include "comms.h"
 #include "db.h"
 #include "log.h"
+#include "dbcache.h"
 
 #include "operations.h"
-#include "zbxserver.h"
-
-#include "poller/poller.h"
-#include "poller/checks_agent.h"
-#include "poller/checks_ipmi.h"
 
 /******************************************************************************
  *                                                                            *
- * Function: run_remote_commands                                              *
+ * Function: select_discovered_host                                           *
  *                                                                            *
- * Purpose: run remote command on specific host                               *
- *                                                                            *
- * Parameters: host_name - host name                                          *
- *             command - remote command                                       *
- *                                                                            *
- * Author: Eugene Grigorjev                                                   *
- *                                                                            *
- ******************************************************************************/
-
-static void run_remote_command(char* host_name, char* command)
-{
-	int		ret = 9;
-	AGENT_RESULT	agent_result;
-	DC_ITEM         item;
-	DB_RESULT	result;
-	DB_ROW		row;
-	char		*p, *host_esc, *param;
-#ifdef HAVE_OPENIPMI
-	int		val;
-	char		error[MAX_STRING_LEN];
-#endif
-
-	assert(host_name);
-	assert(command);
-
-	zabbix_log(LOG_LEVEL_DEBUG, "In run_remote_command(hostname:%s,command:%s)",
-		host_name,
-		command);
-
-	host_esc = DBdyn_escape_string(host_name);
-	result = DBselect(
-			"select hostid,host,useip,ip,dns,port,useipmi,ipmi_ip,ipmi_port,ipmi_authtype,"
-				"ipmi_privilege,ipmi_username,ipmi_password"
-			" from hosts"
-			" where status in (%d)"
-				" and host='%s'"
-				DB_NODE,
-			HOST_STATUS_MONITORED,
-			host_esc,
-			DBnode_local("hostid"));
-	zbx_free(host_esc);
-
-	if (NULL != (row = DBfetch(result)))
-	{
-		memset(&item, 0, sizeof(item));
-
-		ZBX_STR2UINT64(item.host.hostid, row[0]);
-		zbx_strlcpy(item.host.host, row[1], sizeof(item.host.host));
-		item.host.useip = (unsigned char)atoi(row[2]);
-		zbx_strlcpy(item.host.ip, row[3], sizeof(item.host.ip));
-		zbx_strlcpy(item.host.dns, row[4], sizeof(item.host.dns));
-		item.host.port = (unsigned short)atoi(row[5]);
-
-		p = command;
-		while (*p == ' ' && *p != '\0')
-			p++;
-
-#ifdef HAVE_OPENIPMI
-		if (0 == strncmp(p, "IPMI", 4))
-		{
-			if (1 == atoi(row[6]))
-			{
-				zbx_strlcpy(item.host.ipmi_ip_orig, row[7], sizeof(item.host.ipmi_ip_orig));
-				item.host.ipmi_port = (unsigned short)atoi(row[8]);
-				item.host.ipmi_authtype = atoi(row[9]);
-				item.host.ipmi_privilege = atoi(row[10]);
-				zbx_strlcpy(item.host.ipmi_username, row[11], sizeof(item.host.ipmi_username));
-				zbx_strlcpy(item.host.ipmi_password, row[12], sizeof(item.host.ipmi_password));
-			}
-			else
-			{
-				*item.host.ipmi_ip_orig = '\0';
-				item.host.ipmi_port = 623;
-				item.host.ipmi_authtype = 0;
-				item.host.ipmi_privilege = 2;
-				*item.host.ipmi_username = '\0';
-				*item.host.ipmi_password = '\0';
-			}
-
-			item.host.ipmi_ip = zbx_strdup(item.host.ipmi_ip, item.host.ipmi_ip_orig);
-			substitute_simple_macros(NULL, NULL, NULL, &item, NULL, &item.host.ipmi_ip,
-						MACRO_TYPE_HOST_IPMI_IP, NULL, 0);
-
-			if (SUCCEED == (ret = parse_ipmi_command(p, item.ipmi_sensor, &val)))
-			{
-				item.key = item.ipmi_sensor;
-				ret = set_ipmi_control_value(&item, val, error, sizeof(error));
-			}
-
-			zbx_free(item.host.ipmi_ip);
-		}
-		else
-		{
-#endif
-			param = zbx_dyn_escape_string(p, "\"");
-			item.key = zbx_dsprintf(NULL, "system.run[\"%s\",\"nowait\"]", param);
-			zbx_free(param);
-
-			init_result(&agent_result);
-
-			alarm(CONFIG_TIMEOUT);
-			ret = get_value_agent(&item, &agent_result);
-			alarm(0);
-
-			free_result(&agent_result);
-
-			zbx_free(item.key);
-#ifdef HAVE_OPENIPMI
-		}
-#endif
-	}
-	DBfree_result(result);
-
-	zabbix_log(LOG_LEVEL_DEBUG, "End run_remote_command(result:%d)",
-		ret);
-}
-
-/******************************************************************************
- *                                                                            *
- * Function: get_next_command                                                 *
- *                                                                            *
- * Purpose: parse action script on remote commands                            *
- *                                                                            *
- * Parameters: command_list - command list                                    *
- *             alias - (output) of host name or group name                    *
- *             is_group - (output) 0 if alias is a host name                  *
- *                               1 if alias is a group name                   *
- *             command - (output) remote command                              *
- *                                                                            *
- * Return value: 0 - correct command is read                                  *
- *               1 - EOL                                                      *
- *                                                                            *
- * Author: Eugene Grigorjev                                                   *
- *                                                                            *
- ******************************************************************************/
-
-#define CMD_ALIAS 0
-#define CMD_REM_COMMAND 1
-
-static int get_next_command(char** command_list, char** alias, int* is_group, char** command)
-{
-	int state = CMD_ALIAS;
-	int len = 0;
-	int i = 0;
-
-	assert(alias);
-	assert(is_group);
-	assert(command);
-
-	zabbix_log(LOG_LEVEL_DEBUG, "In get_next_command(command_list:%s)",
-		*command_list);
-
-	*alias = NULL;
-	*is_group = 0;
-	*command = NULL;
-
-	if((*command_list)[0] == '\0' || (*command_list)==NULL) {
-		zabbix_log(LOG_LEVEL_DEBUG, "Result get_next_command [EOL]");
-		return 1;
-	}
-
-	*alias = *command_list;
-	len = strlen(*command_list);
-
-	for(i=0; i < len; i++)
-	{
-		if(state == CMD_ALIAS)
-		{
-			if((*command_list)[i] == '#'){
-				*is_group = 1;
-				(*command_list)[i] = '\0';
-				state = CMD_REM_COMMAND;
-				*command = &(*command_list)[i+1];
-			}else if((*command_list)[i] == ':'){
-				*is_group = 0;
-				(*command_list)[i] = '\0';
-				state = CMD_REM_COMMAND;
-				*command = &(*command_list)[i+1];
-			}
-		} else if(state == CMD_REM_COMMAND) {
-			if((*command_list)[i] == '\r')
-			{
-				(*command_list)[i] = '\0';
-			} else if((*command_list)[i] == '\n')
-			{
-				(*command_list)[i] = '\0';
-				(*command_list) = &(*command_list)[i+1];
-				break;
-			}
-		}
-		if((*command_list)[i+1] == '\0')
-		{
-			(*command_list) = &(*command_list)[i+1];
-			break;
-		}
-	}
-
-	zabbix_log(LOG_LEVEL_DEBUG, "End get_next_command(alias:%s,is_group:%i,command:%s)",
-		*alias,
-		*is_group,
-		*command);
-
-	return 0;
-}
-
-/******************************************************************************
- *                                                                            *
- * Function: run_commands                                                     *
- *                                                                            *
- * Purpose: run remote commandlist for specific action                        *
- *                                                                            *
- * Parameters: trigger - trigger data                                         *
- *             action  - action data                                          *
- *                                                                            *
- * Author: Eugene Grigorjev                                                   *
- *                                                                            *
- * Comments: commands separated with newline                                  *
- *                                                                            *
- ******************************************************************************/
-void	op_run_commands(char *cmd_list)
-{
-	DB_RESULT	result;
-	DB_ROW		row;
-	char		*alias, *alias_esc, *command;
-	int		is_group;
-
-	assert(cmd_list);
-
-	zabbix_log(LOG_LEVEL_DEBUG, "In run_commands()");
-
-	while (1 != get_next_command(&cmd_list, &alias, &is_group, &command))
-	{
-		if (!alias || *alias == '\0' || !command || *command == '\0')
-			continue;
-
-		if (is_group)
-		{
-			alias_esc = DBdyn_escape_string(alias);
-			result = DBselect("select distinct h.host from hosts_groups hg,hosts h,groups g"
-					" where hg.hostid=h.hostid and hg.groupid=g.groupid and g.name='%s'" DB_NODE,
-					alias_esc,
-					DBnode_local("h.hostid"));
-			zbx_free(alias_esc);
-
-			while (NULL != (row = DBfetch(result)))
-				run_remote_command(row[0], command);
-
-			DBfree_result(result);
-		}
-		else
-			run_remote_command(alias, command);
-	}
-	zabbix_log( LOG_LEVEL_DEBUG, "End run_commands()");
-}
-
-/******************************************************************************
- *                                                                            *
- * Function: select hostid of discovered host                                 *
- *                                                                            *
- * Purpose: select discovered host                                            *
+ * Purpose: select hostid of discovered host                                  *
  *                                                                            *
  * Parameters: dhostid - discovered host id                                   *
  *                                                                            *
@@ -330,23 +53,27 @@ static zbx_uint64_t	select_discovered_host(DB_EVENT *event)
 	{
 		case EVENT_OBJECT_DHOST:
 			zbx_snprintf(sql, sizeof(sql),
-					"select h.hostid"
-					" from hosts h,dservices ds"
-					" where ds.ip=h.ip"
-						" and ds.dhostid=" ZBX_FS_UI64
-						DB_NODE
-					" order by h.hostid",
-					event->objectid, DBnode_local("h.hostid"));
+				"select h.hostid"
+				" from hosts h,interface i,dservices ds"
+				" where h.hostid=i.hostid"
+					" and i.useip=1"
+					" and i.ip=ds.ip"
+					" and ds.dhostid=" ZBX_FS_UI64
+					DB_NODE
+				" order by i.hostid",
+				event->objectid, DBnode_local("i.interfaceid"));
 			break;
 		case EVENT_OBJECT_DSERVICE:
 			zbx_snprintf(sql, sizeof(sql),
-					"select h.hostid"
-					" from hosts h,dservices ds"
-					" where ds.ip=h.ip"
-						" and ds.dserviceid=" ZBX_FS_UI64
-						DB_NODE
-					" order by h.hostid",
-					event->objectid, DBnode_local("h.hostid"));
+				"select h.hostid"
+				" from hosts h,interface i,dservices ds"
+				" where h.hostid=i.hostid"
+					" and i.useip=1"
+					" and i.ip=ds.ip"
+					" and ds.dserviceid =" ZBX_FS_UI64
+					DB_NODE
+				" order by i.hostid",
+				event->objectid, DBnode_local("i.interfaceid"));
 			break;
 		default:
 			goto exit;
@@ -355,73 +82,14 @@ static zbx_uint64_t	select_discovered_host(DB_EVENT *event)
 	result = DBselectN(sql, 1);
 
 	if (NULL != (row = DBfetch(result)))
-	{
 		ZBX_STR2UINT64(hostid, row[0]);
-	}
 	DBfree_result(result);
 exit:
-	zabbix_log(LOG_LEVEL_DEBUG, "End %s():" ZBX_FS_UI64, __function_name, hostid);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():" ZBX_FS_UI64, __function_name, hostid);
 
 	return hostid;
 }
 
-/******************************************************************************
- *                                                                            *
- * Function: get_discovered_agent_port                                        *
- *                                                                            *
- * Purpose: return port of the discovered zabbix_agent                        *
- *                                                                            *
- * Return value: discovered port number, otherwise default port - 10050       *
- *                                                                            *
- * Author: Alexander Vladishev                                                *
- *                                                                            *
- ******************************************************************************/
-static unsigned short	get_discovered_agent_port(DB_EVENT *event)
-{
-	const char	*__function_name = "get_discovered_agent_port";
-	DB_RESULT	result;
-	DB_ROW		row;
-	unsigned short	port = 10050;
-	char		sql[256];
-
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
-
-	if (event->source != EVENT_SOURCE_DISCOVERY)
-		goto end;
-
-	switch (event->object) {
-	case EVENT_OBJECT_DHOST:
-		zbx_snprintf(sql, sizeof(sql), "select port from dservices where type=%d and dhostid=" ZBX_FS_UI64
-				" order by dserviceid",
-				SVC_AGENT,
-				event->objectid);
-		break;
-	case EVENT_OBJECT_DSERVICE:
-		zbx_snprintf(sql, sizeof(sql), "select port from dservices where type=%d and"
-				" dhostid in (select dhostid from dservices where dserviceid=" ZBX_FS_UI64 ")"
-				" order by dserviceid",
-				SVC_AGENT,
-				event->objectid);
-		break;
-	default:
-		goto end;
-	}
-
-	result = DBselectN(sql, 1);
-	if (NULL != (row = DBfetch(result)))
-	{
-		port = atoi(row[0]);
-
-		zabbix_log(LOG_LEVEL_DEBUG, "%s() port:%d",
-				__function_name, (int)port);
-	}
-	DBfree_result(result);
-end:
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
-
-	return port;
-}
-
 /******************************************************************************
  *                                                                            *
  * Function: add_discovered_host_groups                                       *
@@ -439,13 +107,14 @@ static void	add_discovered_host_groups(zbx_uint64_t hostid, zbx_vector_uint64_t
 	DB_ROW		row;
 	zbx_uint64_t	groupid;
 	char		*sql = NULL;
-	int		sql_alloc = 256, sql_offset = 0, i;
+	size_t		sql_alloc = 256, sql_offset = 0;
+	int		i;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
 	sql = zbx_malloc(sql, sql_alloc);
 
-	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 256,
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
 			"select groupid"
 			" from hosts_groups"
 			" where hostid=" ZBX_FS_UI64
@@ -473,18 +142,11 @@ static void	add_discovered_host_groups(zbx_uint64_t hostid, zbx_vector_uint64_t
 	{
 		const char	*ins_hosts_groups_sql = "insert into hosts_groups (hostgroupid,hostid,groupid) values ";
 		zbx_uint64_t	hostgroupid;
-#ifdef HAVE_MULTIROW_INSERT
-		const char	*row_dl = ",";
-#else
-		const char	*row_dl = ";\n";
-#endif
 
 		hostgroupid = DBget_maxid_num("hosts_groups", groupids->values_num);
 
 		sql_offset = 0;
-#ifdef HAVE_ORACLE
-		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 7, "begin\n");
-#endif
+		DBbegin_multiple_update(&sql, &sql_alloc, &sql_offset);
 #ifdef HAVE_MULTIROW_INSERT
 		zbx_strcpy_alloc(&sql, &sql_alloc, &sql_offset, ins_hosts_groups_sql);
 #endif
@@ -493,19 +155,17 @@ static void	add_discovered_host_groups(zbx_uint64_t hostid, zbx_vector_uint64_t
 #ifndef HAVE_MULTIROW_INSERT
 			zbx_strcpy_alloc(&sql, &sql_alloc, &sql_offset, ins_hosts_groups_sql);
 #endif
-			zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 67,
-					"(" ZBX_FS_UI64 "," ZBX_FS_UI64 "," ZBX_FS_UI64 ")%s",
-					hostgroupid++, hostid, groupids->values[i], row_dl);
-
-#ifdef HAVE_ORACLE
-			zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 6, "end;\n");
-#endif
+			zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+					"(" ZBX_FS_UI64 "," ZBX_FS_UI64 "," ZBX_FS_UI64 ")" ZBX_ROW_DL,
+					hostgroupid++, hostid, groupids->values[i]);
 		}
 
 #ifdef HAVE_MULTIROW_INSERT
 		sql_offset--;
 		zbx_strcpy_alloc(&sql, &sql_alloc, &sql_offset, ";\n");
 #endif
+		DBend_multiple_update(&sql, &sql_alloc, &sql_offset);
+
 		DBexecute("%s", sql);
 	}
 
@@ -516,9 +176,9 @@ static void	add_discovered_host_groups(zbx_uint64_t hostid, zbx_vector_uint64_t
 
 /******************************************************************************
  *                                                                            *
- * Function: add host if not added already                                    *
+ * Function: add_discovered_host                                              *
  *                                                                            *
- * Purpose: add discovered host                                               *
+ * Purpose: add discovered host if it was not added already                   *
  *                                                                            *
  * Parameters: dhostid - discovered host id                                   *
  *                                                                            *
@@ -535,50 +195,42 @@ static zbx_uint64_t	add_discovered_host(DB_EVENT *event)
 	DB_RESULT		result2;
 	DB_ROW			row;
 	DB_ROW			row2;
-	zbx_uint64_t		hostid = 0, proxy_hostid, host_proxy_hostid;
-	char			host[MAX_STRING_LEN], *host_esc, *ip_esc, *host_unique, *host_unique_esc;
-	int			port;
+	zbx_uint64_t		dhostid, hostid = 0, proxy_hostid, host_proxy_hostid;
+	char			*host = NULL, *host_esc, *host_unique;
+	unsigned short		port;
 	zbx_uint64_t		groupid;
 	zbx_vector_uint64_t	groupids;
+	unsigned char		svc_type, interface_type;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() eventid:" ZBX_FS_UI64, __function_name, event->eventid);
 
 	zbx_vector_uint64_create(&groupids);
 
-	result = DBselect(
-			"select discovery_groupid"
-			" from config"
-			" where 1=1"
-				DB_NODE,
-			DBnode_local("configid"));
-
-	if (NULL != (row = DBfetch(result)))
-	{
-		ZBX_STR2UINT64(groupid, row[0]);
-		zbx_vector_uint64_append(&groupids, groupid);
-	}
-	else
+	if (0 == *(zbx_uint64_t *)DCconfig_get_config_data(&groupid, CONFIG_DISCOVERY_GROUPID))
 	{
 		zabbix_log(LOG_LEVEL_WARNING, "cannot add discovered host: group for discovered hosts is not defined");
 		goto clean;
 	}
-	DBfree_result(result);
 
-	switch (event->object)
+	zbx_vector_uint64_append(&groupids, groupid);
+
+	if (EVENT_OBJECT_DHOST == event->object || EVENT_OBJECT_DSERVICE == event->object)
 	{
-		case EVENT_OBJECT_DHOST:
+		if (EVENT_OBJECT_DHOST == event->object)
+		{
 			result = DBselect(
-					"select dr.proxy_hostid,ds.ip"
+					"select ds.dhostid,dr.proxy_hostid,ds.ip,ds.dns,ds.port,ds.type"
 					" from drules dr,dchecks dc,dservices ds"
 					" where dc.druleid=dr.druleid"
 						" and ds.dcheckid=dc.dcheckid"
 						" and ds.dhostid=" ZBX_FS_UI64
 					" order by ds.dserviceid",
 					event->objectid);
-			break;
-		case EVENT_OBJECT_DSERVICE:
+		}
+		else
+		{
 			result = DBselect(
-					"select dr.proxy_hostid,ds.ip"
+					"select ds.dhostid,dr.proxy_hostid,ds.ip,ds.dns,ds.port,ds.type"
 					" from drules dr,dchecks dc,dservices ds,dservices ds1"
 					" where dc.druleid=dr.druleid"
 						" and ds.dcheckid=dc.dcheckid"
@@ -586,33 +238,134 @@ static zbx_uint64_t	add_discovered_host(DB_EVENT *event)
 						" and ds1.dserviceid=" ZBX_FS_UI64
 					" order by ds.dserviceid",
 					event->objectid);
-			break;
-		case EVENT_OBJECT_ZABBIX_ACTIVE:
-			result = DBselect("select proxy_hostid,host from autoreg_host"
-					" where autoreg_hostid=" ZBX_FS_UI64,
-					event->objectid);
-			break;
-		default:
-			goto clean;
+		}
+
+		while (NULL != (row = DBfetch(result)))
+		{
+			ZBX_STR2UINT64(dhostid, row[0]);
+			ZBX_DBROW2UINT64(proxy_hostid, row[1]);
+			svc_type = (unsigned char)atoi(row[5]);
+
+			switch (svc_type)
+			{
+				case SVC_AGENT:
+					port = (unsigned short)atoi(row[4]);
+					interface_type = INTERFACE_TYPE_AGENT;
+					break;
+				case SVC_SNMPv1:
+				case SVC_SNMPv2c:
+				case SVC_SNMPv3:
+					port = (unsigned short)atoi(row[4]);
+					interface_type = INTERFACE_TYPE_SNMP;
+					break;
+				default:
+					port = ZBX_DEFAULT_AGENT_PORT;
+					interface_type = INTERFACE_TYPE_AGENT;
+			}
+
+			if (0 == hostid)
+			{
+				result2 = DBselect(
+						"select distinct h.hostid,h.proxy_hostid"
+						" from hosts h,interface i,dservices ds"
+						" where h.hostid=i.hostid"
+							" and i.ip=ds.ip"
+							" and ds.dhostid=" ZBX_FS_UI64
+						       	DB_NODE
+						" order by h.hostid",
+						dhostid,
+						DBnode_local("h.hostid"));
+
+				if (NULL != (row2 = DBfetch(result2)))
+				{
+					ZBX_STR2UINT64(hostid, row2[0]);
+					ZBX_DBROW2UINT64(host_proxy_hostid, row2[1]);
+				}
+				DBfree_result(result2);
+			}
+
+			if (0 == hostid)
+			{
+				hostid = DBget_maxid("hosts");
+
+				/* for host uniqueness purposes */
+				host = zbx_strdup(host, '\0' != *row[3] ? row[3] : row[2]);
+
+				make_hostname(host);	/* replace not-allowed symbols */
+				host_unique = DBget_unique_hostname_by_sample(host);
+				host_esc = DBdyn_escape_string(host_unique);
+
+				zbx_free(host);
+
+				DBexecute("insert into hosts"
+							" (hostid,proxy_hostid,host,name)"
+						" values"
+							" (" ZBX_FS_UI64 ",%s,'%s','%s')",
+						hostid, DBsql_id_ins(proxy_hostid), host_esc, host_esc);
+
+				DBadd_interface(hostid, interface_type, 1, row[2], row[3], port);
+
+				zbx_free(host_unique);
+				zbx_free(host_esc);
+
+				add_discovered_host_groups(hostid, &groupids);
+			}
+			else
+			{
+				if (host_proxy_hostid != proxy_hostid)
+				{
+					DBexecute("update hosts"
+							" set proxy_hostid=%s"
+							" where hostid=" ZBX_FS_UI64,
+							DBsql_id_ins(proxy_hostid), hostid);
+				}
+
+				DBadd_interface(hostid, interface_type, 1, row[2], row[3], port);
+			}
+		}
+		DBfree_result(result);
 	}
-
-	if (NULL != (row = DBfetch(result)))
+	else if (EVENT_OBJECT_ZABBIX_ACTIVE == event->object)
 	{
-		ZBX_STR2UINT64(proxy_hostid, row[0]);
+		result = DBselect(
+				"select proxy_hostid,host,listen_ip,listen_dns,listen_port"
+				" from autoreg_host"
+				" where autoreg_hostid=" ZBX_FS_UI64,
+				event->objectid);
 
-		if (EVENT_OBJECT_ZABBIX_ACTIVE == event->object)
+		if (NULL != (row = DBfetch(result)))
 		{
 			char	sql[512];
 
+			ZBX_DBROW2UINT64(proxy_hostid, row[0]);
 			host_esc = DBdyn_escape_string_len(row[1], HOST_HOST_LEN);
+			port = (unsigned short)atoi(row[4]);
+
+			result2 = DBselect(
+					"select null"
+					" from hosts"
+					" where host='%s'"
+						" and status=%d",
+					host_esc, HOST_STATUS_TEMPLATE);
+
+			if (NULL != (row2 = DBfetch(result2)))
+			{
+				zabbix_log(LOG_LEVEL_WARNING, "cannot add discovered host \"%s\":"
+						" template with the same name already exists", row[1]);
+				DBfree_result(result2);
+				goto out;
+			}
+			DBfree_result(result2);
 
 			zbx_snprintf(sql, sizeof(sql),
 					"select hostid,proxy_hostid"
 					" from hosts"
 					" where host='%s'"
+						" and status in (%d,%d)"
 						DB_NODE
 					" order by hostid",
 					host_esc,
+					HOST_STATUS_MONITORED, HOST_STATUS_NOT_MONITORED,
 					DBnode_local("hostid"));
 
 			result2 = DBselectN(sql, 1);
@@ -621,97 +374,37 @@ static zbx_uint64_t	add_discovered_host(DB_EVENT *event)
 			{
 				hostid = DBget_maxid("hosts");
 
-				DBexecute("insert into hosts (hostid,proxy_hostid,host,useip,dns)"
-						" values (" ZBX_FS_UI64 "," ZBX_FS_UI64 ",'%s',0,'%s')",
-						hostid, proxy_hostid, host_esc, host_esc);
+				DBexecute("insert into hosts"
+							" (hostid,proxy_hostid,host,name)"
+						" values"
+							" (" ZBX_FS_UI64 ",%s,'%s','%s')",
+						hostid, DBsql_id_ins(proxy_hostid), host_esc, host_esc);
+
+				DBadd_interface(hostid, INTERFACE_TYPE_AGENT, 1, row[2], row[3], port);
 
 				add_discovered_host_groups(hostid, &groupids);
 			}
 			else
 			{
 				ZBX_STR2UINT64(hostid, row2[0]);
-				ZBX_STR2UINT64(host_proxy_hostid, row2[1]);
+				ZBX_DBROW2UINT64(host_proxy_hostid, row2[1]);
 
 				if (host_proxy_hostid != proxy_hostid)
 				{
 					DBexecute("update hosts"
-							" set proxy_hostid=" ZBX_FS_UI64
+							" set proxy_hostid=%s"
 							" where hostid=" ZBX_FS_UI64,
-							proxy_hostid, hostid);
+							DBsql_id_ins(proxy_hostid), hostid);
 				}
+
+				DBadd_interface(hostid, INTERFACE_TYPE_AGENT, 1, row[2], row[3], port);
 			}
 			DBfree_result(result2);
-
+out:
 			zbx_free(host_esc);
 		}
-		else	/* EVENT_OBJECT_DHOST, EVENT_OBJECT_DSERVICE */
-		{
-			alarm(CONFIG_TIMEOUT);
-			zbx_gethost_by_ip(row[1], host, sizeof(host));
-			alarm(0);
-
-			host_esc = DBdyn_escape_string_len(host, HOST_HOST_LEN);
-			ip_esc = DBdyn_escape_string_len(row[1], HOST_IP_LEN);
-
-			port = get_discovered_agent_port(event);
-
-			result2 = DBselect(
-					"select hostid,dns,port,proxy_hostid"
-					" from hosts"
-					" where ip='%s'"
-						DB_NODE,
-					ip_esc,
-					DBnode_local("hostid"));
-
-			if (NULL == (row2 = DBfetch(result2)))
-			{
-				hostid = DBget_maxid("hosts");
-
-				/* for host uniqueness purposes */
-				if ('\0' != *host)
-				{
-					/* by host name */
-					make_hostname(host);	/* replace not-allowed characters */
-					host_unique = DBget_unique_hostname_by_sample(host);
-				}
-				else
-				{
-					/* by ip */
-					make_hostname(row[1]);	/* replace not-allowed characters */
-					host_unique = DBget_unique_hostname_by_sample(row[1]);
-				}
-
-				host_unique_esc = DBdyn_escape_string(host_unique);
-
-				DBexecute("insert into hosts (hostid,proxy_hostid,host,useip,ip,dns,port)"
-						" values (" ZBX_FS_UI64 "," ZBX_FS_UI64 ",'%s',1,'%s','%s',%d)",
-						hostid, proxy_hostid, host_unique_esc, ip_esc, host_esc, port);
-
-				zbx_free(host_unique);
-				zbx_free(host_unique_esc);
-
-				add_discovered_host_groups(hostid, &groupids);
-			}
-			else
-			{
-				ZBX_STR2UINT64(hostid, row2[0]);
-				ZBX_STR2UINT64(host_proxy_hostid, row2[3]);
-
-				if (0 != strcmp(host, row2[1]) || host_proxy_hostid != proxy_hostid)
-				{
-					DBexecute("update hosts"
-							" set dns='%s',proxy_hostid=" ZBX_FS_UI64
-							" where hostid=" ZBX_FS_UI64,
-							host_esc, proxy_hostid, hostid);
-				}
-			}
-			DBfree_result(result2);
-
-			zbx_free(host_esc);
-			zbx_free(ip_esc);
-		}
+		DBfree_result(result);
 	}
-	DBfree_result(result);
 clean:
 	zbx_vector_uint64_destroy(&groupids);
 
@@ -901,7 +594,7 @@ void	op_groups_del(DB_EVENT *event, zbx_vector_uint64_t *groupids)
 	DB_RESULT	result;
 	zbx_uint64_t	hostid;
 	char		*sql = NULL;
-	int		sql_alloc = 256, sql_offset = 0;
+	size_t		sql_alloc = 256, sql_offset = 0;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
@@ -917,12 +610,12 @@ void	op_groups_del(DB_EVENT *event, zbx_vector_uint64_t *groupids)
 	sql = zbx_malloc(sql, sql_alloc);
 
 	/* make sure host belongs to at least one hostgroup */
-	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 256,
-		"select groupid"
-		" from hosts_groups"
-		" where hostid=" ZBX_FS_UI64
-			" and not",
-		hostid);
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+			"select groupid"
+			" from hosts_groups"
+			" where hostid=" ZBX_FS_UI64
+				" and not",
+			hostid);
 	DBadd_condition_alloc(&sql, &sql_alloc, &sql_offset, "groupid", groupids->values, groupids->values_num);
 
 	result = DBselectN(sql, 1);
@@ -935,7 +628,7 @@ void	op_groups_del(DB_EVENT *event, zbx_vector_uint64_t *groupids)
 	else
 	{
 		sql_offset = 0;
-		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 256,
+		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
 				"delete from hosts_groups"
 				" where hostid=" ZBX_FS_UI64
 					" and",
@@ -957,13 +650,13 @@ void	op_groups_del(DB_EVENT *event, zbx_vector_uint64_t *groupids)
  *                                                                            *
  * Purpose: link host with template                                           *
  *                                                                            *
- * Parameters: event      - [IN] event data                                   *
- *             templateid - [IN] host template identificator from database    *
+ * Parameters: event           - [IN] event data                              *
+ *             lnk_templateids - [IN] array of template IDs                   *
  *                                                                            *
  * Author: Eugene Grigorjev                                                   *
  *                                                                            *
  ******************************************************************************/
-void	op_template_add(DB_EVENT *event, zbx_uint64_t templateid)
+void	op_template_add(DB_EVENT *event, zbx_vector_uint64_t *lnk_templateids)
 {
 	const char	*__function_name = "op_template_add";
 	zbx_uint64_t	hostid;
@@ -979,7 +672,7 @@ void	op_template_add(DB_EVENT *event, zbx_uint64_t templateid)
 	if (0 == (hostid = add_discovered_host(event)))
 		return;
 
-	DBcopy_template_elements(hostid, templateid);
+	DBcopy_template_elements(hostid, lnk_templateids);
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
 }
@@ -990,13 +683,13 @@ void	op_template_add(DB_EVENT *event, zbx_uint64_t templateid)
  *                                                                            *
  * Purpose: unlink and clear host from template                               *
  *                                                                            *
- * Parameters: event      - [IN] event data                                   *
- *             templateid - [IN] host template identificator from database    *
+ * Parameters: event           - [IN] event data                              *
+ *             lnk_templateids - [IN] array of template IDs                   *
  *                                                                            *
  * Author: Eugene Grigorjev                                                   *
  *                                                                            *
  ******************************************************************************/
-void	op_template_del(DB_EVENT *event, zbx_uint64_t templateid)
+void	op_template_del(DB_EVENT *event, zbx_vector_uint64_t *del_templateids)
 {
 	const char	*__function_name = "op_template_del";
 	zbx_uint64_t	hostid;
@@ -1012,7 +705,7 @@ void	op_template_del(DB_EVENT *event, zbx_uint64_t templateid)
 	if (0 == (hostid = select_discovered_host(event)))
 		return;
 
-	DBdelete_template_elements(hostid, templateid);
+	DBdelete_template_elements(hostid, del_templateids);
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
 }
