@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
@@ -39,75 +39,70 @@ extern char	*CONFIG_EXTERNALSCRIPTS;
  *                                                                            *
  * Author: Mike Nestor, rewritten by Alexander Vladishev                      *
  *                                                                            *
- * Comments:                                                                  *
- *                                                                            *
  ******************************************************************************/
 int	get_value_external(DC_ITEM *item, AGENT_RESULT *result)
 {
 	const char	*__function_name = "get_value_external";
-	char		*conn, *params = NULL, *command = NULL,
-			*p, *pl, *pr = NULL, error[ITEM_ERROR_LEN_MAX],
-			*buf = NULL;
-	int		ret = SUCCEED;
+	char		key[MAX_STRING_LEN], params[MAX_STRING_LEN], error[ITEM_ERROR_LEN_MAX],
+			*cmd = NULL, *buf = NULL;
+	size_t		cmd_alloc = ZBX_KIBIBYTE, cmd_offset = 0;
+	int		rc, ret = NOTSUPPORTED;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() key:'%s'", __function_name, item->key_orig);
 
-	conn = item->host.useip == 1 ? item->host.ip : item->host.dns;
-
-	/* find optional parameters enclosed between the leftmost and the rightmost brackets */
-	if (NULL != (pl = strchr(item->key, '[')))
+	if (0 == (rc = parse_command(item->key, key, sizeof(key), params, sizeof(params))))
 	{
-		*pl = '\0';
-		params = pl + 1;
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "Key is badly formatted"));
+		goto notsupported;
+	}
 
-		if (NULL != (pr = strrchr(params, ']')))
-			*pr = '\0';
-		else
+	cmd = zbx_malloc(cmd, cmd_alloc);
+	zbx_snprintf_alloc(&cmd, &cmd_alloc, &cmd_offset, "%s/%s", CONFIG_EXTERNALSCRIPTS, key);
+
+	if (-1 == access(cmd, X_OK))
+	{
+		SET_MSG_RESULT(result, zbx_dsprintf(NULL, "%s: %s", cmd, zbx_strerror(errno)));
+		goto notsupported;
+	}
+
+	if (2 == rc)	/* key with parameters */
+	{
+		int	i, n;
+		char	param[MAX_STRING_LEN], *param_esc;
+
+		if (0 == (n = num_param(params)))
 		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "External check is not supported."
-						" No closing bracket ']' found."));
-			ret = NOTSUPPORTED;
-			goto exit;
+			SET_MSG_RESULT(result, zbx_strdup(NULL, "Key is badly formatted"));
+			goto notsupported;
+		}
+
+		for (i = 1; i <= n; i++)
+		{
+			if (0 != get_param(params, i, param, sizeof(param)))
+			{
+				THIS_SHOULD_NEVER_HAPPEN;
+				*param = '\0';
+			}
+
+			param_esc = zbx_dyn_escape_string(param, "\"\\");
+			zbx_snprintf_alloc(&cmd, &cmd_alloc, &cmd_offset, " \"%s\"", param_esc);
+			zbx_free(param_esc);
 		}
 	}
 
-	if (NULL != params)
-		command = zbx_dsprintf(command, "%s/%s %s %s",
-				CONFIG_EXTERNALSCRIPTS, item->key, conn, params);
-	else
-		command = zbx_dsprintf(command, "%s/%s %s",
-				CONFIG_EXTERNALSCRIPTS, item->key, conn);
-
-	if (SUCCEED == zbx_execute(command, &buf, error, sizeof(error), CONFIG_TIMEOUT))
+	if (SUCCEED == zbx_execute(cmd, &buf, error, sizeof(error), CONFIG_TIMEOUT))
 	{
-		/* copy only the first line from the command output into result */
-		if (NULL != (p = strchr(buf, '\n')))
-			*p = '\0';
-
 		zbx_rtrim(buf, ZBX_WHITESPACE);
 
-		if ('\0' == *buf)
-		{
-			SET_MSG_RESULT(result, zbx_strdup(NULL, "Script returned nothing"));
-			ret = NOTSUPPORTED;
-		}
-		else if (SUCCEED != set_result_type(result, item->value_type, item->data_type, buf))
-			ret = NOTSUPPORTED;
+		if (SUCCEED == set_result_type(result, item->value_type, item->data_type, buf))
+			ret = SUCCEED;
+
+		zbx_free(buf);
 	}
 	else
-	{
-		zabbix_log(LOG_LEVEL_DEBUG, "Cannot execute script: %s", error);
 		SET_MSG_RESULT(result, zbx_strdup(NULL, error));
-		ret = NOTSUPPORTED;
-	}
-
-	zbx_free(buf);
-	zbx_free(command);
-exit:
-	if (NULL != pl)
-		*pl = '[';
-	if (NULL != pr)
-		*pr = ']';
+notsupported:
+	zbx_free(cmd);
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
 
