@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #ifndef ZABBIX_CHECKS_IPMI_H
@@ -30,8 +30,8 @@
 int	init_ipmi_handler();
 int	free_ipmi_handler();
 int	get_value_ipmi(DC_ITEM *item, AGENT_RESULT *value);
-int	parse_ipmi_command(char *command, char *c_name, int *val);
-int	set_ipmi_control_value(DC_ITEM *item, int value, char *error, size_t error_max_len);
+int	parse_ipmi_command(const char *command, char *c_name, int *val, char *error, size_t max_error_len);
+int	set_ipmi_control_value(DC_ITEM *item, int value, char *error, size_t max_error_len);
 
 #endif	/* HAVE_OPENIPMI */
 #endif	/* ZABBIX_CHECKS_IPMI_H */
