@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2000-2011 Zabbix SIA
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
@@ -44,7 +44,7 @@ extern unsigned char	process_type;
  * Author: Alexei Vladishev, Aleksandrs Saveljevs                             *
  *                                                                            *
  ******************************************************************************/
-static void	process_time_functions()
+static void	process_time_functions(void)
 {
 	const char		*__function_name = "process_time_functions";
 	char			*sql = NULL;
@@ -131,8 +131,8 @@ typedef struct
 	char		*host;
 	time_t		maintenance_from;
 	zbx_uint64_t	maintenanceid;
-	int		maintenance_type;
 	zbx_uint64_t	host_maintenanceid;
+	int		maintenance_type;
 	int		host_maintenance_status;
 	int		host_maintenance_type;
 	int		host_maintenance_from;
@@ -664,7 +664,7 @@ static int	day_in_month(int year, int mon)
 		return month[mon];
 }
 
-static void	process_maintenance()
+static void	process_maintenance(void)
 {
 	const char			*__function_name = "process_maintenance";
 	DB_RESULT			result;
@@ -802,7 +802,8 @@ static void	process_maintenance()
 			continue;
 		}
 
-		if (db_start_date < db_active_since)
+		/* allow one time periods to start before active time */
+		if (db_start_date < db_active_since && TIMEPERIOD_TYPE_ONETIME != db_timeperiod_type)
 			continue;
 
 		if (db_start_date > now || now >= db_start_date + db_period)
@@ -810,6 +811,9 @@ static void	process_maintenance()
 
 		maintenance_from = db_start_date;
 
+		if (maintenance_from < db_active_since)
+			maintenance_from = db_active_since;
+
 		process_maintenance_hosts(&hm, &hm_alloc, &hm_count, maintenance_from, db_maintenanceid, db_maintenance_type);
 	}
 	DBfree_result(result);
@@ -818,6 +822,8 @@ static void	process_maintenance()
 
 	while (0 != hm_count--)
 		zbx_free(hm[hm_count].host);
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
 }
 
 /******************************************************************************
@@ -835,7 +841,7 @@ static void	process_maintenance()
  * Comments: does update once per 30 seconds (hardcoded)                      *
  *                                                                            *
  ******************************************************************************/
-void	main_timer_loop()
+void	main_timer_loop(void)
 {
 	int	now, nextcheck, sleeptime;
 
