@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,18 +9,19 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
 #include "db.h"
 #include "dbcache.h"
 #include "log.h"
+#include "zbxserver.h"
 
 #include "active.h"
 
@@ -42,19 +43,21 @@ extern unsigned char	daemon_type;
  * Comments:                                                                  *
  *                                                                            *
  ******************************************************************************/
-static int	get_hostid_by_host(const char *host, zbx_uint64_t *hostid, char *error)
+static int	get_hostid_by_host(const char *host, const char *ip, unsigned short port, zbx_uint64_t *hostid, char *error)
 {
-	char		*host_esc;
+	const char	*__function_name = "get_hostid_by_host";
+
+	char		*host_esc, dns[INTERFACE_DNS_LEN_MAX];
 	DB_RESULT	result;
 	DB_ROW		row;
 	int		res = FAIL;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In get_hostid_by_host() host:'%s'", host);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() host:'%s'", __function_name, host);
 
 	if (FAIL == zbx_check_hostname(host))
 	{
 		zbx_snprintf(error, MAX_STRING_LEN, "invalid host name [%s]", host);
-		return res;
+		goto out;
 	}
 
 	host_esc = DBdyn_escape_string(host);
@@ -64,7 +67,7 @@ static int	get_hostid_by_host(const char *host, zbx_uint64_t *hostid, char *erro
 			" from hosts"
 			" where host='%s'"
 				" and status in (%d,%d)"
-		       		" and proxy_hostid=0"
+		       		" and proxy_hostid is null"
 				DB_NODE,
 			host_esc,
 			HOST_STATUS_MONITORED,
@@ -85,15 +88,23 @@ static int	get_hostid_by_host(const char *host, zbx_uint64_t *hostid, char *erro
 	{
 		zbx_snprintf(error, MAX_STRING_LEN, "host [%s] not found", host);
 
+		/* remove ::ffff: prefix from IPv4-mapped IPv6 addresses */
+		if (0 == strncmp("::ffff:", ip, 7) && SUCCEED == is_ip4(ip + 7))
+			ip += 7;
+
+		alarm(CONFIG_TIMEOUT);
+		zbx_gethost_by_ip(ip, dns, sizeof(dns));
+		alarm(0);
+
 		DBbegin();
 
 		if (0 != (daemon_type & ZBX_DAEMON_TYPE_SERVER))
 		{
-			DBregister_host(0, host, (int)time(NULL));
+			DBregister_host(0, host, ip, dns, port, (int)time(NULL));
 		}
 		else if (0 != (daemon_type & ZBX_DAEMON_TYPE_PROXY))
 		{
-			DBproxy_register_host(host);
+			DBproxy_register_host(host, ip, dns, port);
 		}
 
 		DBcommit();
@@ -102,6 +113,8 @@ static int	get_hostid_by_host(const char *host, zbx_uint64_t *hostid, char *erro
 	DBfree_result(result);
 
 	zbx_free(host_esc);
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(res));
 
 	return res;
 }
@@ -126,18 +139,19 @@ static int	get_hostid_by_host(const char *host, zbx_uint64_t *hostid, char *erro
  ******************************************************************************/
 int	send_list_of_active_checks(zbx_sock_t *sock, char *request)
 {
+	const char	*__function_name = "send_list_of_active_checks";
+
 	char		*host = NULL, *p;
 	DB_RESULT	result;
 	DB_ROW		row;
 	char		*buffer = NULL;
-	int		buffer_alloc = 2048;
-	int		buffer_offset = 0;
-	int		res = FAIL;
+	size_t		buffer_alloc = 2 * ZBX_KIBIBYTE, buffer_offset = 0;
+	int		res = FAIL, refresh_unsupported, now;
 	zbx_uint64_t	hostid;
-	char		error[MAX_STRING_LEN];
+	char		error[MAX_STRING_LEN], ip[INTERFACE_IP_LEN_MAX];
 	DC_ITEM		dc_item;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In send_list_of_active_checks()");
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
 	if (NULL != (host = strchr(request, '\n')))
 	{
@@ -152,34 +166,32 @@ int	send_list_of_active_checks(zbx_sock_t *sock, char *request)
 		goto out;
 	}
 
-	if (FAIL == get_hostid_by_host(host, &hostid, error))
+	strscpy(ip, get_ip_by_socket(sock));
+
+	if (FAIL == get_hostid_by_host(host, ip, ZBX_DEFAULT_AGENT_PORT, &hostid, error))
 		goto out;
 
+	DCconfig_get_config_data(&refresh_unsupported, CONFIG_REFRESH_UNSUPPORTED);
+
+	now = time(NULL);
+
 	buffer = zbx_malloc(buffer, buffer_alloc);
 
 	buffer_offset = 0;
-	zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset, 1024,
-			"select i.key_,i.delay,i.lastlogsize from items i,hosts h"
-			" where i.hostid=h.hostid and h.status=%d and i.type=%d and h.hostid=" ZBX_FS_UI64
-			" and h.proxy_hostid=0",
+	zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset,
+			"select i.key_,i.delay,i.lastlogsize"
+			" from items i,hosts h"
+			" where i.hostid=h.hostid"
+				" and h.status=%d"
+				" and i.type=%d"
+				" and i.flags<>%d"
+				" and h.hostid=" ZBX_FS_UI64
+				" and h.proxy_hostid is null",
 			HOST_STATUS_MONITORED,
 			ITEM_TYPE_ZABBIX_ACTIVE,
+			ZBX_FLAG_DISCOVERY_CHILD,
 			hostid);
 
-	if (0 != CONFIG_REFRESH_UNSUPPORTED)
-	{
-		zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset, 256,
-				" and (i.status=%d or (i.status=%d and i.lastclock+%d<=%d))",
-				ITEM_STATUS_ACTIVE, ITEM_STATUS_NOTSUPPORTED,
-				CONFIG_REFRESH_UNSUPPORTED, time(NULL));
-	}
-	else
-	{
-		zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset, 256,
-				" and i.status=%d",
-				ITEM_STATUS_ACTIVE);
-	}
-
 	result = DBselect("%s", buffer);
 
 	buffer_offset = 0;
@@ -187,23 +199,35 @@ int	send_list_of_active_checks(zbx_sock_t *sock, char *request)
 	{
 		if (FAIL == DCconfig_get_item_by_key(&dc_item, (zbx_uint64_t)0, host, row[0]))
 		{
-			zabbix_log(LOG_LEVEL_DEBUG, "Item '%s' was not found in the server cache. Not sending now.", row[0]);
+			zabbix_log(LOG_LEVEL_DEBUG, "%s() Item '%s' was not found in the server cache. Not sending now.",
+					__function_name, row[0]);
 			continue;
 		}
 
-		zabbix_log(LOG_LEVEL_DEBUG, "Item '%s' was successfully found in the server cache. Sending.", row[0]);
+		if (ITEM_STATUS_NOTSUPPORTED == dc_item.status)
+		{
+			if (0 == refresh_unsupported || dc_item.lastclock + refresh_unsupported > now)
+			{
+				DCconfig_clean_items(&dc_item, NULL, 1);
+				continue;
+			}
+		}
 
-		zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset, 512, "%s:%s:%s\n",
+		DCconfig_clean_items(&dc_item, NULL, 1);
+
+		zabbix_log(LOG_LEVEL_DEBUG, "%s() Item '%s' was successfully found in the server cache. Sending.",
+				__function_name, row[0]);
+
+		zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset, "%s:%s:%s\n",
 				row[0],		/* item key */
 				row[1],		/* item delay */
 				row[2]);	/* item lastlogsize */
 	}
 	DBfree_result(result);
 
-	zbx_snprintf_alloc(&buffer, &buffer_alloc, &buffer_offset, 512, "ZBX_EOF\n");
+	zbx_strcpy_alloc(&buffer, &buffer_alloc, &buffer_offset, "ZBX_EOF\n");
 
-	zabbix_log(LOG_LEVEL_DEBUG, "Sending [%s]",
-			buffer);
+	zabbix_log(LOG_LEVEL_DEBUG, "%s() sending [%s]", __function_name, buffer);
 
 	alarm(CONFIG_TIMEOUT);
 	if (SUCCEED != zbx_tcp_send_raw(sock, buffer))
@@ -213,11 +237,12 @@ int	send_list_of_active_checks(zbx_sock_t *sock, char *request)
 	alarm(0);
 
 	zbx_free(buffer);
-
 out:
 	if (FAIL == res)
-		zabbix_log(LOG_LEVEL_WARNING, "Send list of active checks to [%s] failed: %s",
-				get_ip_by_socket(sock), error);
+		zabbix_log(LOG_LEVEL_WARNING, "cannot send list of active checks to [%s]: %s", get_ip_by_socket(sock),
+				error);
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(res));
 
 	return res;
 }
@@ -258,27 +283,28 @@ static void	add_regexp_name(char ***regexp, int *regexp_alloc, int *regexp_num,
  ******************************************************************************/
 int	send_list_of_active_checks_json(zbx_sock_t *sock, struct zbx_json_parse *jp)
 {
+	const char	*__function_name = "send_list_of_active_checks_json";
 	char		host[HOST_HOST_LEN_MAX], *name_esc, params[MAX_STRING_LEN],
 			pattern[MAX_STRING_LEN], tmp[32],
-			key_severity[MAX_STRING_LEN], key_logeventid[MAX_STRING_LEN];
+			key_severity[MAX_STRING_LEN], key_logeventid[MAX_STRING_LEN],
+			ip[INTERFACE_IP_LEN_MAX];
 	DB_RESULT	result;
 	DB_ROW		row;
-	DB_ITEM		item;
 	struct zbx_json	json;
-	int		res = FAIL;
+	int		res = FAIL, refresh_unsupported, now;
 	zbx_uint64_t	hostid;
-	char		error[MAX_STRING_LEN];
+	char		error[MAX_STRING_LEN], *key = NULL;
 	DC_ITEM		dc_item;
+	unsigned short	port;
 
 	char		**regexp = NULL;
 	int		regexp_alloc = 0;
 	int		regexp_num = 0, n;
 
 	char		*sql = NULL;
-	int		sql_alloc = 2048;
-	int		sql_offset;
+	size_t		sql_alloc = 2 * ZBX_KIBIBYTE, sql_offset = 0;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In send_list_of_active_checks_json()");
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
 	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_HOST, host, sizeof(host)))
 	{
@@ -286,30 +312,40 @@ int	send_list_of_active_checks_json(zbx_sock_t *sock, struct zbx_json_parse *jp)
 		goto error;
 	}
 
-	if (FAIL == get_hostid_by_host(host, &hostid, error))
+	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_IP, ip, sizeof(ip)))
+		strscpy(ip, get_ip_by_socket(sock));
+
+	if (FAIL == zbx_json_value_by_name(jp, ZBX_PROTO_TAG_PORT, tmp, sizeof(tmp)))
+		*tmp = '\0';
+
+	if (FAIL == is_ushort(tmp, &port))
+		port = ZBX_DEFAULT_AGENT_PORT;
+
+	if (FAIL == get_hostid_by_host(host, ip, port, &hostid, error))
 		goto error;
 
+	DCconfig_get_config_data(&refresh_unsupported, CONFIG_REFRESH_UNSUPPORTED);
+
+	now = time(NULL);
+
 	sql = zbx_malloc(sql, sql_alloc);
 
 	name_esc = DBdyn_escape_string(host);
 
-	sql_offset = 0;
-	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 1024,
-			"select %s where i.hostid=h.hostid and h.status=%d and i.type=%d and h.hostid=" ZBX_FS_UI64
-			" and h.proxy_hostid=0",
-			ZBX_SQL_ITEM_SELECT,
+	zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+			"select i.key_,i.delay,i.lastlogsize,i.mtime"
+			" from items i,hosts h"
+			" where i.hostid=h.hostid"
+				" and h.status=%d"
+				" and i.type=%d"
+				" and i.flags<>%d"
+				" and h.hostid=" ZBX_FS_UI64
+				" and h.proxy_hostid is null",
 			HOST_STATUS_MONITORED,
 			ITEM_TYPE_ZABBIX_ACTIVE,
+			ZBX_FLAG_DISCOVERY_CHILD,
 			hostid);
 
-	if (0 != CONFIG_REFRESH_UNSUPPORTED)
-		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 256, " and (i.status=%d or (i.status=%d and i.lastclock+%d<=%d))",
-				ITEM_STATUS_ACTIVE, ITEM_STATUS_NOTSUPPORTED,
-				CONFIG_REFRESH_UNSUPPORTED, time(NULL));
-	else
-		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 256, " and i.status=%d",
-				ITEM_STATUS_ACTIVE);
-
 	zbx_free(name_esc);
 
 	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
@@ -320,73 +356,84 @@ int	send_list_of_active_checks_json(zbx_sock_t *sock, struct zbx_json_parse *jp)
 
 	while (NULL != (row = DBfetch(result)))
 	{
-		if (FAIL == DCconfig_get_item_by_key(&dc_item, (zbx_uint64_t)0, host, row[1]))
+		if (FAIL == DCconfig_get_item_by_key(&dc_item, (zbx_uint64_t)0, host, row[0]))
 		{
-			zabbix_log(LOG_LEVEL_DEBUG, "Item '%s' was not found in the server cache. Not sending now.", row[1]);
+			zabbix_log(LOG_LEVEL_DEBUG, "%s() Item '%s' was not found in the server cache. Not sending now.", __function_name, row[0]);
 			continue;
 		}
 
-		zabbix_log(LOG_LEVEL_DEBUG, "Item '%s' was successfully found in the server cache. Sending.", row[1]);
+		if (ITEM_STATUS_NOTSUPPORTED == dc_item.status)
+		{
+			if (0 == refresh_unsupported || dc_item.lastclock + refresh_unsupported > now)
+			{
+				DCconfig_clean_items(&dc_item, NULL, 1);
+				continue;
+			}
+		}
 
-		DBget_item_from_db(&item, row);
+		zabbix_log(LOG_LEVEL_DEBUG, "%s() Item '%s' was successfully found in the server cache. Sending.", __function_name, row[0]);
+
+		ZBX_STRDUP(key, row[0]);
+		substitute_key_macros(&key, NULL, &dc_item, NULL, MACRO_TYPE_ITEM_KEY, NULL, 0);
+
+		DCconfig_clean_items(&dc_item, NULL, 1);
 
 		zbx_json_addobject(&json, NULL);
-		zbx_json_addstring(&json, ZBX_PROTO_TAG_KEY, item.key, ZBX_JSON_TYPE_STRING);
-		if (0 != strcmp(item.key, item.key_orig))
-			zbx_json_addstring(&json, ZBX_PROTO_TAG_KEY_ORIG, item.key_orig, ZBX_JSON_TYPE_STRING);
-		zbx_snprintf(tmp, sizeof(tmp), "%d", item.delay);
-		zbx_json_addstring(&json, ZBX_PROTO_TAG_DELAY, tmp, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_KEY, key, ZBX_JSON_TYPE_STRING);
+		if (0 != strcmp(key, row[0]))
+			zbx_json_addstring(&json, ZBX_PROTO_TAG_KEY_ORIG, row[0], ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_DELAY, row[1], ZBX_JSON_TYPE_INT);
 		/* The agent expects ALWAYS to have lastlogsize and mtime tags. Removing those would cause older agents to fail. */
-		zbx_snprintf(tmp, sizeof(tmp), "%d", item.lastlogsize);
-		zbx_json_addstring(&json, ZBX_PROTO_TAG_LOGLASTSIZE, tmp, ZBX_JSON_TYPE_STRING);
-		zbx_snprintf(tmp, sizeof(tmp), "%d", item.mtime);
-		zbx_json_addstring(&json, ZBX_PROTO_TAG_MTIME, tmp, ZBX_JSON_TYPE_STRING);
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_LOGLASTSIZE, row[2], ZBX_JSON_TYPE_INT);
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_MTIME, row[3], ZBX_JSON_TYPE_INT);
 		zbx_json_close(&json);
 
-		/* Special processing for log[] and logrt[] items */
+		/* special processing for log[] and logrt[] items */
 		do {	/* simple try realization */
 
 			/* log[filename,pattern,encoding,maxlinespersec] */
 			/* logrt[filename_format,pattern,encoding,maxlinespersec] */
 
-			if (0 != strncmp(item.key, "log[", 4) && 0 != strncmp(item.key, "logrt[", 6))
+			if (0 != strncmp(key, "log[", 4) && 0 != strncmp(key, "logrt[", 6))
 				break;
 
-			if (2 != parse_command(item.key, NULL, 0, params, MAX_STRING_LEN))
+			if (2 != parse_command(key, NULL, 0, params, sizeof(params)))
 				break;
 
-			/*dealing with `pattern' parameter*/
+			/* dealing with `pattern' parameter */
 			if (0 == get_param(params, 2, pattern, sizeof(pattern)) &&
 				*pattern == '@')
 					add_regexp_name(&regexp, &regexp_alloc, &regexp_num, pattern + 1);
 		} while (0);	/* simple try realization */
 
-		/* Special processing for eventlog[] items */
+		/* special processing for eventlog[] items */
 		do {	/* simple try realization */
 
 			/* eventlog[filename,pattern,severity,source,logeventid,maxlinespersec] */
 
-			if (0 != strncmp(item.key, "eventlog[", 9))
+			if (0 != strncmp(key, "eventlog[", 9))
 				break;
 
-			if (2 != parse_command(item.key, NULL, 0, params, MAX_STRING_LEN))
+			if (2 != parse_command(key, NULL, 0, params, sizeof(params)))
 				break;
 
-			/*dealing with `pattern' parameter*/
+			/* dealing with `pattern' parameter */
 			if (0 == get_param(params, 2, pattern, sizeof(pattern)) &&
 				*pattern == '@')
 					add_regexp_name(&regexp, &regexp_alloc, &regexp_num, pattern + 1);
 
-			/*dealing with `severity' parameter*/
+			/* dealing with `severity' parameter */
 			if (0 == get_param(params, 3, key_severity, sizeof(key_severity)) &&
 				*key_severity == '@')
 					add_regexp_name(&regexp, &regexp_alloc, &regexp_num, key_severity + 1);
 
-			/*dealing with `logeventid' parameter*/
+			/* dealing with `logeventid' parameter */
 			if (0 == get_param(params, 5, key_logeventid, sizeof(key_logeventid)) &&
 				*key_logeventid == '@')
 					add_regexp_name(&regexp, &regexp_alloc, &regexp_num, key_logeventid + 1);
 		} while (0);	/* simple try realization */
+
+		zbx_free(key);
 	}
 	zbx_json_close(&json);
 
@@ -397,20 +444,24 @@ int	send_list_of_active_checks_json(zbx_sock_t *sock, struct zbx_json_parse *jp)
 		zbx_json_addarray(&json, ZBX_PROTO_TAG_REGEXP);
 
 		sql_offset = 0;
-		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 512,
+		zbx_strcpy_alloc(&sql, &sql_alloc, &sql_offset,
 				"select r.name,e.expression,e.expression_type,e.exp_delimiter,e.case_sensitive"
-				" from regexps r,expressions e where r.regexpid=e.regexpid and r.name in (");
+				" from regexps r,expressions e"
+				" where r.regexpid=e.regexpid"
+					" and r.name in (");
 
 		for (n = 0; n < regexp_num; n++)
 		{
 			name_esc = DBdyn_escape_string(regexp[n]);
-			zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 512, "%s'%s'",
+			zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, "%s'%s'",
 					n == 0 ? "" : ",",
 					name_esc);
 			zbx_free(name_esc);
 			zbx_free(regexp[n]);
 		}
-		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, 8, ")");
+		zbx_chrcpy_alloc(&sql, &sql_alloc, &sql_offset, ')');
+
+		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset, DB_NODE, DBnode_local("r.regexpid"));
 
 		result = DBselect("%s", sql);
 		while (NULL != (row = DBfetch(result)))
@@ -427,12 +478,11 @@ int	send_list_of_active_checks_json(zbx_sock_t *sock, struct zbx_json_parse *jp)
 	}
 	zbx_free(regexp);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "Sending [%s]",
-			json.buffer);
+	zabbix_log(LOG_LEVEL_DEBUG, "%s() sending [%s]", __function_name, json.buffer);
 
 	alarm(CONFIG_TIMEOUT);
 	if (SUCCEED != zbx_tcp_send(sock, json.buffer))
-		zbx_snprintf(error, MAX_STRING_LEN, "%s", zbx_tcp_strerror());
+		strscpy(error, zbx_tcp_strerror());
 	else
 		res = SUCCEED;
 	alarm(0);
@@ -440,21 +490,21 @@ int	send_list_of_active_checks_json(zbx_sock_t *sock, struct zbx_json_parse *jp)
 	zbx_json_free(&json);
 	zbx_free(sql);
 
-	return res;
+	goto out;
 error:
-	zabbix_log(LOG_LEVEL_WARNING, "Sending list of active checks to [%s] failed: %s",
-			get_ip_by_socket(sock), error);
+	zabbix_log(LOG_LEVEL_WARNING, "cannot send list of active checks to [%s]: %s", get_ip_by_socket(sock), error);
 
 	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
 	zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_FAILED, ZBX_JSON_TYPE_STRING);
 	zbx_json_addstring(&json, ZBX_PROTO_TAG_INFO, error, ZBX_JSON_TYPE_STRING);
 
-	zabbix_log(LOG_LEVEL_DEBUG, "Sending [%s]",
-			json.buffer);
+	zabbix_log(LOG_LEVEL_DEBUG, "%s() sending [%s]", __function_name, json.buffer);
 
 	res = zbx_tcp_send(sock, json.buffer);
 
 	zbx_json_free(&json);
+out:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(res));
 
 	return res;
 }
