@@ -1,6 +1,6 @@
 /*
-** ZABBIX
-** Copyright (C) 2000-2005 SIA Zabbix
+** Zabbix
+** Copyright (C) 2001-2013 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -9,12 +9,12 @@
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
-** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
 #include "common.h"
@@ -22,7 +22,6 @@
 #include "db.h"
 #include "zbxdb.h"
 #include "log.h"
-#include "zlog.h"
 #include "daemon.h"
 #include "zbxself.h"
 #include "zbxalgo.h"
@@ -100,16 +99,18 @@ static void	sync_config()
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	result = DBselect_once("select mt.mediatypeid,mt.type,mt.description,"
-				"mt.smtp_server,mt.smtp_helo,mt.smtp_email,"
-				"mt.exec_path,mt.gsm_modem,"
+	result = DBselect_once(
+			"select mt.mediatypeid,mt.type,mt.description,mt.smtp_server,"
+				"mt.smtp_helo,mt.smtp_email,mt.exec_path,mt.gsm_modem,"
 				"mt.username,mt.passwd,m.sendto"
-				" from media m,users_groups u,config c,media_type mt"
-				" where m.userid=u.userid"
-					" and u.usrgrpid=c.alert_usrgrpid"
-					" and m.mediatypeid=mt.mediatypeid"
-					" and m.active=%d",
-				MEDIA_STATUS_ACTIVE);
+			" from media m,users_groups u,config c,media_type mt"
+			" where m.userid=u.userid"
+				" and u.usrgrpid=c.alert_usrgrpid"
+				" and m.mediatypeid=mt.mediatypeid"
+				" and m.active=%d"
+				" and mt.status=%d",
+			MEDIA_STATUS_ACTIVE,
+			MEDIA_TYPE_STATUS_ACTIVE);
 
 	if (NULL == result || (DB_RESULT)ZBX_DB_DOWN == result)
 	{
@@ -145,7 +146,6 @@ static void	sync_config()
 		STR_REPLACE(recipient->mediatype.gsm_modem, row[7]);
 		STR_REPLACE(recipient->mediatype.username, row[8]);
 		STR_REPLACE(recipient->mediatype.passwd, row[9]);
-
 		STR_REPLACE(recipient->alert.sendto, row[10]);
 
 		if (NULL == recipient->alert.subject)
@@ -207,9 +207,6 @@ void	main_watchdog_loop()
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In main_watchdog_loop()");
 
-	/* disable writing to database in zabbix_syslog() */
-	CONFIG_ENABLE_LOG = 0;
-
 	zbx_vector_ptr_create(&recipients);
 
 	for (;;)
